@@ -16,7 +16,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     In addition, as a special exception, the copyright holders give
     permission to link the code of this program with any edition of
@@ -47,13 +47,14 @@
 // have 2 threads talking to gpgconf anyway? :)
 static bool s_duringClear = false;
 
-#define GPGCONF_FLAG_GROUP 1
-#define GPGCONF_FLAG_OPTIONAL 2
-#define GPGCONF_FLAG_LIST 4
-#define GPGCONF_FLAG_RUNTIME 8
-#define GPGCONF_FLAG_DEFAULT 16 // fixed default value available
-#define GPGCONF_FLAG_DEFAULT_DESC 32 // runtime default value available
-#define GPGCONF_FLAG_NOARG_DESC 64 // option with optional arg; special meaning if no arg set
+static const int GPGCONF_FLAG_GROUP = 1;
+static const int GPGCONF_FLAG_OPTIONAL = 2;
+static const int GPGCONF_FLAG_LIST = 4;
+static const int GPGCONF_FLAG_RUNTIME = 8;
+static const int GPGCONF_FLAG_DEFAULT = 16; // fixed default value available
+static const int GPGCONF_FLAG_DEFAULT_DESC = 32; // runtime default value available
+static const int GPGCONF_FLAG_NOARG_DESC = 64; // option with optional arg; special meaning if no arg set
+static const int GPGCONF_FLAG_NO_CHANGE = 128; // readonly
 // Change size of mFlags bitfield if adding new values here
 
 QGpgMECryptoConfig::QGpgMECryptoConfig()
@@ -183,8 +184,8 @@ void QGpgMECryptoConfigComponent::runGpgConf()
   else
     rc = ( proc.normalExit() ) ? proc.exitStatus() : -1 ;
 
-  if( rc != 0 ) // Can it really be non-0, when gpg-config --list-components worked?
-    kdWarning(5150) << k_funcinfo << ":" << strerror( rc ) << endl;
+  if( rc != 0 ) // can happen when using the wrong version of gpg...
+    kdWarning(5150) << "Running 'gpgconf --list-options " << mName << "' failed. " << strerror( rc ) << ", but try that command to see the real output" << endl;
   else {
     if ( mCurrentGroup && !mCurrentGroup->mEntries.isEmpty() ) // only add non-empty groups
       mGroups.insert( mCurrentGroupName, mCurrentGroup );
@@ -198,10 +199,10 @@ void QGpgMECryptoConfigComponent::slotCollectStdOut( KProcIO* proc )
   while( ( result = proc->readln(line) ) != -1 ) {
     //kdDebug(5150) << "GOT LINE:" << line << endl;
     // Format: NAME:FLAGS:LEVEL:DESCRIPTION:TYPE:ALT-TYPE:ARGNAME:DEFAULT:ARGDEF:VALUE
-    QStringList lst = QStringList::split( ':', line, true );
+    const QStringList lst = QStringList::split( ':', line, true );
     if ( lst.count() >= 10 ) {
-      int flags = lst[1].toInt();
-      int level = lst[2].toInt();
+      const int flags = lst[1].toInt();
+      const int level = lst[2].toInt();
       if ( level > 2 ) // invisible or internal -> skip it;
         continue;
       if ( flags & GPGCONF_FLAG_GROUP ) {
@@ -209,12 +210,12 @@ void QGpgMECryptoConfigComponent::slotCollectStdOut( KProcIO* proc )
           mGroups.insert( mCurrentGroupName, mCurrentGroup );
         //else
         //  kdDebug(5150) << "Discarding empty group " << mCurrentGroupName << endl;
-        mCurrentGroup = new QGpgMECryptoConfigGroup( lst[3], level );
+        mCurrentGroup = new QGpgMECryptoConfigGroup( lst[0], lst[3], level );
         mCurrentGroupName = lst[0];
       } else {
         // normal entry
         if ( !mCurrentGroup ) {  // first toplevel entry -> create toplevel group
-          mCurrentGroup = new QGpgMECryptoConfigGroup( QString::null, 0 );
+          mCurrentGroup = new QGpgMECryptoConfigGroup( "<nogroup>", QString::null, 0 );
           mCurrentGroupName = "<nogroup>";
         }
         mCurrentGroup->mEntries.insert( lst[0], new QGpgMECryptoConfigEntry( lst ) );
@@ -320,8 +321,9 @@ void QGpgMECryptoConfigComponent::sync( bool runtime )
 
 ////
 
-QGpgMECryptoConfigGroup::QGpgMECryptoConfigGroup( const QString& description, int level )
+QGpgMECryptoConfigGroup::QGpgMECryptoConfigGroup( const QString & name, const QString& description, int level )
   : mEntries( 29 ),
+    mName( name ),
     mDescription( description ),
     mLevel( static_cast<Kleo::CryptoConfigEntry::Level>( level ) )
 {
@@ -400,10 +402,10 @@ QGpgMECryptoConfigEntry::QGpgMECryptoConfigEntry( const QStringList& parsedLine
   // Format: NAME:FLAGS:LEVEL:DESCRIPTION:TYPE:ALT-TYPE:ARGNAME:DEFAULT:ARGDEF:VALUE
   assert( parsedLine.count() >= 10 ); // called checked for it already
   QStringList::const_iterator it = parsedLine.begin();
-  ++it; // skip name, stored in group
+  mName = *it++;
   mFlags = (*it++).toInt();
   mLevel = (*it++).toInt();
-  mDescription = (*it++);
+  mDescription = *it++;
   bool ok;
   // we keep the real (int) arg type, since it influences the parsing (e.g. for ldap urls)
   mRealArgType = (*it++).toInt();
@@ -491,6 +493,11 @@ bool QGpgMECryptoConfigEntry::isOptional() const
   return mFlags & GPGCONF_FLAG_OPTIONAL;
 }
 
+bool QGpgMECryptoConfigEntry::isReadOnly() const
+{
+  return mFlags & GPGCONF_FLAG_NO_CHANGE;
+}
+
 bool QGpgMECryptoConfigEntry::isList() const
 {
   return mFlags & GPGCONF_FLAG_LIST;
