@@ -14,13 +14,17 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
     without including the source code for Qt in the source distribution.
 */
 
+// Needed for ugly hack, to be removed in 4.0
+#include <unistd.h> // for usleep
+#include <qeventloop.h>
+
 #include <qclipboard.h>
 #include <qdir.h>
 #include <qfile.h>
@@ -83,13 +87,18 @@
 KABCore::KABCore( KXMLGUIClient *client, bool readWrite, QWidget *parent,
                   const QString &file, const char *name )
   : KAB::Core( client, parent, name ), mStatusBar( 0 ), mViewManager( 0 ),
-    mExtensionManager( 0 ), mCategorySelectDialog( 0 ), mCategoryEditDialog( 0 ),
-    mLdapSearchDialog( 0 ), mReadWrite( readWrite ), mModified( false )
+    mExtensionManager( 0 ), mJumpButtonBar( 0 ), mCategorySelectDialog( 0 ),
+    mCategoryEditDialog( 0 ), mLdapSearchDialog( 0 ), mReadWrite( readWrite ),
+    mModified( false )
 {
   mWidget = new QWidget( parent, name );
 
   mIsPart = !parent->isA( "KAddressBookMain" );
 
+  mAddressBookChangedTimer = new QTimer( this );
+  connect( mAddressBookChangedTimer, SIGNAL( timeout() ),
+           this, SLOT( addressBookChanged() ) );
+
   if ( file.isEmpty() ) {
     mAddressBook = KABC::StdAddressBook::self( true );
   } else {
@@ -110,7 +119,7 @@ KABCore::KABCore( KXMLGUIClient *client, bool readWrite, QWidget *parent,
                                 "X-AssistantsName", "KADDRESSBOOK" );
   mAddressBook->addCustomField( i18n( "Manager's Name" ), KABC::Field::Organization,
                                 "X-ManagersName", "KADDRESSBOOK" );
-  mAddressBook->addCustomField( i18n( "Spouse's Name" ), KABC::Field::Personal,
+  mAddressBook->addCustomField( i18n( "Partner's Name" ), KABC::Field::Personal,
                                 "X-SpousesName", "KADDRESSBOOK" );
   mAddressBook->addCustomField( i18n( "Office" ), KABC::Field::Personal,
                                 "X-Office", "KADDRESSBOOK" );
@@ -119,16 +128,19 @@ KABCore::KABCore( KXMLGUIClient *client, bool readWrite, QWidget *parent,
   mAddressBook->addCustomField( i18n( "Anniversary" ), KABC::Field::Personal,
                                 "X-Anniversary", "KADDRESSBOOK" );
   mAddressBook->addCustomField( i18n( "Blog" ), KABC::Field::Personal,
-                                "Blog", "KADDRESSBOOK" );
+                                "BlogFeed", "KADDRESSBOOK" );
 
   mSearchManager = new KAB::SearchManager( mAddressBook, parent );
 
+  connect( mSearchManager, SIGNAL( contactsUpdated() ),
+           this, SLOT( slotContactsUpdated() ) );
+
   initGUI();
 
   connect( mAddressBook, SIGNAL( addressBookChanged( AddressBook* ) ),
-           SLOT( addressBookChanged() ) );
+           SLOT( delayedAddressBookChanged() ) );
   connect( mAddressBook, SIGNAL( loadingFinished( Resource* ) ),
-           SLOT( addressBookChanged() ) );
+           SLOT( delayedAddressBookChanged() ) );
 
   mIncSearchWidget->setFocus();
 
@@ -144,15 +156,12 @@ KABCore::KABCore( KXMLGUIClient *client, bool readWrite, QWidget *parent,
            SLOT( updateIncSearchWidget() ) );
   connect( mExtensionManager, SIGNAL( modified( const KABC::Addressee::List& ) ),
            this, SLOT( extensionModified( const KABC::Addressee::List& ) ) );
+  connect( mExtensionManager, SIGNAL( deleted( const QStringList& ) ),
+           this, SLOT( extensionDeleted( const QStringList& ) ) );
 
   connect( mXXPortManager, SIGNAL( modified() ),
            SLOT( setModified() ) );
 
-  connect( mJumpButtonBar, SIGNAL( jumpToLetter( const QString& ) ),
-           SLOT( incrementalJumpButtonSearch( const QString& ) ) );
-  connect( mViewManager, SIGNAL( sortFieldChanged() ),
-           mJumpButtonBar, SLOT( updateButtons() ) );
-
   connect( mDetails, SIGNAL( highlightedMessage( const QString& ) ),
            SLOT( detailsHighlighted( const QString& ) ) );
 
@@ -163,6 +172,10 @@ KABCore::KABCore( KXMLGUIClient *client, bool readWrite, QWidget *parent,
 
   mAddressBookService = new KAddressBookService( this );
 
+  mCommandHistory = new KCommandHistory( actionCollection(), true );
+  connect( mCommandHistory, SIGNAL( commandExecuted() ),
+           mSearchManager, SLOT( reload() ) );
+
   mSearchManager->reload();
 
   setModified( false );
@@ -280,7 +293,7 @@ QWidget *KABCore::widget() const
 KAboutData *KABCore::createAboutData()
 {
   KAboutData *about = new KAboutData( "kaddressbook", I18N_NOOP( "KAddressBook" ),
-                                      "3.4", I18N_NOOP( "The KDE Address Book" ),
+                                      "3.5.7", I18N_NOOP( "The KDE Address Book" ),
                                       KAboutData::License_GPL_V2,
                                       I18N_NOOP( "(c) 1997-2005, The KDE PIM Team" ) );
   about->addAuthor( "Tobias Koenig", I18N_NOOP( "Current maintainer" ), "tokoe@kde.org" );
@@ -323,7 +336,10 @@ void KABCore::setContactSelected( const QString &uid )
 
   if ( mReadWrite ) {
     mActionCut->setEnabled( selected );
-    mActionPaste->setEnabled( selected );
+
+    QClipboard *cb = QApplication::clipboard();
+    KABC::Addressee::List list = AddresseeUtil::clipboardToAddressees( cb->text() );
+    mActionPaste->setEnabled( !list.isEmpty() );
   }
 
   mActionCopy->setEnabled( selected );
@@ -389,19 +405,20 @@ void KABCore::deleteContacts( const QStringList &uids )
   if ( uids.count() > 0 ) {
     QStringList names;
     QStringList::ConstIterator it = uids.begin();
-    while ( it != uids.end() ) {
+    const QStringList::ConstIterator endIt( uids.end() );
+    while ( it != endIt ) {
       KABC::Addressee addr = mAddressBook->findByUid( *it );
       names.append( addr.realName().isEmpty() ? addr.preferredEmail() : addr.realName() );
       ++it;
     }
 
-	    if ( KMessageBox::warningContinueCancelList( mWidget, i18n( "Do you really want to delete this contact?", "Do you really want to delete these %n contacts?", uids.count() ),
-		 names, "", KStdGuiItem::del() ) == KMessageBox::Cancel )
+    if ( KMessageBox::warningContinueCancelList( mWidget, i18n( "Do you really want to delete this contact?",
+                                                 "Do you really want to delete these %n contacts?", uids.count() ),
+                                                 names, QString::null, KStdGuiItem::del() ) == KMessageBox::Cancel )
       return;
 
-    PwDeleteCommand *command = new PwDeleteCommand( mAddressBook, uids );
-    UndoStack::instance()->push( command );
-    RedoStack::instance()->clear();
+    DeleteCommand *command = new DeleteCommand( mAddressBook, uids );
+    mCommandHistory->addCommand( command );
 
     // now if we deleted anything, refresh
     setContactSelected( QString::null );
@@ -426,9 +443,8 @@ void KABCore::cutContacts()
   QStringList uidList = mViewManager->selectedUids();
 
   if ( uidList.size() > 0 ) {
-    PwCutCommand *command = new PwCutCommand( mAddressBook, uidList );
-    UndoStack::instance()->push( command );
-    RedoStack::instance()->clear();
+    CutCommand *command = new CutCommand( mAddressBook, uidList );
+    mCommandHistory->addCommand( command );
 
     setModified( true );
   }
@@ -446,13 +462,16 @@ void KABCore::pasteContacts()
 void KABCore::pasteContacts( KABC::Addressee::List &list )
 {
   KABC::Resource *resource = requestResource( mWidget );
+  if ( !resource )
+    return;
+
   KABC::Addressee::List::Iterator it;
-  for ( it = list.begin(); it != list.end(); ++it )
+  const KABC::Addressee::List::Iterator endIt( list.end() );
+  for ( it = list.begin(); it != endIt; ++it )
     (*it).setResource( resource );
 
-  PwPasteCommand *command = new PwPasteCommand( this, list );
-  UndoStack::instance()->push( command );
-  RedoStack::instance()->clear();
+  PasteCommand *command = new PasteCommand( this, list );
+  mCommandHistory->addCommand( command );
 
   setModified( true );
 }
@@ -466,13 +485,20 @@ void KABCore::mergeContacts()
   KABC::Addressee addr = KABTools::mergeContacts( list );
 
   KABC::Addressee::List::Iterator it = list.begin();
+  const KABC::Addressee::List::Iterator endIt( list.end() );
+  KABC::Addressee origAddr = *it;
+  QStringList uids;
   ++it;
-  while ( it != list.end() ) {
-    mAddressBook->removeAddressee( *it );
+  while ( it != endIt ) {
+    uids.append( (*it).uid() );
     ++it;
   }
 
-  mAddressBook->insertAddressee( addr );
+  DeleteCommand *command = new DeleteCommand( mAddressBook, uids );
+  mCommandHistory->addCommand( command );
+
+  EditCommand *editCommand = new EditCommand( mAddressBook, origAddr, addr );
+  mCommandHistory->addCommand( editCommand );
 
   mSearchManager->reload();
 }
@@ -487,7 +513,7 @@ void KABCore::setWhoAmI()
   }
 
   QString text( i18n( "<qt>Do you really want to use <b>%1</b> as your new personal contact?</qt>" ) );
-  if ( KMessageBox::questionYesNo( mWidget, text.arg( addrList[ 0 ].assembledName() ) ) == KMessageBox::Yes )
+  if ( KMessageBox::questionYesNo( mWidget, text.arg( addrList[ 0 ].assembledName() ), QString::null, i18n("Use"), i18n("Do Not Use") ) == KMessageBox::Yes )
     static_cast<KABC::StdAddressBook*>( KABC::StdAddressBook::self( true ) )->setWhoAmI( addrList[ 0 ] );
 }
 
@@ -506,7 +532,8 @@ void KABCore::incrementalJumpButtonSearch( const QString& character )
   if ( field ) {
     list.sortByField( field );
     KABC::AddresseeList::ConstIterator it;
-    for ( it = list.begin(); it != list.end(); ++it ) {
+    const KABC::AddresseeList::ConstIterator endIt( list.end() );
+    for ( it = list.begin(); it != endIt; ++it ) {
       if ( field->value( *it ).startsWith( character, false ) ) {
         mViewManager->setSelected( (*it).uid(), true );
         return;
@@ -540,13 +567,14 @@ void KABCore::contactModified( const KABC::Addressee &addr )
   // check if it exists already
   KABC::Addressee origAddr = mAddressBook->findByUid( addr.uid() );
   if ( origAddr.isEmpty() ) {
-    command = new PwNewCommand( mAddressBook, addr );
+    KABC::Addressee::List addressees;
+    addressees.append( addr );
+    command = new NewCommand( mAddressBook, addressees );
   } else {
-    command = new PwEditCommand( mAddressBook, origAddr, addr );
+    command = new EditCommand( mAddressBook, origAddr, addr );
   }
 
-  UndoStack::instance()->push( command );
-  RedoStack::instance()->clear();
+  mCommandHistory->addCommand( command );
 
   setContactSelected( addr.uid() );
   setModified( true );
@@ -581,11 +609,22 @@ void KABCore::addEmail( const QString &aStr )
 
   KABC::Addressee::parseEmailAddress( aStr, fullName, email );
 
+#if KDE_IS_VERSION(3,4,89)
+  // This ugly hack will be removed in 4.0
+  // addressbook may not be reloaded yet, as done asynchronously sometimes, so wait
+  while ( !mAddressBook->loadingHasFinished() ) {
+    QApplication::eventLoop()->processEvents( QEventLoop::ExcludeUserInput );
+    // use sleep here to reduce cpu usage
+    usleep( 100 );
+  }
+#endif
+
   // Try to lookup the addressee matching the email address
   bool found = false;
   QStringList emailList;
   KABC::AddressBook::Iterator it;
-  for ( it = mAddressBook->begin(); !found && (it != mAddressBook->end()); ++it ) {
+  const KABC::AddressBook::Iterator endIt( mAddressBook->end() );
+  for ( it = mAddressBook->begin(); !found && (it != endIt); ++it ) {
     emailList = (*it).emails();
     if ( emailList.contains( email ) > 0 ) {
       found = true;
@@ -627,6 +666,16 @@ void KABCore::editContact( const QString &uid )
     if ( uidList.count() > 0 )
       localUID = *( uidList.at( 0 ) );
   }
+#if KDE_IS_VERSION(3,4,89)
+  // This ugly hack will be removed in 4.0
+  // for calls with given uid, as done from commandline and DCOP
+  // addressbook may not be reloaded yet, as done asynchronously, so wait
+  else while ( !mAddressBook->loadingHasFinished() ) {
+    QApplication::eventLoop()->processEvents( QEventLoop::ExcludeUserInput );
+    // use sleep here to reduce cpu usage
+    usleep( 100 );
+  }
+#endif
 
   KABC::Addressee addr = mAddressBook->findByUid( localUID );
   if ( !addr.isEmpty() ) {
@@ -660,10 +709,13 @@ void KABCore::storeContactIn( const QString &uid )
     uidList << uid;
   }
   KABC::Resource *resource = requestResource( mWidget );
-  if ( !resource ) return;
+  if ( !resource )
+    return;
+
   KABLock::self( mAddressBook )->lock( resource );
   QStringList::Iterator it( uidList.begin() );
-  while ( it != uidList.end() ) {
+  const QStringList::Iterator endIt( uidList.end() );
+  while ( it != endIt ) {
     KABC::Addressee addr = mAddressBook->findByUid( *it++ );
     if ( !addr.isEmpty() ) {
       KABC::Addressee newAddr( addr );
@@ -704,28 +756,15 @@ void KABCore::save()
   }
 }
 
-void KABCore::undo()
-{
-  UndoStack::instance()->undo();
-
-  // Refresh the view
-  mViewManager->refreshView();
-}
-
-void KABCore::redo()
-{
-  RedoStack::instance()->redo();
-
-  // Refresh the view
-  mViewManager->refreshView();
-}
-
 void KABCore::setJumpButtonBarVisible( bool visible )
 {
-  if ( visible )
+  if ( visible ) {
+    if ( !mJumpButtonBar )
+      createJumpButtonBar();
     mJumpButtonBar->show();
-  else
-    mJumpButtonBar->hide();
+  } else
+    if ( mJumpButtonBar )
+      mJumpButtonBar->hide();
 }
 
 void KABCore::setDetailsVisible( bool visible )
@@ -740,43 +779,65 @@ void KABCore::extensionModified( const KABC::Addressee::List &list )
 {
   if ( list.count() != 0 ) {
     KABC::Addressee::List::ConstIterator it;
-    for ( it = list.begin(); it != list.end(); ++it ) {
+    const KABC::Addressee::List::ConstIterator endIt( list.end() );
+    for ( it = list.begin(); it != endIt; ++it ) {
       Command *command = 0;
 
       // check if it exists already
       KABC::Addressee origAddr = mAddressBook->findByUid( (*it).uid() );
-      if ( origAddr.isEmpty() )
-        command = new PwNewCommand( mAddressBook, *it );
-      else
-        command = new PwEditCommand( mAddressBook, origAddr, *it );
+      if ( origAddr.isEmpty() ) {
+        KABC::Addressee::List addressees;
+        addressees.append( *it );
+        command = new NewCommand( mAddressBook, addressees );
+      } else
+        command = new EditCommand( mAddressBook, origAddr, *it );
 
-      UndoStack::instance()->push( command );
-      RedoStack::instance()->clear();
+      mCommandHistory->blockSignals( true );
+      mCommandHistory->addCommand( command );
+      mCommandHistory->blockSignals( false );
     }
 
-    setModified( true );
+    setModified(true);
   }
+}
 
-  if ( list.count() == 0 )
-    mViewManager->refreshView();
-  else
-    mViewManager->refreshView( list[ 0 ].uid() );
+void KABCore::extensionDeleted( const QStringList &uidList )
+{
+  DeleteCommand *command = new DeleteCommand( mAddressBook, uidList );
+  mCommandHistory->addCommand( command );
+
+  // now if we deleted anything, refresh
+  setContactSelected( QString::null );
+  setModified( true );
 }
 
 QString KABCore::getNameByPhone( const QString &phone )
 {
+#if KDE_IS_VERSION(3,4,89)
+  // This ugly hack will be removed in 4.0
+  // addressbook may not be reloaded yet, as done asynchronously, so wait
+  while ( !mAddressBook->loadingHasFinished() ) {
+    QApplication::eventLoop()->processEvents( QEventLoop::ExcludeUserInput );
+    // use sleep here to reduce cpu usage
+    usleep( 100 );
+  }
+#endif
+
   QRegExp r( "[/*/-/ ]" );
   QString localPhone( phone );
 
   bool found = false;
   QString ownerName = "";
-  KABC::AddressBook::ConstIterator iter;
-  KABC::PhoneNumber::List::Iterator phoneIter;
   KABC::PhoneNumber::List phoneList;
-  for ( iter = mAddressBook->begin(); !found && ( iter != mAddressBook->end() ); ++iter ) {
+
+  KABC::AddressBook::ConstIterator iter;
+  const KABC::AddressBook::ConstIterator endIter( mAddressBook->end() );
+
+  for ( iter = mAddressBook->begin(); !found && ( iter != endIter ); ++iter ) {
     phoneList = (*iter).phoneNumbers();
-    for ( phoneIter = phoneList.begin(); !found && ( phoneIter != phoneList.end() );
-          ++phoneIter) {
+    KABC::PhoneNumber::List::Iterator phoneIter( phoneList.begin() );
+    const KABC::PhoneNumber::List::Iterator phoneEndIter( phoneList.end() );
+    for ( ; !found && ( phoneIter != phoneEndIter ); ++phoneIter) {
       // Get rid of separator chars so just the numbers are compared.
       if ( (*phoneIter).number().replace( r, "" ) == localPhone.replace( r, "" ) ) {
         ownerName = (*iter).realName();
@@ -799,7 +860,7 @@ void KABCore::openLDAPDialog()
 
   if ( !mLdapSearchDialog ) {
     mLdapSearchDialog = new LDAPSearchDialog( mAddressBook, this, mWidget );
-    connect( mLdapSearchDialog, SIGNAL( addresseesAdded() ), mSearchManager,
+    connect( mLdapSearchDialog, SIGNAL( addresseesAdded() ),
             SLOT( addressBookChanged() ) );
     connect( mLdapSearchDialog, SIGNAL( addresseesAdded() ),
             SLOT( setModified() ) );
@@ -843,8 +904,12 @@ void KABCore::print()
 
 void KABCore::detailsHighlighted( const QString &msg )
 {
-  if ( mStatusBar )
-    mStatusBar->changeItem( msg, 1 );
+  if ( mStatusBar ) {
+    if ( !mStatusBar->hasItem( 2 ) )
+      mStatusBar->insertItem( msg, 2 );
+    else
+      mStatusBar->changeItem( msg, 2 );
+  }
 }
 
 void KABCore::showContactsAddress( const QString &addrUid )
@@ -859,7 +924,8 @@ void KABCore::showContactsAddress( const QString &addrUid )
 
   const KABC::Address::List list = addr.addresses();
   KABC::Address::List::ConstIterator it;
-  for ( it = list.begin(); it != list.end(); ++it )
+  const KABC::Address::List::ConstIterator endIt( list.end() );
+  for ( it = list.begin(); it != endIt; ++it )
     if ( (*it).id() == addrUid ) {
       LocationMap::instance()->showAddress( *it );
       break;
@@ -887,12 +953,35 @@ bool KABCore::queryClose()
   return true;
 }
 
+void KABCore::reinitXMLGUI()
+{
+  mExtensionManager->createActions();
+}
+void KABCore::delayedAddressBookChanged()
+{
+  mAddressBookChangedTimer->start( 1000 );
+}
+
 void KABCore::addressBookChanged()
 {
-  mJumpButtonBar->updateButtons();
+  const QStringList selectedUids = mViewManager->selectedUids();
+
+  mAddressBookChangedTimer->stop();
+
+  if ( mJumpButtonBar )
+    mJumpButtonBar->updateButtons();
+
   mSearchManager->reload();
+
   mViewManager->setSelected( QString::null, false );
-  setContactSelected( QString::null );
+
+  QString uid = QString::null;
+  if ( !selectedUids.isEmpty() ) {
+    uid = selectedUids.first();
+    mViewManager->setSelected( uid, true );
+  }
+
+  setContactSelected( uid );
 
   updateCategories();
 }
@@ -942,14 +1031,15 @@ void KABCore::initGUI()
   mExtensionBarSplitter = new QSplitter( mDetailsSplitter );
   mExtensionBarSplitter->setOrientation( Qt::Vertical );
 
-  QWidget *detailsWidget = new QWidget( mDetailsSplitter );
-  QHBoxLayout *detailsLayout = new QHBoxLayout( detailsWidget );
+  mDetailsWidget = new QWidget( mDetailsSplitter );
+  mDetailsLayout = new QHBoxLayout( mDetailsWidget );
 
-  mDetailsPage = new QWidget( detailsWidget );
-  detailsLayout->addWidget( mDetailsPage );
+  mDetailsPage = new QWidget( mDetailsWidget );
+  mDetailsLayout->addWidget( mDetailsPage );
 
   QHBoxLayout *detailsPageLayout = new QHBoxLayout( mDetailsPage, 0, 0 );
   mDetails = new KPIM::AddresseeView( mDetailsPage );
+  mDetails->setVScrollBarMode( QScrollView::Auto );
   detailsPageLayout->addWidget( mDetails );
 
   connect( mDetails, SIGNAL( addressClicked( const QString&) ),
@@ -963,10 +1053,6 @@ void KABCore::initGUI()
 
   mExtensionManager = new ExtensionManager( this, mExtensionBarSplitter );
 
-  mJumpButtonBar = new JumpButtonBar( this, detailsWidget );
-  detailsLayout->addWidget( mJumpButtonBar );
-  detailsLayout->setStretchFactor( mJumpButtonBar, 1 );
-
   topLayout->setStretchFactor( mDetailsSplitter, 1 );
 
   mXXPortManager = new XXPortManager( this, mWidget );
@@ -974,6 +1060,18 @@ void KABCore::initGUI()
   initActions();
 }
 
+void KABCore::createJumpButtonBar()
+{
+  mJumpButtonBar = new JumpButtonBar( this, mDetailsWidget );
+  mDetailsLayout->addWidget( mJumpButtonBar );
+  mDetailsLayout->setStretchFactor( mJumpButtonBar, 1 );
+
+  connect( mJumpButtonBar, SIGNAL( jumpToLetter( const QString& ) ),
+           SLOT( incrementalJumpButtonSearch( const QString& ) ) );
+  connect( mViewManager, SIGNAL( sortFieldChanged() ),
+           mJumpButtonBar, SLOT( updateButtons() ) );
+}
+
 void KABCore::initActions()
 {
   connect( QApplication::clipboard(), SIGNAL( dataChanged() ),
@@ -1020,14 +1118,12 @@ void KABCore::initActions()
   mActionCut = KStdAction::cut( this, SLOT( cutContacts() ), actionCollection() );
   mActionPaste = KStdAction::paste( this, SLOT( pasteContacts() ), actionCollection() );
   action = KStdAction::selectAll( this, SLOT( selectAllContacts() ), actionCollection() );
-  mActionUndo = KStdAction::undo( this, SLOT( undo() ), actionCollection() );
-  mActionRedo = KStdAction::redo( this, SLOT( redo() ), actionCollection() );
   mActionCopy->setWhatsThis( i18n( "Copy the currently selected contact(s) to system clipboard in vCard format." ) );
   mActionCut->setWhatsThis( i18n( "Cuts the currently selected contact(s) to system clipboard in vCard format." ) );
   mActionPaste->setWhatsThis( i18n( "Paste the previously cut or copied contacts from clipboard." ) );
   action->setWhatsThis( i18n( "Selects all visible contacts from current view." ) );
-  mActionUndo->setWhatsThis( i18n( "Undoes the last <b>Cut</b>, <b>Copy</b> or <b>Paste</b>." ) );
-  mActionRedo->setWhatsThis( i18n( "Redoes the last <b>Cut</b>, <b>Copy</b> or <b>Paste</b>." ) );
+//  mActionUndo->setWhatsThis( i18n( "Undoes the last <b>Cut</b>, <b>Copy</b> or <b>Paste</b>." ) );
+//  mActionRedo->setWhatsThis( i18n( "Redoes the last <b>Cut</b>, <b>Copy</b> or <b>Paste</b>." ) );
 
   mActionDelete = new KAction( i18n( "&Delete Contact" ), "editdelete",
                                Key_Delete, this, SLOT( deleteContacts() ),
@@ -1040,10 +1136,6 @@ void KABCore::initActions()
                                       actionCollection(), "edit_store_in" );
   mActionStoreAddresseeIn->setWhatsThis( i18n( "Store a contact in a different Addressbook<p>You will be presented with a dialog where you can select a new storage place for this contact." ) );
 
-
-  mActionUndo->setEnabled( false );
-  mActionRedo->setEnabled( false );
-
   // settings menu
   mActionJumpBar = new KToggleAction( i18n( "Show Jump Bar" ), "next", 0,
                                       actionCollection(), "options_show_jump_bar" );
@@ -1058,7 +1150,7 @@ void KABCore::initActions()
   connect( mActionDetails, SIGNAL( toggled( bool ) ), SLOT( setDetailsVisible( bool ) ) );
 
   if ( mIsPart )
-    action = new KAction( i18n( "&Configure KAddressBook..." ), "configure", 0,
+    action = new KAction( i18n( "&Configure Address Book..." ), "configure", 0,
                           this, SLOT( configure() ), actionCollection(),
                           "kaddressbook_configure" );
   else
@@ -1088,9 +1180,6 @@ void KABCore::initActions()
 				     "Clears the content of the quick search bar." ) );
 
   clipboardDataChanged();
-
-  connect( UndoStack::instance(), SIGNAL( changed() ), SLOT( updateActionMenu() ) );
-  connect( RedoStack::instance(), SIGNAL( changed() ), SLOT( updateActionMenu() ) );
 }
 
 void KABCore::clipboardDataChanged()
@@ -1099,26 +1188,6 @@ void KABCore::clipboardDataChanged()
     mActionPaste->setEnabled( !QApplication::clipboard()->text().isEmpty() );
 }
 
-void KABCore::updateActionMenu()
-{
-  UndoStack *undo = UndoStack::instance();
-  RedoStack *redo = RedoStack::instance();
-
-  if ( undo->isEmpty() )
-    mActionUndo->setText( i18n( "Undo" ) );
-  else
-    mActionUndo->setText( i18n( "Undo %1" ).arg( undo->top()->name() ) );
-
-  mActionUndo->setEnabled( !undo->isEmpty() );
-
-  if ( !redo->top() )
-    mActionRedo->setText( i18n( "Redo" ) );
-  else
-    mActionRedo->setText( i18n( "Redo %1" ).arg( redo->top()->name() ) );
-
-  mActionRedo->setEnabled( !redo->isEmpty() );
-}
-
 void KABCore::updateIncSearchWidget()
 {
   mIncSearchWidget->setViewFields( mViewManager->viewFields() );
@@ -1131,7 +1200,8 @@ void KABCore::updateCategories()
 
   const QStringList customCategories( KABPrefs::instance()->customCategories() );
   QStringList::ConstIterator it;
-  for ( it = customCategories.begin(); it != customCategories.end(); ++it ) {
+  const QStringList::ConstIterator endIt( customCategories.end() );
+  for ( it = customCategories.begin(); it != endIt; ++it ) {
     if ( categories.find( *it ) == categories.end() ) {
       categories.append( *it );
     }
@@ -1150,11 +1220,14 @@ QStringList KABCore::allCategories() const
   QStringList::ConstIterator catIt;
 
   KABC::AddressBook::ConstIterator it;
-  for ( it = mAddressBook->begin(); it != mAddressBook->end(); ++it ) {
+  const KABC::AddressBook::ConstIterator endIt( mAddressBook->end() );
+  for ( it = mAddressBook->begin(); it != endIt; ++it ) {
     categories = (*it).categories();
-    for ( catIt = categories.begin(); catIt != categories.end(); ++catIt )
+    const QStringList::ConstIterator catEndIt( categories.end() );
+    for ( catIt = categories.begin(); catIt != catEndIt; ++catIt ) {
       if ( !allCategories.contains( *catIt ) )
         allCategories.append( *catIt );
+	}
   }
 
   return allCategories;
@@ -1178,12 +1251,13 @@ void KABCore::categoriesSelected( const QStringList &categories )
 {
   bool merge = false;
   QString msg = i18n( "Merge with existing categories?" );
-  if ( KMessageBox::questionYesNo( mWidget, msg ) == KMessageBox::Yes )
+  if ( KMessageBox::questionYesNo( mWidget, msg, QString::null, i18n( "Merge" ), i18n( "Do Not Merge" ) ) == KMessageBox::Yes )
     merge = true;
 
   QStringList uids = mViewManager->selectedUids();
   QStringList::ConstIterator it;
-  for ( it = uids.begin(); it != uids.end(); ++it ) {
+  const QStringList::ConstIterator endIt( uids.end() );
+  for ( it = uids.begin(); it != endIt; ++it ) {
     KABC::Addressee addr = mAddressBook->findByUid( *it );
     if ( !addr.isEmpty() ) {
       if ( !merge )
@@ -1191,7 +1265,8 @@ void KABCore::categoriesSelected( const QStringList &categories )
       else {
         QStringList addrCategories = addr.categories();
         QStringList::ConstIterator catIt;
-        for ( catIt = categories.begin(); catIt != categories.end(); ++catIt ) {
+        const QStringList::ConstIterator catEndIt( categories.end() );
+        for ( catIt = categories.begin(); catIt != catEndIt; ++catIt ) {
           if ( !addrCategories.contains( *catIt ) )
             addrCategories.append( *catIt );
         }
@@ -1224,6 +1299,19 @@ void KABCore::slotClearSearchBar()
   mIncSearchWidget->setFocus();
 }
 
+void KABCore::slotContactsUpdated()
+{
+  if ( mStatusBar ) {
+    QString msg( i18n( "%n contact matches", "%n contacts matching", mSearchManager->contacts().count() ) );
+    if ( !mStatusBar->hasItem( 1 ) )
+      mStatusBar->insertItem( msg, 1 );
+    else
+      mStatusBar->changeItem( msg, 1 );
+  }
+
+  emit contactsUpdated();
+}
+
 bool KABCore::handleCommandLine( KAddressBookIface* iface )
 {
   KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
@@ -1262,4 +1350,16 @@ bool KABCore::handleCommandLine( KAddressBookIface* iface )
   return doneSomething;
 }
 
+#ifdef KDEPIM_NEW_DISTRLISTS
+KPIM::DistributionList::List KABCore::distributionLists() const
+{
+  return mSearchManager->distributionLists();
+}
+
+QStringList KABCore::distributionListNames() const
+{
+  return mSearchManager->distributionListNames();
+}
+#endif
+
 #include "kabcore.moc"
