@@ -1,7 +1,7 @@
 /*
  *  alarmevent.h  -  represents calendar alarms and events
  *  Program:  kalarm
- *  (C) 2001 - 2004 by David Jarvie <software@astrojar.org.uk>
+ *  Copyright © 2001-2007 by David Jarvie <software@astrojar.org.uk>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,24 +13,31 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef KALARMEVENT_H
 #define KALARMEVENT_H
 
+/**  @file alarmevent.h - represents calendar alarms and events */
+
 #include <qcolor.h>
 #include <qfont.h>
 
 #include <libkcal/person.h>
 #include <libkcal/event.h>
-#include <libkcal/recurrence.h>
+namespace KCal {
+	class Calendar;
+	class Recurrence;
+}
 
 #include "datetime.h"
+#include "karecurrence.h"
 
 class AlarmCalendar;
+class KARecurrence;
 struct AlarmData;
 
 
@@ -68,13 +75,13 @@ class KAAlarmEventBase
 		int                lateCancel() const          { return mLateCancel; }
 		bool               autoClose() const           { return mAutoClose; }
 		bool               commandScript() const       { return mCommandScript; }
-		bool               commandXterm() const        { return mCommandXterm; }
 		bool               confirmAck() const          { return mConfirmAck; }
 		bool               repeatAtLogin() const       { return mRepeatAtLogin; }
 		int                repeatCount() const         { return mRepeatCount; }
 		int                repeatInterval() const      { return mRepeatInterval; }
 		bool               displaying() const          { return mDisplaying; }
 		bool               beep() const                { return mBeep; }
+		bool               speak() const               { return (mActionType == T_MESSAGE) && mSpeak; }
 		int                flags() const;
 #ifdef NDEBUG
 		void               dumpDebug() const  { }
@@ -103,14 +110,17 @@ class KAAlarmEventBase
 		QString            mEmailSubject;     // SUMMARY: subject line of email
 		QStringList        mEmailAttachments; // ATTACH: email attachment file names
 		float              mSoundVolume;      // volume for sound file, or < 0 for unspecified
+		float              mFadeVolume;       // initial volume for sound file, or < 0 for no fade
+		int                mFadeSeconds;      // fade time for sound file, or 0 if none
 		Type               mActionType;       // alarm action type
 		int                mRepeatCount;      // simple repetition count (excluding the first time)
 		int                mRepeatInterval;   // simple repetition interval (minutes)
+		int                mNextRepeat;       // repetition count of next due repetition
 		int                mLateCancel;       // how many minutes late will cancel the alarm, or 0 for no cancellation
 		bool               mAutoClose;        // whether to close the alarm window after the late-cancel period
 		bool               mCommandScript;    // the command text is a script, not a shell command line
-		bool               mCommandXterm;     // command alarm is to be executed in a terminal window
 		bool               mBeep;             // whether to beep when the alarm is displayed
+		bool               mSpeak;            // whether to speak the message when the alarm is displayed
 		bool               mRepeatSound;      // whether to repeat the sound file while the alarm is displayed
 		bool               mRepeatAtLogin;    // whether to repeat the alarm at every login
 		bool               mDisplaying;       // whether the alarm is currently being displayed
@@ -185,11 +195,15 @@ class KAAlarm : public KAAlarmEventBase
 		Type               type() const                 { return static_cast<Type>(mType & ~TIMED_DEFERRAL_FLAG); }
 		SubType            subType() const              { return mType; }
 		const QString&     eventID() const              { return mEventID; }
-		const DateTime&    dateTime() const             { return mNextMainDateTime; }
+		DateTime           dateTime(bool withRepeats = false) const
+		                                                { return (withRepeats && mNextRepeat && mRepeatInterval)
+		                                                    ? mNextMainDateTime.addSecs(mNextRepeat * mRepeatInterval * 60) : mNextMainDateTime; }
 		QDate              date() const                 { return mNextMainDateTime.date(); }
 		QTime              time() const                 { return mNextMainDateTime.time(); }
 		QString            audioFile() const            { return (mActionType == T_AUDIO) && !mBeep ? mText : QString::null; }
-		float              soundVolume() const          { return (mActionType == T_AUDIO) && mSoundVolume >= 0 && !mBeep && !mText.isEmpty() ? mSoundVolume : -1; }
+		float              soundVolume() const          { return (mActionType == T_AUDIO) && !mBeep && !mText.isEmpty() ? mSoundVolume : -1; }
+		float              fadeVolume() const           { return (mActionType == T_AUDIO) && mSoundVolume >= 0 && mFadeSeconds && !mBeep && !mText.isEmpty() ? mFadeVolume : -1; }
+		int                fadeSeconds() const          { return (mActionType == T_AUDIO) && mSoundVolume >= 0 && mFadeVolume >= 0 && !mBeep && !mText.isEmpty() ? mFadeSeconds : 0; }
 		bool               repeatSound() const          { return (mActionType == T_AUDIO) && mRepeatSound && !mBeep && !mText.isEmpty(); }
 		bool               reminder() const             { return mType == REMINDER__ALARM; }
 		bool               deferred() const             { return mDeferred; }
@@ -205,8 +219,6 @@ class KAAlarm : public KAAlarmEventBase
 #endif
 
 	private:
-		void               set(int flags);
-
 		SubType            mType;             // alarm type
 		bool               mRecurs;           // there is a recurrence rule for the alarm
 		bool               mDeferred;         // whether the alarm is an extra deferred/deferred-reminder alarm
@@ -215,7 +227,7 @@ class KAAlarm : public KAAlarmEventBase
 };
 
 
-// KAEvent corresponds to a KCal::Event instance
+/** KAEvent corresponds to a KCal::Event instance */
 class KAEvent : public KAAlarmEventBase
 {
 	public:
@@ -238,43 +250,29 @@ class KAEvent : public KAAlarmEventBase
 			AUTO_CLOSE      = 0x200,   // auto-close alarm window after late-cancel period
 			SCRIPT          = 0x400,   // command is a script, not a shell command line
 			EXEC_IN_XTERM   = 0x800,   // execute command in terminal window
+			SPEAK           = 0x1000,  // speak the message when the alarm is displayed
+		        COPY_KORGANIZER = 0x2000,  // KOrganizer should hold a copy of the event
 #ifdef OLD_DCOP
 			// The following are read-only internal values, and may be changed
 #else
 			// The following are read-only internal values
 #endif
-			REMINDER        = 0x1000,
-			DEFERRAL        = 0x2000,
-			TIMED_FLAG      = 0x4000,
+			REMINDER        = 0x10000,
+			DEFERRAL        = 0x20000,
+			TIMED_FLAG      = 0x40000,
 			DATE_DEFERRAL   = DEFERRAL,
 			TIME_DEFERRAL   = DEFERRAL | TIMED_FLAG,
-			DISPLAYING_     = 0x8000,
-			READ_ONLY_FLAGS = 0xF000   // mask for all read-only internal values
-		};
-		enum RecurType
-		{
-#ifdef OLD_DCOP
-			// *** DON'T CHANGE THESE VALUES ***
-			// because they are part of KAlarm's external DCOP interface.
-			// (But it's alright to add new values.)
-#endif
-			NO_RECUR    = 0,
-			MINUTELY    = 1,
-			DAILY       = 3,
-			WEEKLY      = 4,
-			MONTHLY_POS = 5,
-			MONTHLY_DAY = 6,
-			ANNUAL_DATE = 7,
-			ANNUAL_POS  = 9,
-			// The following values are not implemented in KAlarm
-			ANNUAL_DAY  = 8
+			DISPLAYING_     = 0x80000,
+			READ_ONLY_FLAGS = 0xF0000   // mask for all read-only internal values
 		};
+		/** The category of an event, indicated by the middle part of its UID. */
 		enum Status
 		{
 			ACTIVE,      // the event is currently active
 			EXPIRED,     // the event has expired
 			DISPLAYING,  // the event is currently being displayed
-			TEMPLATE     // the event is an alarm template
+			TEMPLATE,    // the event is an alarm template
+			KORGANIZER   // the event is a copy of a KAlarm event, held by KOrganizer
 		};
 		enum Action
 		{
@@ -286,12 +284,12 @@ class KAEvent : public KAAlarmEventBase
 		enum OccurType     // what type of occurrence is due
 		{
 			NO_OCCURRENCE               = 0,      // no occurrence is due
-			FIRST_OCCURRENCE            = 0x01,   // the first occurrence (takes precedence over LAST_RECURRENCE)
+			FIRST_OR_ONLY_OCCURRENCE    = 0x01,   // the first occurrence (takes precedence over LAST_RECURRENCE)
 			RECURRENCE_DATE             = 0x02,   // a recurrence with only a date, not a time
 			RECURRENCE_DATE_TIME        = 0x03,   // a recurrence with a date and time
 			LAST_RECURRENCE             = 0x04,   // the last recurrence
 			OCCURRENCE_REPEAT = 0x10,    // (bitmask for a repetition of an occurrence)
-			FIRST_OCCURRENCE_REPEAT     = OCCURRENCE_REPEAT | FIRST_OCCURRENCE,     // a repetition of the first occurrence
+			FIRST_OR_ONLY_OCCURRENCE_REPEAT = OCCURRENCE_REPEAT | FIRST_OR_ONLY_OCCURRENCE,     // a repetition of the first occurrence
 			RECURRENCE_DATE_REPEAT      = OCCURRENCE_REPEAT | RECURRENCE_DATE,      // a repetition of a date-only recurrence
 			RECURRENCE_DATE_TIME_REPEAT = OCCURRENCE_REPEAT | RECURRENCE_DATE_TIME, // a repetition of a date/time recurrence
 			LAST_RECURRENCE_REPEAT      = OCCURRENCE_REPEAT | LAST_RECURRENCE       // a repetition of the last recurrence
@@ -330,19 +328,17 @@ class KAEvent : public KAAlarmEventBase
 		                            { set(d, filename, bg, fg, f, FILE, lateCancel, flags | ANY_TIME); }
 		void               setFileName(const QDateTime& dt, const QString& filename, const QColor& bg, const QColor& fg, const QFont& f, int lateCancel, int flags)
 		                            { set(dt, filename, bg, fg, f, FILE, lateCancel, flags); }
-		void               setCommand(const QDate& d, const QString& command, int lateCancel, int flags)
-		                            { set(d, command, QColor(), QColor(), QFont(), COMMAND, lateCancel, flags | ANY_TIME); }
-		void               setCommand(const QDateTime& dt, const QString& command, int lateCancel, int flags)
-		                            { set(dt, command, QColor(), QColor(), QFont(), COMMAND, lateCancel, flags); }
+		void               setCommand(const QDate&, const QString& command, int lateCancel, int flags, const QString& logfile = QString::null);
+		void               setCommand(const QDateTime&, const QString& command, int lateCancel, int flags, const QString& logfile = QString::null);
 		void               setEmail(const QDate&, const QString& from, const EmailAddressList&, const QString& subject,
 		                            const QString& message, const QStringList& attachments, int lateCancel, int flags);
 		void               setEmail(const QDateTime&, const QString& from, const EmailAddressList&, const QString& subject,
 		                            const QString& message, const QStringList& attachments, int lateCancel, int flags);
 		void               setEmail(const QString& from, const EmailAddressList&, const QString& subject, const QStringList& attachments);
-		void               setAudioFile(const QString& filename, float volume) { mAudioFile = filename;  mSoundVolume = volume;  mUpdated = true; }
-		void               setTemplate(const QString& name, bool defaultTime)  { mTemplateName = name; mTemplateDefaultTime = defaultTime;  mUpdated = true; }
-		void               setActions(const QString& pre, const QString& post) { mPreAction = pre;  mPostAction = post;  mUpdated = true; }
-		OccurType          setNextOccurrence(const QDateTime& preDateTime, bool includeRepetitions = false);
+		void               setAudioFile(const QString& filename, float volume, float fadeVolume, int fadeSeconds);
+		void               setTemplate(const QString& name, int afterTime = -1)  { mTemplateName = name;  mTemplateAfterTime = afterTime;  mUpdated = true; }
+		void               setActions(const QString& pre, const QString& post)   { mPreAction = pre;  mPostAction = post;  mUpdated = true; }
+		OccurType          setNextOccurrence(const QDateTime& preDateTime);
 		void               setFirstRecurrence();
 		void               setEventID(const QString& id)                     { mEventID = id;  mUpdated = true; }
 		void               adjustStartDate(const QDate&);
@@ -354,10 +350,13 @@ class KAEvent : public KAAlarmEventBase
 		void               setRepeatAtLogin(bool rl)                         { mRepeatAtLogin = rl;  mUpdated = true; }
 		void               set(int flags);
 		void               setUid(Status s)                                  { mEventID = uid(mEventID, s);  mUpdated = true; }
+		void               setKMailSerialNumber(unsigned long n)             { mKMailSerialNumber = n; }
+		void               setLogFile(const QString& logfile);
 		void               setReminder(int minutes, bool onceOnly);
 		bool               defer(const DateTime&, bool reminder, bool adjustRecurrence = false);
 		void               cancelDefer();
 		void               cancelCancelledDeferral();
+		void               setDeferDefaultMinutes(int minutes)               { mDeferDefaultMinutes = minutes;  mUpdated = true; }
 		bool               setDisplaying(const KAEvent&, KAAlarm::Type, const QDateTime&);
 		void               reinstateFromDisplaying(const KAEvent& dispEvent);
 		void               setArchive()                                      { mArchive = true;  mUpdated = true; }
@@ -367,9 +366,11 @@ class KAEvent : public KAAlarmEventBase
 		void               removeExpiredAlarm(KAAlarm::Type);
 		void               incrementRevision()                               { ++mRevision;  mUpdated = true; }
 
+		KCal::Event*       event() const;    // convert to new Event
 		bool               isTemplate() const             { return !mTemplateName.isEmpty(); }
 		const QString&     templateName() const           { return mTemplateName; }
-		bool               usingDefaultTime() const       { return mTemplateDefaultTime; }
+		bool               usingDefaultTime() const       { return mTemplateAfterTime == 0; }
+		int                templateAfterTime() const      { return mTemplateAfterTime; }
 		KAAlarm            alarm(KAAlarm::Type) const;
 		KAAlarm            firstAlarm() const;
 		KAAlarm            nextAlarm(const KAAlarm& al) const  { return nextAlarm(al.type()); }
@@ -382,38 +383,44 @@ class KAEvent : public KAAlarmEventBase
 		bool               valid() const                  { return mAlarmCount  &&  (mAlarmCount != 1 || !mRepeatAtLogin); }
 		int                alarmCount() const             { return mAlarmCount; }
 		const DateTime&    startDateTime() const          { return mStartDateTime; }
-		const DateTime&    mainDateTime() const           { return mNextMainDateTime; }
+		DateTime           mainDateTime(bool withRepeats = false) const
+		                                                  { return (withRepeats && mNextRepeat && mRepeatInterval)
+		                                                    ? mNextMainDateTime.addSecs(mNextRepeat * mRepeatInterval * 60) : mNextMainDateTime; }
 		QDate              mainDate() const               { return mNextMainDateTime.date(); }
 		QTime              mainTime() const               { return mNextMainDateTime.time(); }
-		DateTime           mainEndRepeatTime() const      { return mRepeatCount ? mNextMainDateTime.addSecs(mRepeatCount * mRepeatInterval * 60) : mNextMainDateTime; }
+		DateTime           mainEndRepeatTime() const      { return (mRepeatCount > 0 && mRepeatInterval)
+		                                                    ? mNextMainDateTime.addSecs(mRepeatCount * mRepeatInterval * 60) : mNextMainDateTime; }
 		int                reminder() const               { return mReminderMinutes; }
 		bool               reminderOnceOnly() const       { return mReminderOnceOnly; }
 		bool               reminderDeferral() const       { return mDeferral == REMINDER_DEFERRAL; }
 		int                reminderArchived() const       { return mArchiveReminderMinutes; }
 		DateTime           deferDateTime() const          { return mDeferralTime; }
 		DateTime           deferralLimit(DeferLimitType* = 0) const;
+		int                deferDefaultMinutes() const    { return mDeferDefaultMinutes; }
 		DateTime           nextDateTime(bool includeReminders = true) const;
 		const QString&     messageFileOrCommand() const   { return mText; }
+		QString            logFile() const                { return mLogFile; }
+		bool               commandXterm() const           { return mCommandXterm; }
+		unsigned long      kmailSerialNumber() const      { return mKMailSerialNumber; }
+		bool               copyToKOrganizer() const       { return mCopyToKOrganizer; }
 		const QString&     audioFile() const              { return mAudioFile; }
-		float              soundVolume() const            { return mSoundVolume >= 0  &&  !mAudioFile.isEmpty() ? mSoundVolume : -1; }
+		float              soundVolume() const            { return !mAudioFile.isEmpty() ? mSoundVolume : -1; }
+		float              fadeVolume() const             { return !mAudioFile.isEmpty() && mSoundVolume >= 0 && mFadeSeconds ? mFadeVolume : -1; }
+		int                fadeSeconds() const            { return !mAudioFile.isEmpty() && mSoundVolume >= 0 && mFadeVolume >= 0 ? mFadeSeconds : 0; }
 		bool               repeatSound() const            { return mRepeatSound  &&  !mAudioFile.isEmpty(); }
 		const QString&     preAction() const              { return mPreAction; }
 		const QString&     postAction() const             { return mPostAction; }
-		bool               recurs() const                 { return checkRecur() != NO_RECUR; }
-		RecurType          recurType() const              { return checkRecur(); }
-		KCal::Recurrence*  recurrence() const             { return mRecurrence; }
-		bool               recursFeb29() const            { return mRecursFeb29; }
+		bool               recurs() const                 { return checkRecur() != KARecurrence::NO_RECUR; }
+		KARecurrence::Type recurType() const              { return checkRecur(); }
+		KARecurrence*      recurrence() const             { return mRecurrence; }
 		int                recurInterval() const;    // recurrence period in units of the recurrence period type (minutes, days, etc)
-		int                longestRecurrenceInterval() const    { return mRecurrence ? longestRecurrenceInterval(*mRecurrence) : 0; }
-		static int         longestRecurrenceInterval(const KCal::Recurrence&);   // longest interval between any recurrences, in minutes
+		int                longestRecurrenceInterval() const    { return mRecurrence ? mRecurrence->longestInterval() : 0; }
 		QString            recurrenceText(bool brief = false) const;
 		QString            repetitionText(bool brief = false) const;
 		int                remainingRecurrences() const   { return mRemainingRecurrences; }
 		bool               occursAfter(const QDateTime& preDateTime, bool includeRepetitions) const;
 		OccurType          nextOccurrence(const QDateTime& preDateTime, DateTime& result, OccurOption = IGNORE_REPETITION) const;
 		OccurType          previousOccurrence(const QDateTime& afterDateTime, DateTime& result, bool includeRepetitions = false) const;
-		const KCal::DateList& exceptionDates() const      { return mExceptionDates; }
-		const KCal::DateTimeList& exceptionDateTimes() const { return mExceptionDateTimes; }
 		int                flags() const;
 		bool               deferred() const               { return mDeferral > 0; }
 		bool               toBeArchived() const           { return mArchive; }
@@ -433,100 +440,25 @@ class KAEvent : public KAAlarmEventBase
 			QBitArray  days;        // days in week
 		};
 		bool               setRepetition(int interval, int count);
-		void               setExceptionDates(const KCal::DateList& d)      { mExceptionDates = d;  mUpdated = true; }
-		void               setExceptionDates(const KCal::DateTimeList& dt) { mExceptionDateTimes = dt;  mUpdated = true; }
-		void               setNoRecur()            { initRecur(); }
-		void               setRecurrence(const KCal::Recurrence&);
-		void               setRecurMinutely(int freq, int count)
-		                                           { setRecurMinutely(freq, count, QDateTime()); }
-		void               setRecurMinutely(int freq, const QDateTime& end)
-		                                           { setRecurMinutely(freq, 0, end); }
-		void               setRecurDaily(int freq, int count)
-		                                           { setRecurDaily(freq, count, QDate()); }
-		void               setRecurDaily(int freq, const QDate& end)
-		                                           { setRecurDaily(freq, 0, end); }
-		void               setRecurWeekly(int freq, const QBitArray& days, int count)
-		                                           { setRecurWeekly(freq, days, count, QDate()); }
-		void               setRecurWeekly(int freq, const QBitArray& days, const QDate& end)
-		                                           { setRecurWeekly(freq, days, 0, end); }
-		void               setRecurMonthlyByDate(int freq, const QValueList<int>& days, int count)
-		                                           { setRecurMonthlyByDate(freq, days, count, QDate()); }
-		void               setRecurMonthlyByDate(int freq, const QValueList<int>& days, const QDate& end)
-		                                           { setRecurMonthlyByDate(freq, days, 0, end); }
-		void               setRecurMonthlyByPos(int freq, const QValueList<MonthPos>& mp, int count)
-		                                           { setRecurMonthlyByPos(freq, mp, count, QDate()); }
-		void               setRecurMonthlyByPos(int freq, const QValueList<MonthPos>& mp, const QDate& end)
-		                                           { setRecurMonthlyByPos(freq, mp, 0, end); }
-		void               setRecurMonthlyByPos(int freq, const QPtrList<KCal::Recurrence::rMonthPos>& mp, int count)
-		                                           { setRecurMonthlyByPos(freq, mp, count, QDate()); }
-		void               setRecurMonthlyByPos(int freq, const QPtrList<KCal::Recurrence::rMonthPos>& mp, const QDate& end)
-		                                           { setRecurMonthlyByPos(freq, mp, 0, end); }
-		void               setRecurAnnualByDate(int freq, const QValueList<int>& months, int day, int count)
-		                                           { setRecurAnnualByDate(freq, months, day, -1, count, QDate()); }
-		void               setRecurAnnualByDate(int freq, const QValueList<int>& months, int day, const QDate& end)
-		                                           { setRecurAnnualByDate(freq, months, day, -1, 0, end); }
-		void               setRecurAnnualByDate(int freq, const QValueList<int>& months, int day, bool feb29, int count)
-		                                           { setRecurAnnualByDate(freq, months, day, feb29, count, QDate()); }
-		void               setRecurAnnualByDate(int freq, const QValueList<int>& months, int day, bool feb29, const QDate& end)
-		                                           { setRecurAnnualByDate(freq, months, day, feb29, 0, end); }
-		void               setRecurAnnualByPos(int freq, const QValueList<MonthPos>& mp, const QValueList<int>& months, int count)
-		                                           { setRecurAnnualByPos(freq, mp, months, count, QDate()); }
-		void               setRecurAnnualByPos(int freq, const QValueList<MonthPos>& mp, const QValueList<int>& months, const QDate& end)
-		                                           { setRecurAnnualByPos(freq, mp, months, 0, end); }
-		void               setRecurAnnualByDay(int freq, const QValueList<int>& days, int count)
-		                                           { setRecurAnnualByDay(freq, days, count, QDate()); }
-		void               setRecurAnnualByDay(int freq, const QValueList<int>& days, const QDate& end)
-		                                           { setRecurAnnualByDay(freq, days, 0, end); }
-
-		void               setRecurMinutely(int freq, int count, const QDateTime& end)
-		                                           { if (initRecur(end.date(), count))  setRecurMinutely(*mRecurrence, freq, count, end); }
-		void               setRecurDaily(int freq, int count, const QDate& end)
-		                                           { if (initRecur(end, count))  setRecurDaily(*mRecurrence, freq, count, end); }
-		void               setRecurWeekly(int freq, const QBitArray& days, int count, const QDate& end)
-		                                           { if (initRecur(end, count))  setRecurWeekly(*mRecurrence, freq, days, count, end); }
-		void               setRecurMonthlyByDate(int freq, const QValueList<int>& days, int count, const QDate& end)
-		                                           { if (initRecur(end, count))  setRecurMonthlyByDate(*mRecurrence, freq, days, count, end); }
-		void               setRecurMonthlyByDate(int freq, const QPtrList<int>& days, int count, const QDate& end)
-		                                           { if (initRecur(end, count))  setRecurMonthlyByDate(*mRecurrence, freq, days, count, end); }
-		void               setRecurMonthlyByPos(int freq, const QValueList<MonthPos>& pos, int count, const QDate& end)
-		                                           { if (initRecur(end, count))  setRecurMonthlyByPos(*mRecurrence, freq, pos, count, end); }
-		void               setRecurMonthlyByPos(int freq, const QPtrList<KCal::Recurrence::rMonthPos>& pos, int count, const QDate& end)
-		                                           { if (initRecur(end, count))  setRecurMonthlyByPos(*mRecurrence, freq, pos, count, end); }
-		void               setRecurAnnualByDate(int freq, const QValueList<int>& months, int day, bool feb29, int count, const QDate& end)
-		                                           { if (initRecur(end, count, feb29))  setRecurAnnualByDate(*mRecurrence, freq, months, day, count, end); }
-		void               setRecurAnnualByDate(int freq, const QPtrList<int>& months, int day, bool feb29, int count, const QDate& end)
-		                                           { if (initRecur(end, count, feb29))  setRecurAnnualByDate(*mRecurrence, freq, months, day, count, end); }
-		void               setRecurAnnualByPos(int freq, const QValueList<MonthPos>& pos, const QValueList<int>& months, int count, const QDate& end)
-		                                           { if (initRecur(end, count))  setRecurAnnualByPos(*mRecurrence, freq, pos, months, count, end); }
-		void               setRecurAnnualByPos(int freq, const QPtrList<KCal::Recurrence::rMonthPos>& pos, const QPtrList<int>& months, int count, const QDate& end)
-		                                           { if (initRecur(end, count))  setRecurAnnualByPos(*mRecurrence, freq, pos, months, count, end); }
-		void               setRecurAnnualByDay(int freq, const QValueList<int>& days, int count, const QDate& end)
-		                                           { if (initRecur(end, count))  setRecurAnnualByDay(*mRecurrence, freq, days, count, end); }
-		void               setRecurAnnualByDay(int freq, const QPtrList<int>& days, int count, const QDate& end)
-		                                           { if (initRecur(end, count))  setRecurAnnualByDay(*mRecurrence, freq, days, count, end); }
-
-		static bool        setRecurMinutely(KCal::Recurrence&, int freq, int count, const QDateTime& end);
-		static bool        setRecurDaily(KCal::Recurrence&, int freq, int count, const QDate& end);
-		static bool        setRecurWeekly(KCal::Recurrence&, int freq, const QBitArray& days, int count, const QDate& end);
-		static bool        setRecurMonthlyByDate(KCal::Recurrence&, int freq, const QValueList<int>& days, int count, const QDate& end);
-		static bool        setRecurMonthlyByDate(KCal::Recurrence&, int freq, const QPtrList<int>& days, int count, const QDate& end);
-		static bool        setRecurMonthlyByPos(KCal::Recurrence&, int freq, const QValueList<MonthPos>&, int count, const QDate& end);
-		static bool        setRecurMonthlyByPos(KCal::Recurrence&, int freq, const QPtrList<KCal::Recurrence::rMonthPos>&, int count, const QDate& end);
-		static bool        setRecurAnnualByDate(KCal::Recurrence&, int freq, const QValueList<int>& months, int day, int count, const QDate& end);
-		static bool        setRecurAnnualByDate(KCal::Recurrence&, int freq, const QPtrList<int>& months, int day, int count, const QDate& end);
-		static bool        setRecurAnnualByPos(KCal::Recurrence&, int freq, const QValueList<MonthPos>&, const QValueList<int>& months, int count, const QDate& end);
-		static bool        setRecurAnnualByPos(KCal::Recurrence&, int freq, const QPtrList<KCal::Recurrence::rMonthPos>&, const QPtrList<int>& months, int count, const QDate& end);
-		static bool        setRecurAnnualByDay(KCal::Recurrence&, int freq, const QValueList<int>& days, int count, const QDate& end);
-		static bool        setRecurAnnualByDay(KCal::Recurrence&, int freq, const QPtrList<int>& days, int count, const QDate& end);
-		static bool        setRecurrence(KCal::Recurrence&, RecurType, int repeatInterval, int repeatCount, const DateTime& start, const QDateTime& end);
+		void               setNoRecur()                   { clearRecur(); }
+		void               setRecurrence(const KARecurrence&);
+		bool               setRecurMinutely(int freq, int count, const QDateTime& end);
+		bool               setRecurDaily(int freq, const QBitArray& days, int count, const QDate& end);
+		bool               setRecurWeekly(int freq, const QBitArray& days, int count, const QDate& end);
+		bool               setRecurMonthlyByDate(int freq, const QValueList<int>& days, int count, const QDate& end);
+		bool               setRecurMonthlyByPos(int freq, const QValueList<MonthPos>& pos, int count, const QDate& end);
+		bool               setRecurAnnualByDate(int freq, const QValueList<int>& months, int day, KARecurrence::Feb29Type, int count, const QDate& end);
+		bool               setRecurAnnualByPos(int freq, const QValueList<MonthPos>& pos, const QValueList<int>& months, int count, const QDate& end);
+//		static QValueList<MonthPos> convRecurPos(const QValueList<KCal::RecurrenceRule::WDayPos>&);
 #ifdef NDEBUG
 		void               dumpDebug() const  { }
 #else
 		void               dumpDebug() const;
 #endif
-		static void        setFeb29RecurType();
+		static int         calVersion();
+		static QString     calVersionString();
 		static bool        adjustStartOfDay(const KCal::Event::List&);
-		static void        convertKCalEvents(AlarmCalendar&);
+		static void        convertKCalEvents(KCal::Calendar&, int version, bool adjustSummerTime);
 
 	private:
 		enum DeferType {
@@ -537,11 +469,13 @@ class KAEvent : public KAAlarmEventBase
 		};
 
 		void               copy(const KAEvent&);
-		bool               initRecur(const QDate& end = QDate(), int count = 0, bool feb29 = false);
-		RecurType          checkRecur() const;
+		bool               setRecur(KCal::RecurrenceRule::PeriodType, int freq, int count, const QDateTime& end, KARecurrence::Feb29Type = KARecurrence::FEB29_FEB29);
+		void               clearRecur();
+		KARecurrence::Type checkRecur() const;
 		OccurType          nextRecurrence(const QDateTime& preDateTime, DateTime& result, int& remainingCount) const;
 		OccurType          previousRecurrence(const QDateTime& afterDateTime, DateTime& result) const;
 		KCal::Alarm*       initKcalAlarm(KCal::Event&, const DateTime&, const QStringList& types, KAAlarm::Type = KAAlarm::INVALID_ALARM) const;
+		KCal::Alarm*       initKcalAlarm(KCal::Event&, int startOffsetSecs, const QStringList& types, KAAlarm::Type = KAAlarm::INVALID_ALARM) const;
 		static void        readAlarms(const KCal::Event&, void* alarmMap);
 		static void        readAlarm(const KCal::Alarm&, AlarmData&);
 		inline void        set_deferral(DeferType);
@@ -560,15 +494,17 @@ class KAEvent : public KAAlarmEventBase
 		int                mDisplayingFlags;  // type of alarm which is currently being displayed
 		int                mReminderMinutes;  // how long in advance reminder is to be, or 0 if none
 		int                mArchiveReminderMinutes;  // original reminder period if now expired, or 0 if none
+		int                mDeferDefaultMinutes; // default number of minutes for deferral dialogue, or 0 to select time control
 		int                mRevision;         // SEQUENCE: revision number of the original alarm, or 0
-		KCal::Recurrence*  mRecurrence;       // RECUR: recurrence specification, or 0 if none
+		KARecurrence*      mRecurrence;       // RECUR: recurrence specification, or 0 if none
 		int                mRemainingRecurrences; // remaining number of alarm recurrences including initial time, -1 to repeat indefinitely
-		KCal::DateList     mExceptionDates;   // list of dates to exclude from the recurrence
-		KCal::DateTimeList mExceptionDateTimes; // list of date/times to exclude from the recurrence
 		int                mAlarmCount;       // number of alarms: count of !mMainExpired, mRepeatAtLogin, mDeferral, mReminderMinutes, mDisplaying
 		DeferType          mDeferral;         // whether the alarm is an extra deferred/deferred-reminder alarm
-		bool               mTemplateDefaultTime; // time not specified: use default time (applies to templates only)
-		bool               mRecursFeb29;      // the recurrence is yearly on February 29th
+		unsigned long      mKMailSerialNumber;// if email text, message's KMail serial number
+		int                mTemplateAfterTime;// time not specified: use n minutes after default time, or -1 (applies to templates only)
+		QString            mLogFile;          // alarm output is to be logged to this URL
+		bool               mCommandXterm;     // command alarm is to be executed in a terminal window
+		bool               mCopyToKOrganizer; // KOrganizer should hold a copy of the event
 		bool               mReminderOnceOnly; // the reminder is output only for the first recurrence
 		bool               mMainExpired;      // main alarm has expired (in which case a deferral alarm will exist)
 		bool               mArchiveRepeatAtLogin; // if now expired, original event was repeat-at-login
