@@ -1,7 +1,7 @@
 /*
  *  alarmtimewidget.cpp  -  alarm date/time entry widget
  *  Program:  kalarm
- *  (C) 2001 - 2004 by David Jarvie <software@astrojar.org.uk>
+ *  Copyright © 2001-2006 by David Jarvie <software@astrojar.org.uk>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,9 +13,9 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "kalarm.h"
@@ -40,11 +40,20 @@
 #include "alarmtimewidget.moc"
 
 static const QTime time_23_59(23, 59);
-static const int   maxDelayTime = 99*60 + 59;    // < 100 hours
+
+
+const int AlarmTimeWidget::maxDelayTime = 99*60 + 59;    // < 100 hours
+
+QString AlarmTimeWidget::i18n_w_TimeFromNow()     { return i18n("Time from no&w:"); }
+QString AlarmTimeWidget::i18n_TimeAfterPeriod()
+{
+	return i18n("Enter the length of time (in hours and minutes) after "
+	            "the current time to schedule the alarm.");
+}
 
 
 /******************************************************************************
-*  Construct a widget with a group box and title.
+* Construct a widget with a group box and title.
 */
 AlarmTimeWidget::AlarmTimeWidget(const QString& groupBoxTitle, int mode, QWidget* parent, const char* name)
 	: ButtonGroup(groupBoxTitle, parent, name),
@@ -56,7 +65,7 @@ AlarmTimeWidget::AlarmTimeWidget(const QString& groupBoxTitle, int mode, QWidget
 }
 
 /******************************************************************************
-*  Construct a widget without a group box or title.
+* Construct a widget without a group box or title.
 */
 AlarmTimeWidget::AlarmTimeWidget(int mode, QWidget* parent, const char* name)
 	: ButtonGroup(parent, name),
@@ -78,7 +87,7 @@ void AlarmTimeWidget::init(int mode)
 	QBoxLayout* topLayout = new QVBoxLayout(this, 0, KDialog::spacingHint());
 	if (!title().isEmpty())
 	{
-		topLayout->setMargin(marginKDE2 + KDialog::marginHint());
+		topLayout->setMargin(KDialog::marginHint());
 		topLayout->addSpacing(fontMetrics().lineSpacing()/2);
 	}
 
@@ -125,7 +134,7 @@ void AlarmTimeWidget::init(int mode)
 	}
 
 	// 'Time from now' radio button/label
-	mAfterTimeRadio = new RadioButton(((mode & DEFER_TIME) ? i18n("Defer for time &interval:") : i18n("Time from no&w:")),
+	mAfterTimeRadio = new RadioButton(((mode & DEFER_TIME) ? i18n("Defer for time &interval:") : i18n_w_TimeFromNow()),
 	                                  this, "afterTimeRadio");
 	mAfterTimeRadio->setFixedSize(mAfterTimeRadio->sizeHint());
 	QWhatsThis::add(mAfterTimeRadio,
@@ -137,11 +146,9 @@ void AlarmTimeWidget::init(int mode)
 	mDelayTimeEdit->setValue(1439);
 	mDelayTimeEdit->setFixedSize(mDelayTimeEdit->sizeHint());
 	connect(mDelayTimeEdit, SIGNAL(valueChanged(int)), SLOT(delayTimeChanged(int)));
-	static const QString lengthText = i18n("Enter the length of time (in hours and minutes) after "
-	                                       "the current time to schedule the alarm.");
 	QWhatsThis::add(mDelayTimeEdit,
-	                ((mode & DEFER_TIME) ? QString("%1\n\n%2").arg(lengthText).arg(TimeSpinBox::shiftWhatsThis())
-	                                     : QString("%1\n%2\n\n%3").arg(lengthText).arg(recurText).arg(TimeSpinBox::shiftWhatsThis())));
+	                ((mode & DEFER_TIME) ? QString("%1\n\n%2").arg(i18n_TimeAfterPeriod()).arg(TimeSpinBox::shiftWhatsThis())
+	                                     : QString("%1\n%2\n\n%3").arg(i18n_TimeAfterPeriod()).arg(recurText).arg(TimeSpinBox::shiftWhatsThis())));
 	mAfterTimeRadio->setFocusWidget(mDelayTimeEdit);
 
 	// Set up the layout, either narrow or wide
@@ -193,14 +200,29 @@ void AlarmTimeWidget::setReadOnly(bool ro)
 }
 
 /******************************************************************************
-*  Fetch the entered date/time.
-*  If 'checkExpired' is true and the entered value <= current time, an error occurs.
-*  In this case, if 'showErrorMessage' is true, output an error message.
-*  'errorWidget' if non-null, is set to point to the widget containing the error.
-*  Reply = invalid date/time if error.
+* Select the "Time from now" radio button.
 */
-DateTime AlarmTimeWidget::getDateTime(bool checkExpired, bool showErrorMessage, QWidget** errorWidget) const
+void AlarmTimeWidget::selectTimeFromNow(int minutes)
 {
+	mAfterTimeRadio->setChecked(true);
+	slotButtonSet(1);
+	if (minutes > 0)
+		mDelayTimeEdit->setValue(minutes);
+}
+
+/******************************************************************************
+* Fetch the entered date/time.
+* If 'checkExpired' is true and the entered value <= current time, an error occurs.
+* If 'minsFromNow' is non-null, it is set to the number of minutes' delay selected,
+* or to zero if a date/time was entered.
+* In this case, if 'showErrorMessage' is true, output an error message.
+* 'errorWidget' if non-null, is set to point to the widget containing the error.
+* Reply = invalid date/time if error.
+*/
+DateTime AlarmTimeWidget::getDateTime(int* minsFromNow, bool checkExpired, bool showErrorMessage, QWidget** errorWidget) const
+{
+	if (minsFromNow)
+		*minsFromNow = 0;
 	if (errorWidget)
 		*errorWidget = 0;
 	QTime nowt = QTime::currentTime();
@@ -265,12 +287,15 @@ DateTime AlarmTimeWidget::getDateTime(bool checkExpired, bool showErrorMessage,
 				*errorWidget = mDelayTimeEdit;
 			return DateTime();
 		}
-		return now.addSecs(mDelayTimeEdit->value() * 60);
+		int delayMins = mDelayTimeEdit->value();
+		if (minsFromNow)
+			*minsFromNow = delayMins;
+		return now.addSecs(delayMins * 60);
 	}
 }
 
 /******************************************************************************
-*  Set the date/time.
+* Set the date/time.
 */
 void AlarmTimeWidget::setDateTime(const DateTime& dt)
 {
@@ -283,7 +308,7 @@ void AlarmTimeWidget::setDateTime(const DateTime& dt)
 	else
 	{
 		mTimeEdit->setValid(false);
-		mDateEdit->setValid(false);
+		mDateEdit->setInvalid();
 		mDelayTimeEdit->setValid(false);
 	}
 	if (mAnyTimeCheckBox)
@@ -297,7 +322,7 @@ void AlarmTimeWidget::setDateTime(const DateTime& dt)
 }
 
 /******************************************************************************
-*  Set the minimum date/time to track the current time.
+* Set the minimum date/time to track the current time.
 */
 void AlarmTimeWidget::setMinDateTimeIsCurrent()
 {
@@ -309,8 +334,8 @@ void AlarmTimeWidget::setMinDateTimeIsCurrent()
 }
 
 /******************************************************************************
-*  Set the minimum date/time, adjusting the entered date/time if necessary.
-*  If 'dt' is invalid, any current minimum date/time is cleared.
+* Set the minimum date/time, adjusting the entered date/time if necessary.
+* If 'dt' is invalid, any current minimum date/time is cleared.
 */
 void AlarmTimeWidget::setMinDateTime(const QDateTime& dt)
 {
@@ -321,8 +346,8 @@ void AlarmTimeWidget::setMinDateTime(const QDateTime& dt)
 }
 
 /******************************************************************************
-*  Set the maximum date/time, adjusting the entered date/time if necessary.
-*  If 'dt' is invalid, any current maximum date/time is cleared.
+* Set the maximum date/time, adjusting the entered date/time if necessary.
+* If 'dt' is invalid, any current maximum date/time is cleared.
 */
 void AlarmTimeWidget::setMaxDateTime(const DateTime& dt)
 {
@@ -338,8 +363,8 @@ void AlarmTimeWidget::setMaxDateTime(const DateTime& dt)
 }
 
 /******************************************************************************
-*  If the minimum and maximum date/times fall on the same date, set the minimum
-*  and maximum times in the time edit box.
+* If the minimum and maximum date/times fall on the same date, set the minimum
+* and maximum times in the time edit box.
 */
 void AlarmTimeWidget::setMaxMinTimeIf(const QDateTime& now)
 {
@@ -371,8 +396,8 @@ void AlarmTimeWidget::setMaxMinTimeIf(const QDateTime& now)
 }
 
 /******************************************************************************
-*  Set the maximum value for the delay time edit box, depending on the maximum
-*  value for the date/time.
+* Set the maximum value for the delay time edit box, depending on the maximum
+* value for the date/time.
 */
 void AlarmTimeWidget::setMaxDelayTime(const QDateTime& now)
 {
@@ -381,7 +406,8 @@ void AlarmTimeWidget::setMaxDelayTime(const QDateTime& now)
 	{
 		if (now.date().daysTo(mMaxDateTime.date()) < 100)    // avoid possible 32-bit overflow on secsTo()
 		{
-			maxVal = now.secsTo(mMaxDateTime) / 60;
+			QDateTime dt(now.date(), QTime(now.time().hour(), now.time().minute(), 0));   // round down to nearest minute
+			maxVal = dt.secsTo(mMaxDateTime) / 60;
 			if (maxVal > maxDelayTime)
 				maxVal = maxDelayTime;
 		}
@@ -390,7 +416,7 @@ void AlarmTimeWidget::setMaxDelayTime(const QDateTime& now)
 }
 
 /******************************************************************************
-*  Set the status for whether a time is specified, or just a date.
+* Set the status for whether a time is specified, or just a date.
 */
 void AlarmTimeWidget::setAnyTime()
 {
@@ -401,7 +427,7 @@ void AlarmTimeWidget::setAnyTime()
 }
 
 /******************************************************************************
-*  Enable/disable the "any time" checkbox.
+* Enable/disable the "any time" checkbox.
 */
 void AlarmTimeWidget::enableAnyTime(bool enable)
 {
@@ -417,8 +443,8 @@ void AlarmTimeWidget::enableAnyTime(bool enable)
 }
 
 /******************************************************************************
-*  Called every minute to update the alarm time data entry fields.
-*  If the maximum date/time has been reached, a 'pastMax()' signal is emitted.
+* Called every minute to update the alarm time data entry fields.
+* If the maximum date/time has been reached, a 'pastMax()' signal is emitted.
 */
 void AlarmTimeWidget::slotTimer()
 {
@@ -464,8 +490,8 @@ void AlarmTimeWidget::slotTimer()
 
 
 /******************************************************************************
-*  Called when the At or After time radio button states have been set.
-*  Updates the appropriate edit box.
+* Called when the At or After time radio button states have been set.
+* Updates the appropriate edit box.
 */
 void AlarmTimeWidget::slotButtonSet(int)
 {
@@ -484,7 +510,7 @@ void AlarmTimeWidget::slotButtonSet(int)
 }
 
 /******************************************************************************
-*  Called after the mAnyTimeCheckBox checkbox has been toggled.
+* Called after the mAnyTimeCheckBox checkbox has been toggled.
 */
 void AlarmTimeWidget::slotAnyTimeToggled(bool on)
 {
@@ -493,8 +519,8 @@ void AlarmTimeWidget::slotAnyTimeToggled(bool on)
 }
 
 /******************************************************************************
-*  Called when the date or time edit box values have changed.
-*  Updates the time delay edit box accordingly.
+* Called when the date or time edit box values have changed.
+* Updates the time delay edit box accordingly.
 */
 void AlarmTimeWidget::dateTimeChanged()
 {
@@ -510,8 +536,8 @@ void AlarmTimeWidget::dateTimeChanged()
 }
 
 /******************************************************************************
-*  Called when the delay time edit box value has changed.
-*  Updates the Date and Time edit boxes accordingly.
+* Called when the delay time edit box value has changed.
+* Updates the Date and Time edit boxes accordingly.
 */
 void AlarmTimeWidget::delayTimeChanged(int minutes)
 {
