@@ -1,7 +1,7 @@
 /*
  *  mainwindow.cpp  -  main application window
  *  Program:  kalarm
- *  Copyright (C) 2001 - 2005 by David Jarvie <software@astrojar.org.uk>
+ *  Copyright © 2001-2007 by David Jarvie <software@astrojar.org.uk>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 
 #include <qiconset.h>
 #include <qdragobject.h>
+#include <qheader.h>
 
 #include <kmenubar.h>
 #include <ktoolbar.h>
@@ -67,15 +68,17 @@
 
 using namespace KCal;
 
-static const char* UI_FILE = "kalarmui.rc";
-QString   undoText;
-QString   undoTextStripped;
-QString   undoIcon;
-KShortcut undoShortcut;
-QString   redoText;
-QString   redoTextStripped;
-QString   redoIcon;
-KShortcut redoShortcut;
+static const char* UI_FILE     = "kalarmui.rc";
+static const char* WINDOW_NAME = "MainWindow";
+
+static QString   undoText;
+static QString   undoTextStripped;
+static QString   undoIcon;
+static KShortcut undoShortcut;
+static QString   redoText;
+static QString   redoTextStripped;
+static QString   redoIcon;
+static KShortcut redoShortcut;
 
 
 /*=============================================================================
@@ -111,7 +114,7 @@ MainWindow* MainWindow::create(bool restored)
 }
 
 MainWindow::MainWindow(bool restored)
-	: MainWindowBase(0, 0, WGroupLeader | WStyle_ContextHelp | WDestructiveClose),
+	: MainWindowBase(0, "MainWin", WGroupLeader | WStyle_ContextHelp | WDestructiveClose),
 	  mMinuteTimerActive(false),
 	  mHiddenTrayParent(false),
 	  mShowExpired(Preferences::showExpiredAlarms()),
@@ -119,19 +122,22 @@ MainWindow::MainWindow(bool restored)
 	  mShowTimeTo(Preferences::showTimeToAlarm())
 {
 	kdDebug(5950) << "MainWindow::MainWindow()\n";
-	setAutoSaveSettings(QString::fromLatin1("MainWindow"));    // save window sizes etc.
+	setAutoSaveSettings(QString::fromLatin1(WINDOW_NAME));    // save window sizes etc.
 	setPlainCaption(kapp->aboutData()->programName());
 	if (!restored)
 	{
 		QSize s;
-		if (KAlarm::readConfigWindowSize("MainWindow", s))
+		if (KAlarm::readConfigWindowSize(WINDOW_NAME, s))
 			resize(s);
 	}
+	KConfig* config = KGlobal::config();
+	config->setGroup(QString::fromLatin1(WINDOW_NAME));
+	QValueList<int> order = config->readIntListEntry(QString::fromLatin1("ColumnOrder"));
 
 	setAcceptDrops(true);         // allow drag-and-drop onto this window
 	if (!mShowTimeTo)
 		mShowTime = true;     // ensure at least one time column is visible
-	mListView = new AlarmListView(this, "listView");
+	mListView = new AlarmListView(order, this, "listView");
 	mListView->selectTimeColumns(mShowTime, mShowTimeTo);
 	mListView->showExpired(mShowExpired);
 	setCentralWidget(mListView);
@@ -143,6 +149,7 @@ MainWindow::MainWindow(bool restored)
 	connect(mListView, SIGNAL(mouseButtonClicked(int, QListViewItem*, const QPoint&, int)),
 	        SLOT(slotMouseClicked(int, QListViewItem*, const QPoint&, int)));
 	connect(mListView, SIGNAL(executed(QListViewItem*)), SLOT(slotDoubleClicked(QListViewItem*)));
+	connect(mListView->header(), SIGNAL(indexChange(int, int, int)), SLOT(columnsReordered()));
 	initActions();
 
 	mWindowList.append(this);
@@ -173,7 +180,10 @@ MainWindow::~MainWindow()
 	setUpdateTimer();
 	MainWindow* main = mainMainWindow();
 	if (main)
-		KAlarm::writeConfigWindowSize("MainWindow", main->size());
+		KAlarm::writeConfigWindowSize(WINDOW_NAME, main->size());
+	KToolBar* tb = toolBar();
+	if (tb)
+		tb->saveSettings(KGlobal::config(), "Toolbars");
 	KGlobal::config()->sync();    // save any new window size to disc
 	theApp()->quitIf();
 }
@@ -218,6 +228,8 @@ MainWindow* MainWindow::mainMainWindow()
 			return *it;
 	if (tray)
 		return tray;
+	if (mWindowList.isEmpty())
+		return 0;
 	return mWindowList.first();
 }
 
@@ -248,7 +260,7 @@ void MainWindow::resizeEvent(QResizeEvent* re)
 {
 	// Save the window's new size only if it's the first main window
 	if (mainMainWindow() == this)
-		KAlarm::writeConfigWindowSize("MainWindow", re->size());
+		KAlarm::writeConfigWindowSize(WINDOW_NAME, re->size());
 	MainWindowBase::resizeEvent(re);
 }
 
@@ -289,6 +301,18 @@ void MainWindow::hideEvent(QHideEvent* he)
 	MainWindowBase::hideEvent(he);
 }
 
+/******************************************************************************
+*  Called when the list's column order is changed.
+*  Save the new column order as the default the next time the program is run.
+*/
+void MainWindow::columnsReordered()
+{
+	KConfig* config = KGlobal::config();
+	config->setGroup(QString::fromLatin1(WINDOW_NAME));
+	config->writeEntry(QString::fromLatin1("ColumnOrder"), mListView->columnOrder());
+	config->sync();
+}
+
 /******************************************************************************
 *  Initialise the menu, toolbar and main window actions.
 */
@@ -311,8 +335,9 @@ void MainWindow::initActions()
 	mActionShowTimeTo->setCheckedState(i18n("Hide Time t&o Alarms"));
 	mActionShowExpired     = new KToggleAction(i18n_e_ShowExpiredAlarms(), "history", Qt::CTRL+Qt::Key_P, this, SLOT(slotShowExpired()), actions, "showExpiredAlarms");
 	mActionShowExpired->setCheckedState(i18n_e_HideExpiredAlarms());
-	mActionToggleTrayIcon  = new KToggleAction(i18n("Show in System &Tray"), Qt::CTRL+Qt::Key_Y, this, SLOT(slotToggleTrayIcon()), actions, "showInSystemTray");
+	mActionToggleTrayIcon  = new KToggleAction(i18n("Show in System &Tray"), 0, this, SLOT(slotToggleTrayIcon()), actions, "showInSystemTray");
 	mActionToggleTrayIcon->setCheckedState(i18n("Hide From System &Tray"));
+	new KAction(i18n("Import &Alarms..."), 0, this, SLOT(slotImportAlarms()), actions, "importAlarms");
 	new KAction(i18n("Import &Birthdays..."), 0, this, SLOT(slotBirthdays()), actions, "importBirthdays");
 	new KAction(i18n("&Refresh Alarms"), "reload", 0, this, SLOT(slotResetDaemon()), actions, "refreshAlarms");
 	Daemon::createAlarmEnableAction(actions, "alarmEnable");
@@ -337,6 +362,8 @@ void MainWindow::initActions()
 	KStdAction::find(mListView, SLOT(slotFind()), actions);
 	mActionFindNext = KStdAction::findNext(mListView, SLOT(slotFindNext()), actions);
 	mActionFindPrev = KStdAction::findPrev(mListView, SLOT(slotFindPrev()), actions);
+	KStdAction::selectAll(mListView, SLOT(slotSelectAll()), actions);
+	KStdAction::deselect(mListView, SLOT(slotDeselect()), actions);
 	KStdAction::quit(this, SLOT(slotQuit()), actions);
 	KStdAction::keyBindings(this, SLOT(slotConfigureKeys()), actions);
 	KStdAction::configureToolbars(this, SLOT(slotConfigureToolbar()), actions);
@@ -378,6 +405,10 @@ void MainWindow::initActions()
 	mActionEnable->setEnabled(false);
 	mActionCreateTemplate->setEnabled(false);
 
+	KToolBar* tb = toolBar();
+	if (tb)
+		tb->applySettings(KGlobal::config(), "Toolbars");
+
 	Undo::emitChanged();     // set the Undo/Redo menu texts
 	Daemon::checkStatus();
 	Daemon::monitoringAlarms();
@@ -536,7 +567,7 @@ void MainWindow::slotNew()
 */
 void MainWindow::executeNew(MainWindow* win, const KAEvent* evnt, KAEvent::Action action, const AlarmText& text)
 {
-	EditAlarmDlg editDlg(false, i18n("New Alarm"), win, "editDlg", evnt);
+	EditAlarmDlg editDlg(false, i18n("New Alarm"), win, 0, evnt);
 	if (!text.isEmpty())
 		editDlg.setAction(action, text);
 	if (editDlg.exec() == QDialog::Accepted)
@@ -545,8 +576,8 @@ void MainWindow::executeNew(MainWindow* win, const KAEvent* evnt, KAEvent::Actio
 		editDlg.getEvent(event);
 
 		// Add the alarm to the displayed lists and to the calendar file
-		if (KAlarm::addEvent(event, (win ? win->mListView : 0)) == KAlarm::UPDATE_KORG_ERR)
-			KAlarm::displayUpdateError(win, KAlarm::KORG_ERR_ADD);
+		if (KAlarm::addEvent(event, (win ? win->mListView : 0), &editDlg) == KAlarm::UPDATE_KORG_ERR)
+			KAlarm::displayKOrgUpdateError(&editDlg, KAlarm::KORG_ERR_ADD, 1);
 		Undo::saveAdd(event);
 
 		KAlarm::outputAlarmWarnings(&editDlg, &event);
@@ -597,27 +628,37 @@ void MainWindow::slotModify()
 	if (item)
 	{
 		KAEvent event = item->event();
-		EditAlarmDlg editDlg(false, i18n("Edit Alarm"), this, "editDlg", &event);
-		if (editDlg.exec() == QDialog::Accepted)
+		executeEdit(event, this);
+	}
+}
+
+/******************************************************************************
+*  Open the Edit Alarm dialogue to edit the specified alarm.
+*/
+void MainWindow::executeEdit(KAEvent& event, MainWindow* win)
+{
+	EditAlarmDlg editDlg(false, i18n("Edit Alarm"), win, 0, &event);
+	if (editDlg.exec() == QDialog::Accepted)
+	{
+		KAEvent newEvent;
+		bool changeDeferral = !editDlg.getEvent(newEvent);
+
+		// Update the event in the displays and in the calendar file
+		AlarmListView* view = win ? win->mListView : 0;
+		if (changeDeferral)
 		{
-			KAEvent newEvent;
-			bool changeDeferral = !editDlg.getEvent(newEvent);
-
-			// Update the event in the displays and in the calendar file
-			if (changeDeferral)
-			{
-				// The only change has been to an existing deferral
-				KAlarm::updateEvent(newEvent, mListView, true, false);   // keep the same event ID
-			}
-			else
-			{
-				if (KAlarm::modifyEvent(event, newEvent, mListView) == KAlarm::UPDATE_KORG_ERR)
-					KAlarm::displayUpdateError(this, KAlarm::KORG_ERR_MODIFY);
-			}
-			Undo::saveEdit(event, newEvent);
-
-			KAlarm::outputAlarmWarnings(&editDlg, &newEvent);
+			// The only change has been to an existing deferral
+			if (KAlarm::updateEvent(newEvent, view, &editDlg, true, false) != KAlarm::UPDATE_OK)   // keep the same event ID
+				return;   // failed to save event
 		}
+		else
+		{
+			if (KAlarm::modifyEvent(event, newEvent, view, &editDlg) == KAlarm::UPDATE_KORG_ERR)
+				KAlarm::displayKOrgUpdateError(&editDlg, KAlarm::KORG_ERR_MODIFY, 1);
+		}
+		Undo::saveEdit(event, newEvent);
+
+		KAlarm::outputAlarmWarnings(&editDlg, &newEvent);
 	}
 }
 
@@ -631,9 +672,9 @@ void MainWindow::slotView()
 	if (item)
 	{
 		KAEvent event = item->event();
-		EditAlarmDlg editDlg(false, (event.expired() ? i18n("Expired Alarm") + " [" + i18n("read-only") + "]"
+		EditAlarmDlg editDlg(false, (event.expired() ? i18n("Expired Alarm") + " [" + i18n("read-only") + ']'
 		                                             : i18n("View Alarm")),
-		                     this, "editDlg", &event, true);
+		                     this, 0, &event, true);
 		editDlg.exec();
 	}
 }
@@ -645,6 +686,16 @@ void MainWindow::slotView()
 void MainWindow::slotDelete()
 {
 	QValueList<EventListViewItemBase*> items = mListView->selectedItems();
+	// Copy the events to be deleted, in case any are deleted by being
+	// triggered while the confirmation prompt is displayed.
+	QValueList<KAEvent> events;
+	QValueList<KAEvent> origEvents;
+	for (QValueList<EventListViewItemBase*>::Iterator iit = items.begin();  iit != items.end();  ++iit)
+	{
+		AlarmListViewItem* item = (AlarmListViewItem*)(*iit);
+		events.append(item->event());
+		origEvents.append(item->event());
+	}
 	if (Preferences::confirmAlarmDeletion())
 	{
 		int n = items.count();
@@ -657,26 +708,36 @@ void MainWindow::slotDelete()
 			return;
 	}
 
+	int warnErr = 0;
 	int warnKOrg = 0;
-	QValueList<KAEvent> events;
 	AlarmCalendar::activeCalendar()->startUpdate();    // prevent multiple saves of the calendars until we're finished
 	AlarmCalendar::expiredCalendar()->startUpdate();
-	for (QValueList<EventListViewItemBase*>::Iterator it = items.begin();  it != items.end();  ++it)
+	for (QValueList<KAEvent>::Iterator it = events.begin();  it != events.end();  ++it)
 	{
-		AlarmListViewItem* item = (AlarmListViewItem*)(*it);
-		KAEvent event = item->event();
-
 		// Delete the event from the calendar and displays
-		events.append(event);
-		if (KAlarm::deleteEvent(event) == KAlarm::UPDATE_KORG_ERR)
-			++warnKOrg;
+		switch (KAlarm::deleteEvent(*it))
+		{
+			case KAlarm::UPDATE_ERROR:
+			case KAlarm::UPDATE_FAILED:
+			case KAlarm::SAVE_FAILED:
+				++warnErr;
+				break;
+			case KAlarm::UPDATE_KORG_ERR:
+				++warnKOrg;
+				break;
+			default:
+				break;
+		}
 	}
-	AlarmCalendar::activeCalendar()->endUpdate();      // save the calendars now
+	if (!AlarmCalendar::activeCalendar()->endUpdate())      // save the calendars now
+		warnErr = events.count();
 	AlarmCalendar::expiredCalendar()->endUpdate();
-	Undo::saveDeletes(events);
+	Undo::saveDeletes(origEvents);
 
-	if (warnKOrg)
-		KAlarm::displayUpdateError(this, KAlarm::KORG_ERR_DELETE, (warnKOrg > 1));
+	if (warnErr)
+		KAlarm::displayUpdateError(this, KAlarm::UPDATE_FAILED, KAlarm::ERR_DELETE, warnErr);
+	else if (warnKOrg)
+		KAlarm::displayKOrgUpdateError(this, KAlarm::KORG_ERR_DELETE, warnKOrg);
 }
 
 /******************************************************************************
@@ -685,6 +746,7 @@ void MainWindow::slotDelete()
 */
 void MainWindow::slotReactivate()
 {
+	int warnErr = 0;
 	int warnKOrg = 0;
 	QValueList<KAEvent> events;
 	QValueList<EventListViewItemBase*> items = mListView->selectedItems();
@@ -697,15 +759,29 @@ void MainWindow::slotReactivate()
 		AlarmListViewItem* item = (AlarmListViewItem*)(*it);
 		KAEvent event = item->event();
 		events.append(event);
-		if (KAlarm::reactivateEvent(event, mListView, true) == KAlarm::UPDATE_KORG_ERR)
-			++warnKOrg;
+		switch (KAlarm::reactivateEvent(event, mListView, true))
+		{
+			case KAlarm::UPDATE_ERROR:
+			case KAlarm::UPDATE_FAILED:
+			case KAlarm::SAVE_FAILED:
+				++warnErr;
+				break;
+			case KAlarm::UPDATE_KORG_ERR:
+				++warnKOrg;
+				break;
+			default:
+				break;
+		}
 	}
-	AlarmCalendar::activeCalendar()->endUpdate();      // save the calendars now
+	if (!AlarmCalendar::activeCalendar()->endUpdate())      // save the calendars now
+		warnErr = items.count();
 	AlarmCalendar::expiredCalendar()->endUpdate();
 	Undo::saveReactivates(events);
 
-	if (warnKOrg)
-		KAlarm::displayUpdateError(this, KAlarm::KORG_ERR_ADD, (warnKOrg > 1));
+	if (warnErr)
+		KAlarm::displayUpdateError(this, KAlarm::UPDATE_FAILED, KAlarm::ERR_REACTIVATE, warnErr);
+	else if (warnKOrg)
+		KAlarm::displayKOrgUpdateError(this, KAlarm::KORG_ERR_ADD, warnKOrg);
 }
 
 /******************************************************************************
@@ -715,6 +791,7 @@ void MainWindow::slotReactivate()
 void MainWindow::slotEnable()
 {
 	bool enable = mActionEnableEnable;    // save since changed in response to KAlarm::enableEvent()
+	int warnErr = 0;
 	QValueList<EventListViewItemBase*> items = mListView->selectedItems();
 	AlarmCalendar::activeCalendar()->startUpdate();    // prevent multiple saves of the calendars until we're finished
 	for (QValueList<EventListViewItemBase*>::Iterator it = items.begin();  it != items.end();  ++it)
@@ -723,9 +800,13 @@ void MainWindow::slotEnable()
 		KAEvent event = item->event();
 
 		// Enable the alarm in the displayed lists and in the calendar file
-		KAlarm::enableEvent(event, mListView, enable);
+		if (KAlarm::enableEvent(event, mListView, enable) != KAlarm::UPDATE_OK)
+			++warnErr;
 	}
-	AlarmCalendar::activeCalendar()->endUpdate();      // save the calendars now
+	if (!AlarmCalendar::activeCalendar()->endUpdate())      // save the calendars now
+		warnErr = items.count();
+	if (warnErr)
+		KAlarm::displayUpdateError(this, KAlarm::UPDATE_FAILED, KAlarm::ERR_ADD, warnErr);
 }
 
 /******************************************************************************
@@ -767,6 +848,16 @@ void MainWindow::slotShowExpired()
 	mListView->refresh();
 }
 
+/******************************************************************************
+*  Called when the Import Alarms menu item is selected, to merge alarms from an
+*  external calendar into the current calendars.
+*/
+void MainWindow::slotImportAlarms()
+{
+	if (AlarmCalendar::importAlarms(this))
+		mListView->refresh();
+}
+
 /******************************************************************************
 *  Called when the Import Birthdays menu item is selected, to display birthdays
 *  from the address book for selection as alarms.
@@ -780,15 +871,29 @@ void MainWindow::slotBirthdays()
 		if (events.count())
 		{
 			mListView->clearSelection();
+			int warnErr = 0;
 			int warnKOrg = 0;
 			for (QValueList<KAEvent>::Iterator ev = events.begin();  ev != events.end();  ++ev)
 			{
 				// Add alarm to the displayed lists and to the calendar file
-				if (KAlarm::addEvent(*ev, mListView) == KAlarm::UPDATE_KORG_ERR)
-					++warnKOrg;
+				switch (KAlarm::addEvent(*ev, mListView))
+				{
+					case KAlarm::UPDATE_ERROR:
+					case KAlarm::UPDATE_FAILED:
+					case KAlarm::SAVE_FAILED:
+						++warnErr;
+						break;
+					case KAlarm::UPDATE_KORG_ERR:
+						++warnKOrg;
+						break;
+					default:
+						break;
+				}
 			}
-			if (warnKOrg)
-				KAlarm::displayUpdateError(this, KAlarm::KORG_ERR_ADD, (warnKOrg > 1));
+			if (warnErr)
+				KAlarm::displayUpdateError(this, KAlarm::UPDATE_FAILED, KAlarm::ERR_ADD, warnErr);
+			else if (warnKOrg)
+				KAlarm::displayKOrgUpdateError(this, KAlarm::KORG_ERR_ADD, warnKOrg);
 			KAlarm::outputAlarmWarnings(&dlg);
 		}
 	}
@@ -837,7 +942,7 @@ void MainWindow::slotToggleTrayIcon()
 */
 void MainWindow::updateTrayIconAction()
 {
-	mActionToggleTrayIcon->setEnabled(theApp()->KDEDesktop() && !theApp()->wantRunInSystemTray());
+	mActionToggleTrayIcon->setEnabled(theApp()->haveSystemTray() && !theApp()->wantRunInSystemTray());
 	mActionToggleTrayIcon->setChecked(theApp()->trayIconDisplayed());
 }
 
@@ -983,11 +1088,22 @@ void MainWindow::slotConfigureKeys()
 */
 void MainWindow::slotConfigureToolbar()
 {
-	saveMainWindowSettings(KGlobal::config(), "MainWindow");
+	saveMainWindowSettings(KGlobal::config(), WINDOW_NAME);
 	KEditToolbar dlg(factory());
+	connect(&dlg, SIGNAL(newToolbarConfig()), this, SLOT(slotNewToolbarConfig()));
 	dlg.exec();
 }
 
+/******************************************************************************
+*  Called when OK or Apply is clicked in the Configure Toolbars dialog, to save
+*  the new configuration.
+*/
+void MainWindow::slotNewToolbarConfig()
+{
+	createGUI(UI_FILE);
+	applyMainWindowSettings(KGlobal::config(), WINDOW_NAME);
+}
+
 /******************************************************************************
 *  Called when the Quit menu item is selected.
 */
@@ -1108,7 +1224,8 @@ void MainWindow::executeDropEvent(MainWindow* win, QDropEvent* e)
 		content.setContent(mails);
 		content.parse();
 		QString body;
-		content.textContent()->decodedText(body, true, true);    // strip trailing newlines & spaces
+		if (content.textContent())
+			content.textContent()->decodedText(body, true, true);    // strip trailing newlines & spaces
 		unsigned long sernum = 0;
 		if (e->provides(KPIM::MailListDrag::format())
 		&&  KPIM::MailListDrag::decode(e, mailList)
