@@ -1,7 +1,7 @@
 /*
  *  messagewin.h  -  displays an alarm message
  *  Program:  kalarm
- *  Copyright (C) 2001 - 2005 by David Jarvie <software@astrojar.org.uk>
+ *  Copyright © 2001-2007 by David Jarvie <software@astrojar.org.uk>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -99,11 +99,13 @@ class MessageWin : public MainWindowBase
 		QFont               mFont;
 		QColor              mBgColour, mFgColour;
 		DateTime            mDateTime;        // date/time displayed in the message window
+		QDateTime           mCloseTime;       // time at which window should be auto-closed
 		QString             mEventID;
 		QString             mAudioFile;
 		float               mVolume;
 		float               mFadeVolume;
 		int                 mFadeSeconds;
+		int                 mDefaultDeferMinutes;
 		KAAlarm::Type       mAlarmType;
 		KAEvent::Action     mAction;
 		unsigned long       mKMailSerialNumber; // if email text, message's KMail serial number, else 0
@@ -152,6 +154,7 @@ class MessageWin : public MainWindowBase
 		bool                mPositioning;     // true when the window is being positioned initially
 		bool                mNoCloseConfirm;  // the Defer or Edit button is closing the dialog
 		bool                mUsingKMix;       // master volume is being set using kmix
+		bool                mDisableDeferral; // true if past deferral limit, so don't enable Defer button
 };
 
 #endif // MESSAGEWIN_H
