@@ -114,7 +114,9 @@ imapParser::sendCommand (imapCommand * aCmd)
            || command == "LISTRIGHTS"
            || command == "MYRIGHTS"
            || command == "GETANNOTATION"
-           || command == "NAMESPACE")
+           || command == "NAMESPACE"
+           || command == "GETQUOTAROOT"
+           || command == "GETQUOTA")
   {
     lastResults.clear ();
   }
@@ -430,6 +432,16 @@ imapParser::parseUntagged (parseString & result)
       parseAnnotation (result);
     }
     break;
+  case 'Q': // QUOTA or QUOTAROOT
+    if ( what.size() > 5 && qstrncmp(what, "QUOTAROOT", what.size()) == 0)
+    {
+      parseQuotaRoot( result );
+    }
+    else if (qstrncmp(what, "QUOTA", what.size()) == 0)
+    {
+      parseQuota( result );
+    }
+
   default:
     //better be a number
     {
@@ -707,6 +719,48 @@ void imapParser::parseAnnotation (parseString & result)
   }
 }
 
+
+void imapParser::parseQuota (parseString & result)
+{
+  // quota_response  ::= "QUOTA" SP astring SP quota_list
+  // quota_list      ::= "(" #quota_resource ")"
+  // quota_resource  ::= atom SP number SP number
+  QCString root = parseOneWordC( result );
+  if ( root.isEmpty() ) {
+    lastResults.append( "" );
+  } else {
+    lastResults.append( root );
+  }
+  if (result.isEmpty() || result[0] != '(')
+    return;
+  result.pos++;
+  skipWS (result);
+  QStringList triplet;
+  int outlen = 1;
+  while ( outlen && !result.isEmpty() && result[0] != ')' ) {
+    QCString word = parseLiteralC (result, false, false, &outlen);
+    triplet.append(word);
+  }
+  lastResults.append( triplet.join(" ") );
+}
+
+void imapParser::parseQuotaRoot (parseString & result)
+{
+  //    quotaroot_response
+  //         ::= "QUOTAROOT" SP astring *(SP astring)
+  parseOneWordC (result); // skip mailbox name
+  skipWS (result);
+  if ( result.isEmpty() )
+    return;
+  QStringList roots;
+  int outlen = 1;
+  while ( outlen && !result.isEmpty() ) {
+    QCString word = parseLiteralC (result, false, false, &outlen);
+    roots.append (word);
+  }
+  lastResults.append( roots.join(" ") );
+}
+
 void imapParser::parseMyRights (parseString & result)
 {
   parseOneWordC (result); // skip mailbox name
