@@ -33,6 +33,7 @@
 #include "callback.h"
 #include "kmkernel.h"
 #include "kmmessage.h"
+#include "kmmsgpart.h"
 #include <libemailfunctions/email.h>
 #include <libkpimidentities/identity.h>
 #include <libkpimidentities/identitymanager.h>
@@ -50,23 +51,26 @@
 using namespace KMail;
 
 
-Callback::Callback( KMMessage* msg, KMReaderWin* readerWin ) 
+Callback::Callback( KMMessage* msg, KMReaderWin* readerWin )
   : mMsg( msg ), mReaderWin( readerWin ), mReceiverSet( false )
 {
 }
 
 bool Callback::mailICal( const QString& to, const QString iCal,
-                         const QString& subject ) const
+                         const QString& subject, const QString &status ) const
 {
   kdDebug(5006) << "Mailing message:\n" << iCal << endl;
-
   KMMessage *msg = new KMMessage;
   msg->initHeader();
-  msg->setHeaderField( "Content-Type",
-                       "text/calendar; method=reply; charset=\"utf-8\"" );
-  msg->setSubject( subject );
+  if ( GlobalSettings::self()->exchangeCompatibleInvitations() ) {
+    if ( status == QString("cancel") ) msg->setSubject( QString("Declined: %1").arg(subject).replace("Answer: ","") );
+    else if ( status == QString("tentative") ) msg->setSubject(QString("Tentative: %1").arg(subject).replace("Answer: ","") );
+    else if ( status == QString("accepted") ) msg->setSubject( QString("Accepted: %1").arg(subject).replace("Answer: ","") );
+    else msg->setSubject( subject );
+  } else {
+    msg->setSubject( subject );
+  }
   msg->setTo( to );
-  msg->setBody( iCal.utf8() );
   msg->setFrom( receiver() );
   /* We want the triggering mail to be moved to the trash once this one
    * has been sent successfully. Set a link header which accomplishes that. */
@@ -75,16 +79,18 @@ bool Callback::mailICal( const QString& to, const QString iCal,
   // Outlook will only understand the reply if the From: header is the
   // same as the To: header of the invitation message.
   KConfigGroup options( KMKernel::config(), "Groupware" );
-  if( !options.readBoolEntry( "LegacyMangleFromToHeaders", true ) ) {
+  if( !options.readBoolEntry( "LegacyMangleFromToHeaders", true ) )
+  {
     // Try and match the receiver with an identity
     const KPIM::Identity& identity =
       kmkernel->identityManager()->identityForAddress( receiver() );
-    if( identity != KPIM::Identity::null )
+    if( identity != KPIM::Identity::null() ) {
       // Identity found. Use this
       msg->setFrom( identity.fullEmailAddr() );
       msg->setHeaderField("X-KMail-Identity", QString::number( identity.uoid() ));
-      // Remove BCC from identity on ical invitations (https://intevation.de/roundup/kolab/issue474)
-      msg->setBcc( "" );
+    }
+    // Remove BCC from identity on ical invitations (https://intevation.de/roundup/kolab/issue474)
+    msg->setBcc( "" );
   }
 
   KMail::Composer * cWin = KMail::makeComposer();
@@ -93,6 +99,25 @@ bool Callback::mailICal( const QString& to, const QString iCal,
   cWin->slotWordWrapToggled( false );
   cWin->setSigningAndEncryptionDisabled( true );
 
+  if( GlobalSettings::self()->exchangeCompatibleInvitations() ) {
+    // For Exchange, send ical as attachment, with proper
+    // parameters
+    msg->setSubject( status );
+    msg->setCharset( "utf-8" );
+    KMMessagePart *msgPart = new KMMessagePart;
+    msgPart->setName( "cal.ics" );
+    // msgPart->setCteStr( attachCte ); // "base64" ?
+    msgPart->setBodyEncoded( iCal.utf8() );
+    msgPart->setTypeStr( "text" );
+    msgPart->setSubtypeStr( "calendar" );
+    msgPart->setParameter( "method", "reply" );
+    cWin->addAttach( msgPart );
+  } else {
+    msg->setHeaderField( "Content-Type",
+                         "text/calendar; method=reply; charset=\"utf-8\"" );
+    msg->setBody( iCal.utf8() );
+  }
+
   if ( options.readBoolEntry( "AutomaticSending", true ) ) {
     cWin->setAutoDeleteWindow(  true );
     cWin->slotSendNow();
@@ -112,32 +137,45 @@ QString Callback::receiver() const
   mReceiverSet = true;
 
   QStringList addrs = KPIM::splitEmailAddrList( mMsg->to() );
-  if( addrs.count() < 2 )
-    // Only one receiver, so that has to be us
-    mReceiver = mMsg->to();
-  else {
-    int found = 0;
-    for( QStringList::Iterator it = addrs.begin(); it != addrs.end(); ++it ) {
-      if( kmkernel->identityManager()->identityForAddress( *it ) !=
-          KPIM::Identity::null ) {
-	// Ok, this could be us
-        ++found;
-        mReceiver = *it;
-      }
+  int found = 0;
+  for( QStringList::Iterator it = addrs.begin(); it != addrs.end(); ++it ) {
+    if( kmkernel->identityManager()->identityForAddress( *it ) !=
+        KPIM::Identity::null() ) {
+      // Ok, this could be us
+      ++found;
+      mReceiver = *it;
+    }
+  }
+  QStringList ccaddrs = KPIM::splitEmailAddrList( mMsg->cc() );
+  for( QStringList::Iterator it = ccaddrs.begin(); it != ccaddrs.end(); ++it ) {
+    if( kmkernel->identityManager()->identityForAddress( *it ) !=
+        KPIM::Identity::null() ) {
+      // Ok, this could be us
+      ++found;
+      mReceiver = *it;
+    }
+  }
+  if( found != 1 ) {
+    bool ok;
+    QString selectMessage;
+    if (found == 0) {
+      selectMessage = i18n("<qt>None of your identities match the "
+          "receiver of this message,<br>please "
+          "choose which of the following addresses "
+          "is yours, if any:");
+    } else {
+      selectMessage = i18n("<qt>Several of your identities match the "
+          "receiver of this message,<br>please "
+          "choose which of the following addresses "
+          "is yours:");
     }
 
-    if( found != 1 ) {
-      bool ok;
-      mReceiver =
-        KInputDialog::getItem( i18n( "Select Address" ),
-                               i18n( "<qt>None of your identities match the "
-                                     "receiver of this message,<br>please "
-                                     "choose which of the following addresses "
-                                     "is yours:" ),
-                               addrs, 0, FALSE, &ok, kmkernel->mainWin() );
-      if( !ok )
-        mReceiver = QString::null;
-    }
+    mReceiver =
+      KInputDialog::getItem( i18n( "Select Address" ),
+          selectMessage,
+          addrs+ccaddrs, 0, FALSE, &ok, kmkernel->mainWin() );
+    if( !ok )
+      mReceiver = QString::null;
   }
 
   return mReceiver;
