@@ -87,6 +87,7 @@ namespace KMail {
       mAutoExpunge( true ),
       mHiddenFolders( false ),
       mOnlySubscribedFolders( false ),
+      mOnlyLocallySubscribedFolders( false ),
       mLoadOnDemand( true ),
       mListOnlyOpenFolders( false ),
       mProgressEnabled( false ),
@@ -94,6 +95,7 @@ namespace KMail {
       mPasswordDialogIsActive( false ),
       mACLSupport( true ),
       mAnnotationSupport( true ),
+      mQuotaSupport( true ),
       mSlaveConnected( false ),
       mSlaveConnectionError( false ),
       mCheckingSingleFolder( false ),
@@ -118,6 +120,7 @@ namespace KMail {
     mAutoExpunge = true;
     mHiddenFolders = false;
     mOnlySubscribedFolders = false;
+    mOnlyLocallySubscribedFolders = false;
     mLoadOnDemand = true;
     mListOnlyOpenFolders = false;
     mProgressEnabled = false;
@@ -132,10 +135,12 @@ namespace KMail {
     setAutoExpunge( i->autoExpunge() );
     setHiddenFolders( i->hiddenFolders() );
     setOnlySubscribedFolders( i->onlySubscribedFolders() );
+    setOnlyLocallySubscribedFolders( i->onlyLocallySubscribedFolders() );
     setLoadOnDemand( i->loadOnDemand() );
     setListOnlyOpenFolders( i->listOnlyOpenFolders() );
     setNamespaces( i->namespaces() );
     setNamespaceToDelimiter( i->namespaceToDelimiter() );
+    localBlacklistFromStringList( i->locallyBlacklistedFolders() );
   }
 
   unsigned short int ImapAccountBase::defaultPort() const {
@@ -164,6 +169,10 @@ namespace KMail {
     mOnlySubscribedFolders = show;
   }
 
+  void ImapAccountBase::setOnlyLocallySubscribedFolders( bool show ) {
+    mOnlyLocallySubscribedFolders = show;
+  }
+
   void ImapAccountBase::setLoadOnDemand( bool load ) {
     mLoadOnDemand = load;
   }
@@ -184,6 +193,7 @@ namespace KMail {
     setAutoExpunge( config.readBoolEntry( "auto-expunge", false ) );
     setHiddenFolders( config.readBoolEntry( "hidden-folders", false ) );
     setOnlySubscribedFolders( config.readBoolEntry( "subscribed-folders", false ) );
+    setOnlyLocallySubscribedFolders( config.readBoolEntry( "locally-subscribed-folders", false ) );
     setLoadOnDemand( config.readBoolEntry( "loadondemand", false ) );
     setListOnlyOpenFolders( config.readBoolEntry( "listOnlyOpenFolders", false ) );
     // read namespaces
@@ -213,6 +223,7 @@ namespace KMail {
     if ( !mOldPrefix.isEmpty() ) {
       makeConnection();
     }
+    localBlacklistFromStringList( config.readListEntry( "locallyUnsubscribedFolders" ) );
   }
 
   void ImapAccountBase::writeConfig( KConfig/*Base*/ & config ) /*const*/ {
@@ -221,6 +232,7 @@ namespace KMail {
     config.writeEntry( "auto-expunge", autoExpunge() );
     config.writeEntry( "hidden-folders", hiddenFolders() );
     config.writeEntry( "subscribed-folders", onlySubscribedFolders() );
+    config.writeEntry( "locally-subscribed-folders", onlyLocallySubscribedFolders() );
     config.writeEntry( "loadondemand", loadOnDemand() );
     config.writeEntry( "listOnlyOpenFolders", listOnlyOpenFolders() );
     QString data;
@@ -236,6 +248,7 @@ namespace KMail {
       key = "Namespace:" + it.key();
       config.writeEntry( key, it.data() );
     }
+    config.writeEntry( "locallyUnsubscribedFolders", locallyBlacklistedFolders() );
   }
 
   //
@@ -473,6 +486,35 @@ namespace KMail {
     if (mSlave) removeJob(job);
   }
 
+  //-----------------------------------------------------------------------------
+  // Do not remove imapPath, FolderDiaQuotaTab needs to call this with parent==0.
+  void ImapAccountBase::getStorageQuotaInfo( KMFolder* parent, const QString& imapPath )
+  {
+    if ( !mSlave ) return;
+    KURL url = getUrl();
+    url.setPath(imapPath);
+
+    QuotaJobs::GetStorageQuotaJob* job = QuotaJobs::getStorageQuota( mSlave, url );
+    jobData jd( url.url(), parent );
+    jd.cancellable = true;
+    insertJob(job, jd);
+
+    connect(job, SIGNAL(result(KIO::Job *)),
+            SLOT(slotGetStorageQuotaInfoResult(KIO::Job *)));
+  }
+
+  void ImapAccountBase::slotGetStorageQuotaInfoResult( KIO::Job* _job )
+  {
+    QuotaJobs::GetStorageQuotaJob* job = static_cast<QuotaJobs::GetStorageQuotaJob *>( _job );
+    JobIterator it = findJob( job );
+    if ( it == jobsEnd() ) return;
+    if ( job->error() && job->error() == KIO::ERR_UNSUPPORTED_ACTION )
+      setHasNoQuotaSupport();
+
+    KMFolder* folder = (*it).parent; // can be 0
+    emit receivedStorageQuotaInfo( folder, job, job->storageQuotaInfo() );
+    if (mSlave) removeJob(job);
+  }
 
   void ImapAccountBase::slotNoopTimeout()
   {
@@ -520,19 +562,19 @@ namespace KMail {
     if (aSlave != mSlave) return;
     handleError( errorCode, errorMsg, 0, QString::null, true );
     if ( mAskAgain )
-      makeConnection();
-    else {
-      if ( !mSlaveConnected ) {
-        mSlaveConnectionError = true;
-        resetConnectionList( this );
-        if ( mSlave )
-        {
-          KIO::Scheduler::disconnectSlave( slave() );
-          mSlave = 0;
-        }
+      if ( makeConnection() != ImapAccountBase::Error )
+        return;
+
+    if ( !mSlaveConnected ) {
+      mSlaveConnectionError = true;
+      resetConnectionList( this );
+      if ( mSlave )
+      {
+        KIO::Scheduler::disconnectSlave( slave() );
+        mSlave = 0;
       }
-      emit connectionResult( errorCode, errorMsg );
     }
+    emit connectionResult( errorCode, errorMsg );
   }
 
   //-----------------------------------------------------------------------------
@@ -850,7 +892,7 @@ namespace KMail {
     bool jobsKilled = true;
     switch( errorCode ) {
     case KIO::ERR_SLAVE_DIED: slaveDied(); killAllJobs( true ); break;
-    case KIO::ERR_COULD_NOT_LOGIN: // bad password
+    case KIO::ERR_COULD_NOT_AUTHENTICATE: // bad password
       mAskAgain = true;
       // fallthrough intended
     case KIO::ERR_CONNECTION_BROKEN:
@@ -859,6 +901,7 @@ namespace KMail {
       // These mean that we'll have to reconnect on the next attempt, so disconnect and set mSlave to 0.
       killAllJobs( true );
       break;
+    case KIO::ERR_COULD_NOT_LOGIN:
     case KIO::ERR_USER_CANCELED:
       killAllJobs( false );
       break;
@@ -1000,7 +1043,7 @@ namespace KMail {
     msg->deleteBodyParts();
     // make the parts and fill the mBodyPartList
     constructParts( stream, 1, 0, 0, msg->asDwMessage() );
-    if ( mBodyPartList.count() == 1 ) // we directly set the body later
+    if ( mBodyPartList.count() == 1 ) // we directly set the body later, at partsToLoad below
       msg->deleteBodyParts();
 
     if ( !as )
@@ -1026,6 +1069,12 @@ namespace KMail {
         ++partsToLoad;
       }
     }
+    // if the only body part is not text, part->loadPart() would return false
+    // and that part is never loaded, so make sure it loads.
+    // it seems that TEXT does load the single body part even if it is not text/*
+    if ( mBodyPartList.count() == 1 && partsToLoad == 0 )
+        partsToLoad = 1; // this causes the next test to succeed, and loads the whole message
+
     if ( (mBodyPartList.count() * 0.5) < partsToLoad )
     {
       // more than 50% of the parts have to be loaded anyway so it is faster
@@ -1314,6 +1363,40 @@ namespace KMail {
     return createImapPath( path, folderName );
   }
 
+
+  bool ImapAccountBase::locallySubscribedTo( const QString& imapPath )
+  {
+      return mLocalSubscriptionBlackList.find( imapPath ) == mLocalSubscriptionBlackList.end();
+  }
+
+  void ImapAccountBase::changeLocalSubscription( const QString& imapPath, bool subscribe )
+  {
+    if ( subscribe ) {
+      // find in blacklist and remove from it
+      mLocalSubscriptionBlackList.erase( imapPath );
+    } else {
+      // blacklist
+      mLocalSubscriptionBlackList.insert( imapPath );
+    }
+  }
+
+
+  QStringList ImapAccountBase::locallyBlacklistedFolders() const
+  {
+      QStringList list;
+      std::set<QString>::const_iterator it = mLocalSubscriptionBlackList.begin();
+      std::set<QString>::const_iterator end = mLocalSubscriptionBlackList.end();
+      for ( ; it != end ; ++it )
+        list.append( *it );
+      return list;
+  }
+
+  void ImapAccountBase::localBlacklistFromStringList( const QStringList &list )
+  {
+    for( QStringList::ConstIterator it = list.constBegin( ); it != list.constEnd( ); ++it )
+      mLocalSubscriptionBlackList.insert( *it );
+  }
+
 } // namespace KMail
 
 #include "imapaccountbase.moc"
