@@ -4,12 +4,12 @@
  * KMail is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License, version 2, as
  * published by the Free Software Foundation.
- * 
+ *
  * KMail is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
@@ -58,6 +58,7 @@ const unsigned int MaintenanceLimit = 1000;
 const char* const folderIndexDisabledKey = "fulltextIndexDisabled";
 }
 
+#ifdef HAVE_INDEXLIB
 static
 QValueList<int> vectorToQValueList( const std::vector<Q_UINT32>& input ) {
 	QValueList<int> res;
@@ -65,7 +66,6 @@ QValueList<int> vectorToQValueList( const std::vector<Q_UINT32>& input ) {
 	return res;
 }
 
-
 static
 std::vector<Q_UINT32> QValueListToVector( const QValueList<int>& input ) {
 	std::vector<Q_UINT32> res;
@@ -75,6 +75,7 @@ std::vector<Q_UINT32> QValueListToVector( const QValueList<int>& input ) {
 	}
 	return res;
 }
+#endif
 
 KMMsgIndex::KMMsgIndex( QObject* parent ):
 	QObject( parent, "index" ),
@@ -89,11 +90,11 @@ KMMsgIndex::KMMsgIndex( QObject* parent ):
 	//mSyncTimer( new QTimer( this ) ),
 	mSlowDown( false ) {
 	kdDebug( 5006 ) << "KMMsgIndex::KMMsgIndex()" << endl;
-	
-	connect( kmkernel->folderMgr(), SIGNAL( msgRemoved( KMFolder*, Q_UINT32 ) ), SLOT( slotRemoveMessage( KMFolder*, Q_UINT32 ) ) );
-	connect( kmkernel->folderMgr(), SIGNAL( msgAdded( KMFolder*, Q_UINT32 ) ), SLOT( slotAddMessage( KMFolder*, Q_UINT32 ) ) );
-	connect( kmkernel->dimapFolderMgr(), SIGNAL( msgRemoved( KMFolder*, Q_UINT32 ) ), SLOT( slotRemoveMessage( KMFolder*, Q_UINT32 ) ) );
-	connect( kmkernel->dimapFolderMgr(), SIGNAL( msgAdded( KMFolder*, Q_UINT32 ) ), SLOT( slotAddMessage( KMFolder*, Q_UINT32 ) ) );
+
+	connect( kmkernel->folderMgr(), SIGNAL( msgRemoved( KMFolder*, Q_UINT32 ) ), SLOT( slotRemoveMessage( Q_UINT32 ) ) );
+	connect( kmkernel->folderMgr(), SIGNAL( msgAdded( KMFolder*, Q_UINT32 ) ), SLOT( slotAddMessage( Q_UINT32 ) ) );
+	connect( kmkernel->dimapFolderMgr(), SIGNAL( msgRemoved( KMFolder*, Q_UINT32 ) ), SLOT( slotRemoveMessage( Q_UINT32 ) ) );
+	connect( kmkernel->dimapFolderMgr(), SIGNAL( msgAdded( KMFolder*, Q_UINT32 ) ), SLOT( slotAddMessage( Q_UINT32 ) ) );
 
 	connect( mTimer, SIGNAL( timeout() ), SLOT( act() ) );
 	//connect( mSyncTimer, SIGNAL( timeout() ), SLOT( syncIndex() ) );
@@ -226,7 +227,7 @@ void KMMsgIndex::setIndexingEnabled( KMFolder* folder, bool e ) {
 				//else  fall-through
 			case s_idle:
 			case s_processing:
-				
+
 			case s_error:
 			case s_disabled:
 				// nothing can be done
@@ -250,7 +251,7 @@ void KMMsgIndex::clear() {
 	mExisting.clear();
 	mState = s_disabled;
 	for ( std::set<KMFolder*>::const_iterator first = mOpenedFolders.begin(), past = mOpenedFolders.end(); first != past; ++first ) {
-		( *first )->close();
+		( *first )->close("msgindex");
 	}
 	mOpenedFolders.clear();
 	for ( std::vector<Search*>::const_iterator first = mSearches.begin(), past = mSearches.end(); first != past; ++first ) {
@@ -284,7 +285,7 @@ int KMMsgIndex::addMessage( Q_UINT32 serNum ) {
 	if ( !folder || idx == -1 ) return -1;
 	if ( !mOpenedFolders.count( folder ) ) {
 		mOpenedFolders.insert( folder );
-		folder->open();
+		folder->open("msgindex");
 	}
 	KMMessage* msg = folder->getMsg( idx );
 	/* I still don't know whether we should allow decryption or not.
@@ -324,7 +325,7 @@ void KMMsgIndex::act() {
 		mPendingFolders.pop_back();
 		if ( !mOpenedFolders.count( f ) ) {
 			mOpenedFolders.insert( f );
-			f->open();
+			f->open("msgindex");
 		}
 		const KMMsgDict* dict = KMMsgDict::instance();
 		KConfig* config = KMKernel::config();
@@ -344,7 +345,7 @@ void KMMsgIndex::act() {
 	for ( std::set<KMFolder*>::const_iterator first = mOpenedFolders.begin(), past = mOpenedFolders.end();
 			first != past;
 			++first ) {
-		( *first )->close();
+		( *first )->close("msgindex");
 	}
 	mOpenedFolders.clear();
 	mState = s_idle;
@@ -367,7 +368,7 @@ void KMMsgIndex::continueCreation() {
 
 void KMMsgIndex::create() {
 	kdDebug( 5006 ) << "KMMsgIndex::create()" << endl;
-	
+
 #ifdef HAVE_INDEXLIB
 	if ( !QFileInfo( mIndexPath ).exists() ) {
 		::mkdir( mIndexPath, S_IRWXU );
@@ -428,7 +429,7 @@ bool KMMsgIndex::startQuery( KMSearch* s ) {
 //}
 //
 //void KMMsgIndex::finishSync() {
-//	
+//
 //}
 
 void KMMsgIndex::removeSearch( QObject* destroyed ) {
@@ -481,10 +482,10 @@ bool KMMsgIndex::canHandleQuery( const KMSearchPattern* pat ) const {
 	return false;
 }
 
-void KMMsgIndex::slotAddMessage( KMFolder* folder, Q_UINT32 serNum ) {
+void KMMsgIndex::slotAddMessage( Q_UINT32 serNum ) {
 	kdDebug( 5006 ) << "KMMsgIndex::slotAddMessage( . , " << serNum << " )" << endl;
 	if ( mState == s_error || mState == s_disabled ) return;
-	
+
 	if ( mState == s_creating ) mAddedMsgs.push_back( serNum );
 	else mPendingMsgs.push_back( serNum );
 
@@ -492,7 +493,7 @@ void KMMsgIndex::slotAddMessage( KMFolder* folder, Q_UINT32 serNum ) {
 	scheduleAction();
 }
 
-void KMMsgIndex::slotRemoveMessage( KMFolder* folder, Q_UINT32 serNum ) {
+void KMMsgIndex::slotRemoveMessage( Q_UINT32 serNum ) {
 	kdDebug( 5006 ) << "KMMsgIndex::slotRemoveMessage( . , " << serNum << " )" << endl;
 	if ( mState == s_error || mState == s_disabled ) return;
 
@@ -511,7 +512,7 @@ void KMMsgIndex::scheduleAction() {
 void KMMsgIndex::removeMessage( Q_UINT32 serNum ) {
 	kdDebug( 5006 ) << "KMMsgIndex::removeMessage( " << serNum << " )" << endl;
 	if ( mState == s_error || mState == s_disabled ) return;
-	
+
 #ifdef HAVE_INDEXLIB
 	mIndex->remove_doc( QString::number( serNum ).latin1() );
 	++mMaintenanceCount;
