@@ -28,7 +28,10 @@ class KMMsgBase;
 class KMReaderWin;
 class partNode;
 namespace KIO { class Job; }
-namespace KMail { class Composer; }
+namespace KMail {
+  class Composer;
+  class FolderJob;
+}
 namespace GpgME { class Error; }
 namespace Kleo { class SpecialJob; }
 
@@ -733,18 +736,16 @@ public:
   KMCopyCommand( KMFolder* destFolder, KMMessage *msg );
 
 protected slots:
-  void slotMsgAdded( KMFolder*, Q_UINT32 );
+  void slotJobFinished( KMail::FolderJob *job );
 
-  void slotFolderComplete();
+  void slotFolderComplete( KMFolderImap*, bool success );
 
 private:
   virtual Result execute();
 
   KMFolder *mDestFolder;
   QPtrList<KMMsgBase> mMsgList;
-  // List of serial numbers that need to be loaded
-  // Ticked off as they come in via msgAdded signals.
-  QValueList<Q_UINT32> mWaitingForMsgs;
+  QValueList<KMail::FolderJob*> mPendingJobs;
 };
 
 namespace KPIM {
