@@ -111,6 +111,7 @@ public: // mailserviceimpl
    * From MailComposerIface
    */
   void send(int how);
+  void addAttachmentsAndSend(const KURL::List &urls, const QString &comment, int how);
   void addAttachment(KURL url,QString comment);
   void addAttachment(const QString &name,
                     const QCString &cte,
@@ -253,6 +254,7 @@ private slots:
   void slotPrint();
   void slotAttachFile();
   void slotInsertRecentFile(const KURL&);
+  void slotAttachedFile(const KURL&);
 public slots: // kmkernel, callback
   void slotSendNow();
 private slots:
@@ -266,6 +268,7 @@ private slots:
    * Returns true when saving was successful.
    */
   void slotSaveDraft();
+  void slotSaveTemplate();
   void slotNewComposer();
   void slotNewMailReader();
   void slotClose();
@@ -283,6 +286,7 @@ private slots:
   void slotPasteAsAttachment();
   void slotAddQuotes();
   void slotRemoveQuotes();
+  void slotAttachPNGImageData(const QByteArray &image);
 
   void slotMarkAll();
 
@@ -398,7 +402,7 @@ private slots:
   void slotAttachRemove();
   void slotAttachSave();
   void slotAttachProperties();
-
+  void slotAttachOpenWith();
 
   /**
    * Select an email from the addressbook and add it to the line
@@ -412,6 +416,7 @@ private slots:
 
   void slotToggleMarkup();
   void toggleMarkup(bool markup);
+  void htmlToolBarVisibilityChanged( bool visible );
 
 //  void slotSpellConfigure();
   void slotSpellcheckDone(int result);
@@ -451,7 +456,7 @@ private slots:
   void alignmentChanged( int );
 
 public: // kmkernel, attachmentlistview
-  void addAttach(const KURL url);
+  bool addAttach(const KURL url);
 
 public: // kmcommand
   /**
@@ -459,12 +464,6 @@ public: // kmcommand
    */
   void addAttach(const KMMessagePart* msgPart);
 
-private:
-  /**
-   * Add an image from the clipboard as attachment
-   */
-  void addImageFromClipboard();
-
 private:
   const KPIM::Identity & identity() const;
   uint identityUid() const;
@@ -473,6 +472,7 @@ private:
 
 signals:
   void applyChangesDone( bool );
+  void attachmentAdded( const KURL&, bool success );
 
 private:
   /**
@@ -545,9 +545,9 @@ private:
   virtual bool queryExit ();
 
   /**
-   * Open the attachment with the given index.
+   * Open the attachment with the given index and with ("Open with")
    */
-  void openAttach( int index );
+  void openAttach( int index, bool with );
 
   /**
    * View the attachment with the given index.
@@ -613,12 +613,19 @@ private:
    * This function is for example used to restore the unencrypted/unsigned
    * message text for editting.
    */
-   static void decryptOrStripOffCleartextSignature( QCString& );
+  static void decryptOrStripOffCleartextSignature( QCString& );
+
+  /**
+   * Save the message into the Drafts or Templates folder.
+   */
+  bool saveDraftOrTemplate( const QString &folderName, KMMessage *msg );
 
   /**
    * Send the message. Returns true if the message was sent successfully.
    */
-  void doSend( KMail::MessageSender::SendMethod method=KMail::MessageSender::SendDefault, bool saveInDrafts = false);
+  enum SaveIn { None, Drafts, Templates };
+  void doSend( KMail::MessageSender::SendMethod method=KMail::MessageSender::SendDefault,
+               KMComposeWin::SaveIn saveIn = KMComposeWin::None );
 
   /**
    * Returns the autosave interval in milliseconds (as needed for QTimer).
@@ -648,6 +655,14 @@ private:
    */
   static bool validateAddresses( QWidget * parent, const QString & addresses );
 
+  /**
+   * Sets the transport combobox to @p transport. If @p transport is empty
+   * then the combobox remains unchanged. If @p transport is neither a known transport
+   * nor a custom transport then the combobox is set to the default transport.
+   * @param transport the transport the combobox should be set to
+   */
+  void setTransport( const QString & transport );
+
 private slots:
    /**
     * Compress an attachemnt with the given index
@@ -687,7 +702,7 @@ private:
   QPtrList<QListViewItem> mAtmItemList;
   QPtrList<KMMessagePart> mAtmList;
   QPopupMenu *mAttachMenu;
-  int mOpenId, mViewId, mRemoveId, mSaveAsId, mPropertiesId;
+  int mOpenId, mOpenWithId, mViewId, mRemoveId, mSaveAsId, mPropertiesId;
   bool mAutoDeleteMsg;
   bool mSigningAndEncryptionExplicitlyDisabled;
   bool mLastSignActionState, mLastEncryptActionState;
@@ -739,6 +754,9 @@ private:
 
   QStringList mFolderNames;
   QValueList<QGuardedPtr<KMFolder> > mFolderList;
+  QMap<KIO::Job*, KURL> mAttachJobs;
+  KURL::List mAttachFilesPending;
+  int mAttachFilesSend;
 
 private:
   // helper method for slotInsert(My)PublicKey()
@@ -749,7 +767,8 @@ private:
 
   bool mSubjectTextWasSpellChecked;
 
-  QString addQuotesToText(const QString &inputText);
+  QString addQuotesToText( const QString &inputText );
+  QString removeQuotesFromText( const QString &inputText );
   // helper method for rethinkFields
   int calcColumnWidth(int which, long allShowing, int width);
 
@@ -762,7 +781,7 @@ private slots:
   void slotContinueDoSend( bool );
   void slotContinuePrint( bool );
   void slotContinueAutoSave( bool );
-  
+
   void slotEncryptChiasmusToggled( bool );
 
   /**
@@ -791,7 +810,7 @@ private:
 
   // These are for passing on methods over the applyChanges calls
   KMail::MessageSender::SendMethod mSendMethod;
-  bool mSaveInDrafts;
+  KMComposeWin::SaveIn mSaveIn;
 
   KToggleAction *mEncryptChiasmusAction;
   bool mEncryptWithChiasmus;
@@ -820,6 +839,15 @@ private:
 
   QPopupMenu *mActNowMenu;
   QPopupMenu *mActLaterMenu;
+
+  /** If the message in this composer has a cursor position set (for
+   *   instance because it comes from a template containing %CURSOR)
+   *   then we need to preserve that cursor position even when auto-
+   *   appending (or prepending) the signature during composer setup.
+   *   Set to true *once* (and only in setMsg() at that) to avoid
+   *   accidentally moving the cursor.
+   */
+  bool mPreserveUserCursorPosition;
 };
 
 #endif
