@@ -60,6 +60,8 @@ void KMEdit::contentsDragEnterEvent(QDragEnterEvent *e)
 {
     if (e->provides(MailListDrag::format()))
         e->accept(true);
+    else if (e->provides("image/png"))
+        e->accept();
     else
         return KEdit::contentsDragEnterEvent(e);
 }
@@ -68,6 +70,8 @@ void KMEdit::contentsDragMoveEvent(QDragMoveEvent *e)
 {
     if (e->provides(MailListDrag::format()))
         e->accept();
+    else if (e->provides("image/png"))
+        e->accept();
     else
         return KEdit::contentsDragMoveEvent(e);
 }
@@ -144,7 +148,7 @@ void KMEdit::contentsDropEvent(QDropEvent *e)
         QBuffer serNumBuffer(serNums);
         serNumBuffer.open(IO_ReadOnly);
         QDataStream serNumStream(&serNumBuffer);
-        unsigned long serNum;
+        Q_UINT32 serNum;
         KMFolder *folder = 0;
         int idx;
         QPtrList<KMMsgBase> messageList;
@@ -164,6 +168,9 @@ void KMEdit::contentsDropEvent(QDropEvent *e)
                                          identity, mComposer);
         command->start();
     }
+    else if( e->provides("image/png") ) {
+        emit attachPNGImageData(e->encodedData("image/png"));
+    }
     else if( KURLDrag::canDecode( e ) ) {
         KURL::List urlList;
         if( KURLDrag::decode( e, urlList ) ) {
@@ -248,8 +255,6 @@ void KMEdit::initializeAutoSpellChecking()
                                                 col1, col2, col3, col4,
                                                 mSpellConfig );
 
-  connect( mSpellChecker, SIGNAL(activeChanged(const QString &)),
-           mComposer, SLOT(slotStatusMessage(const QString &)));
   connect( mSpellChecker, SIGNAL(newSuggestions(const QString&, const QStringList&, unsigned int)),
            this, SLOT(addSuggestion(const QString&, const QStringList&, unsigned int)) );
 }
@@ -734,4 +739,14 @@ void KMEdit::slotSpellDone()
   }
 }
 
+void KMEdit::setCursorPositionFromStart( unsigned int pos ) {
+  unsigned int l = 0;
+  unsigned int c = 0;
+  posToRowCol( pos, l, c );
+  // kdDebug() << "Num lines: " << numLines() << endl;
+  // kdDebug() << "Position " << pos << " converted to " << l << ":" << c << endl;
+  setCursorPosition( l, c );
+  ensureCursorVisible();
+}
+
 #include "kmedit.moc"
