@@ -1,8 +1,20 @@
 /* -*- mode: C++ -*-
- * Virtual base class for mail folder
+ * kmail: KDE mail client
+ * Copyright (c) 1996-1998 Stefan Taferner <taferner@kde.org>
  *
- * Author: Stefan Taferner <taferner@kde.org>
- * This code is under GPL
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 #ifndef kmfolder_h
@@ -15,25 +27,26 @@
 #include "kmfoldertype.h"
 #include "kmmsginfo.h"
 #include "kmglobal.h"
+#include "kmkernel.h"
 #include "folderjob.h"
 using KMail::FolderJob;
 #include "mailinglist-magic.h"
 using KMail::MailingList;
+#include "kmaccount.h" // for AccountList
 
 #include "mimelib/string.h"
 
 #include <qptrvector.h>
 #include <sys/types.h>
 #include <stdio.h>
+#include <kshortcut.h>
 
 class KMMessage;
 class KMFolderDir;
-class KMAcctList;
-class KMMsgDict;
-class KMMsgDictREntry;
 class QTimer;
 class FolderStorage;
 class KMFolderTreeItem;
+class KMFolderJob;
 
 namespace KMail {
    class AttachmentStrategy;
@@ -45,7 +58,7 @@ typedef QValueList<Q_UINT32> SerNumList;
 /** Mail folder.
  * (description will be here).
  *
- * @sect Accounts:
+ * @section Accounts
  *   The accounts (of KMail) that are fed into the folder are
  *   represented as the children of the folder. They are only stored here
  *   during runtime to have a reference for which accounts point to a
@@ -55,17 +68,64 @@ typedef QValueList<Q_UINT32> SerNumList;
 class KMFolder: public KMFolderNode
 {
   Q_OBJECT
-  friend class KMFolderJob;
+  friend class ::KMFolderJob;
 public:
 
-
-  /** Usually a parent is given. But in some cases there is no
-    fitting parent object available. Then the name of the folder
-    is used as the absolute path to the folder file. */
+  /**
+   * Constructs a new Folder object.
+   * @param parent The directory in the folder storage hierarchy under which
+   * the folder's storage will be found or created.
+   * @param name If name of the folder. In case there is no parent directory, because
+   * the folder is free-standing (/var/spool/mail/foo), this is used for the full path to
+   * the folder's storage location.
+   * @param aFolderType The type of folder to create.
+   * @param withIndex Wether this folder has an index. No-index folders are
+   * those used by KMail internally, the Outbox, and those of local spool accounts,
+   * for example.
+   * @param exportedSernums whether this folder exports its serial numbers to
+   * the global MsgDict for lookup.
+   * @return A new folder instance.
+   */
   KMFolder( KMFolderDir* parent, const QString& name,
-            KMFolderType aFolderType );
+                KMFolderType aFolderType, bool withIndex = true,
+                bool exportedSernums = true );
   ~KMFolder();
 
+  /** Returns true if this folder is the inbox on the local disk */
+  bool isMainInbox() {
+    return this == KMKernel::self()->inboxFolder();
+  }
+  /** Returns true only if this is the outbox for outgoing mail */
+  bool isOutbox() {
+    return this == KMKernel::self()->outboxFolder();
+  }
+  /** Returns true if this folder is the sent-mail box of the local account,
+    or is configured to be the sent mail box of any of the users identities */
+  bool isSent() {
+    return KMKernel::self()->folderIsSentMailFolder( this );
+  }
+  /** Returns true if this folder is configured as a trash folder, locally or
+    for one of the accounts. */
+  bool isTrash() {
+    return KMKernel::self()->folderIsTrash( this );
+  }
+  /** Returns true if this folder is the drafts box of the local account,
+    or is configured to be the drafts box of any of the users identities */
+  bool isDrafts() {
+    return KMKernel::self()->folderIsDrafts( this );
+  }
+  /** Returns true if this folder is the templates folder of the local account,
+    or is configured to be the templates folder of any of the users identities */
+  bool isTemplates() {
+    return KMKernel::self()->folderIsTemplates( this );
+  }
+
+  void setAcctList( AccountList* list ) { mAcctList = list; }
+  AccountList* acctList() { return mAcctList; }
+
+  /** Returns TRUE if accounts are associated with this folder. */
+  bool hasAccounts() const { return (mAcctList != 0); }
+
   /** This is used by the storage to read the folder specific configuration */
   void readConfig( KConfig* config );
 
@@ -123,9 +183,6 @@ public:
   /** Checks if the message is already "gotten" with getMsg */
   bool isMessage(int idx);
 
-  /** Read a message and return a referece to a string */
-  QCString& getMsgString(int idx, QCString& mDest);
-
   /** Read a message and returns a DwString */
   DwString getDwString(int idx);
 
@@ -176,13 +233,19 @@ public:
       header from the mail. */
   int addMsgKeepUID(KMMessage* msg, int* index_return = 0);
 
+  /**
+   * Adds the given messages to the folder. Behaviour is identical
+   * to addMsg(msg)
+   */
+  int addMsg(QPtrList<KMMessage>&, QValueList<int>& index_return);
+
   /** Called by derived classes implementation of addMsg.
       Emits msgAdded signals */
   void emitMsgAddedSignals(int idx);
 
   /** Remove (first occurrence of) given message from the folder. */
-  void removeMsg(int i, bool imapQuiet = FALSE);
-  void removeMsg(QPtrList<KMMessage> msgList, bool imapQuiet = FALSE);
+  void removeMsg(int i, bool imapQuiet = false);
+  void removeMsg(QPtrList<KMMessage> msgList, bool imapQuiet = false);
 
   /** Delete messages in the folder that are older than days. Return the
    * number of deleted messages. */
@@ -215,20 +278,20 @@ public:
                          const KMMsgStatus newStatus,
                          int idx);
 
-  /** Open folder for access. Does not work if the parent is not set.
+  /** Open folder for access.
     Does nothing if the folder is already opened. To reopen a folder
     call close() first.
     Returns zero on success and an error code equal to the c-library
     fopen call otherwise (errno). */
-  int open();
+  int open(const char *owner);
 
   /** Check folder for permissions
     Returns zero if readable and writable. */
   int canAccess();
 
-  /** Close folder. If force is TRUE the files are closed even if
+  /** Close folder. If force is true the files are closed even if
     others still use it (e.g. other mail reader windows). */
-  void close(bool force=FALSE);
+  void close(const char *owner, bool force=false);
 
   /** fsync buffers to disk */
   void sync();
@@ -242,20 +305,15 @@ public:
   /** Mark all new and unread messages as read. */
   void markUnreadAsRead();
 
-  /** Create a new folder with the name of this object and open it.
-      Returns zero on success and an error code equal to the
-      c-library fopen call otherwise. */
-  int create(bool imap = FALSE);
-
   /** Removes the folder physically from disk and empties the contents
     of the folder in memory. Note that the folder is closed during this
     process, whether there are others using it or not.
-    @see KMFolder::removeContents */
+    see KMFolder::removeContents */
   void remove();
 
   /** Delete entire folder. Forces a close *but* opens the
     folder again afterwards. Returns errno(3) error code or zero on
-    success.  @see KMFolder::expungeContents */
+    success.  see KMFolder::expungeContents */
   int expunge();
 
   enum CompactOptions { CompactLater, CompactNow, CompactSilentlyNow };
@@ -271,14 +329,7 @@ public:
     on failure. */
   int rename(const QString& newName, KMFolderDir *aParent = 0);
 
-  /** Returns TRUE if a table of contents file is automatically created. */
-  bool autoCreateIndex() const;
-
-  /** Allow/disallow automatic creation of a table of contents file.
-    Default is TRUE. */
-  void setAutoCreateIndex(bool);
-
-  /** Returns TRUE if the table of contents is dirty. This happens when
+  /** Returns true if the table of contents is dirty. This happens when
     a message is deleted from the folder. The toc will then be re-created
     when the folder is closed. */
   bool dirty() const;
@@ -286,7 +337,7 @@ public:
   /** Change the dirty flag. */
   void setDirty(bool f);
 
-  /** Returns TRUE if the folder contains deleted messages */
+  /** Returns true if the folder contains deleted messages */
   bool needsCompacting() const;
   void setNeedsCompacting(bool f);
 
@@ -303,10 +354,10 @@ public:
   /** Is the folder read-only? */
   bool isReadOnly() const;
 
-  /** Returns TRUE if the folder is a kmail system folder. These are
-    the folders 'outbox', 'sent', 'trash'. The name of these
-    folders is nationalized in the folder display and they cannot have
-    accounts associated. Deletion is also forbidden. Etc. */
+  /** Returns true if the folder is a kmail system folder. These are
+    the folders 'inbox', 'outbox', 'sent', 'trash', 'drafts', 'templates'.
+    The name of these folders is nationalized in the folder display and
+    they cannot have accounts associated. Deletion is also forbidden. Etc. */
   bool isSystemFolder() const { return mIsSystemFolder; }
   void setSystemFolder(bool itIs) { mIsSystemFolder=itIs; }
 
@@ -321,13 +372,7 @@ public:
   /** URL of the node for visualization purposes. */
   virtual QString prettyURL() const;
 
-  /** Type of the folder. Inherited. */
-  const char* type() const;
-
-  /** Returns TRUE if accounts are associated with this folder. */
-  bool hasAccounts() const;
-
-  /** Returns TRUE if this folder is associated with a mailing-list. */
+  /** Returns true if this folder is associated with a mailing-list. */
   void setMailingListEnabled( bool enabled );
   bool isMailingListEnabled() const { return mMailingListEnabled; }
 
@@ -436,25 +481,6 @@ public:
     failure. */
   int writeIndex( bool createEmptyIndex = false );
 
-  /** Inserts messages into the message dictionary.  Might be called
-    during kernel initialization. */
-  void fillMsgDict(KMMsgDict *dict);
-
-  /** Writes the message serial number file. */
-  int writeMsgDict(KMMsgDict *dict = 0);
-
-  /** Touches the message serial number file. */
-  int touchMsgDict();
-
-  /** Append message to end of message serial number file. */
-  int appendtoMsgDict(int idx = -1);
-
-  /** Sets the reverse-dictionary for this folder. */
-  void setRDict(KMMsgDictREntry *rentry);
-
-  /** Returns the reverse-dictionary for this folder. */
-  KMMsgDictREntry *rDict() const;
-
   /** Set the status of the message at index @p idx to @p status. */
   void setStatus(int idx, KMMsgStatus status, bool toggle=false);
 
@@ -470,11 +496,6 @@ public:
 
   void removeJobs();
 
-  /** Convert "\r\n" line endings in "\n" line endings. The conversion
-      happens in place. Returns the length of the resulting string.
-  */
-  static size_t crlf2lf( char* str, const size_t strLen );
-
   void daysToExpire( int& unreadDays, int& readDays );
 
   /**
@@ -497,11 +518,29 @@ public:
   bool ignoreNewMail() const { return mIgnoreNewMail; }
   void setIgnoreNewMail( bool b ) { mIgnoreNewMail = b; }
 
+  const KShortcut &shortcut() const { return mShortcut; }
+  void setShortcut( const KShortcut& );
+
+  /** Returns true if this folder can be moved */
+  bool isMoveable() const;
+
+  /** Returns true if there is currently a move or copy operation going
+      on with this folder as target.
+  */
+  bool moveInProgress() const { return mMoveInProgress; }
+
+  /** Sets the move-in-progress flag. */
+  void setMoveInProgress( bool b ) { mMoveInProgress = b; }
+
 signals:
   /** Emitted when the status, name, or associated accounts of this
     folder changed. */
   void changed();
 
+  /** Emitted when the folder is closed for real - ticket holders should
+   * discard any messages */
+  void closed();
+
   /** Emitted when the contents of a folder have been cleared
      (new search in a search folder, for example) */
   void cleared();
@@ -516,11 +555,14 @@ signals:
   /** Emitted when the name of the folder changes. */
   void nameChanged();
 
+  /** Emitted when the shortcut associated with this folder changes. */
+  void shortcutChanged( KMFolder * );
+
   /** Emitted before a message is removed from the folder. */
   void msgRemoved(KMFolder*, Q_UINT32 sernum);
 
   /** Emitted after a message is removed from the folder. */
-  void msgRemoved(int idx,QString msgIdMD5, QString strippedSubjMD5);
+  void msgRemoved( int idx, QString msgIdMD5 );
   void msgRemoved(KMFolder*);
 
   /** Emitted when a message is added from the folder. */
@@ -542,6 +584,9 @@ signals:
   /** Emitted when a folder was removed */
   void removed(KMFolder*, bool);
 
+  /** Emitted when the variables for the config of the view have changed */
+  void viewConfigChanged();
+
 public slots:
   /** Incrementally update the index if possible else call writeIndex */
   int updateIndex();
@@ -554,18 +599,24 @@ public slots:
       from an IMAP server */
   void reallyAddCopyOfMsg(KMMessage* aMsg);
 
+private slots:
+  /** The type of contents of this folder changed. Do what is needed. */
+  void slotContentsTypeChanged( KMail::FolderContentsType type );
+
 private:
   FolderStorage* mStorage;
-  KMFolderDir* mParent;
   KMFolderDir* mChild;
   bool mIsSystemFolder;
+  bool mHasIndex :1;
+  bool mExportsSernums :1;
+  bool mMoveInProgress :1;
 
   /** nationalized label or QString::null (then name() should be used) */
   QString mLabel;
   QString mSystemLabel;
 
   /** Support for automatic expiry of old messages */
-  bool         mExpireMessages;          // TRUE if old messages are expired
+  bool         mExpireMessages;          // true if old messages are expired
   int          mUnreadExpireAge;         // Given in unreadExpireUnits
   int          mReadExpireAge;           // Given in readExpireUnits
   ExpireUnits  mUnreadExpireUnits;
@@ -582,6 +633,8 @@ private:
   bool                mMailingListEnabled;
   MailingList         mMailingList;
 
+  AccountList* mAcctList;
+
   uint mIdentity;
 
   /** name of the field that is used for "From" in listbox */
@@ -592,6 +645,25 @@ private:
 
   /** Should new mail in this folder be ignored? */
   bool mIgnoreNewMail;
+
+  /** shortcut associated with this folder or null, if none is configured. */
+  KShortcut mShortcut;
+};
+
+/**
+   RAII for KMFolder::open() / close().
+
+   Usage: const KMFolderCloser closer( folder );
+*/
+class KMFolderCloser {
+  KMFolder * f;
+  QString mOwner;
+public:
+  KMFolderCloser( const char *owner, KMFolder * folder ) : f( folder ), mOwner( owner ) {}
+  ~KMFolderCloser() {
+    if ( f ) f->close(mOwner.latin1());
+  }
+  KMFolder * folder() const { return f; }
 };
 
 #endif /*kmfolder_h*/
