@@ -28,7 +28,6 @@ using KMail::MaildirJob;
 #include <kstaticdeleter.h>
 #include <kmessagebox.h>
 
-#include <ctype.h>
 #include <dirent.h>
 #include <errno.h>
 #include <stdlib.h>
@@ -59,7 +58,7 @@ KMFolderMaildir::KMFolderMaildir(KMFolder* folder, const char* name)
 //-----------------------------------------------------------------------------
 KMFolderMaildir::~KMFolderMaildir()
 {
-  if (mOpenCount>0) close(true);
+  if (mOpenCount>0) close("~foldermaildir", true);
   if (kmkernel->undoStack()) kmkernel->undoStack()->folderDestroyed( folder() );
 }
 
@@ -97,7 +96,7 @@ int KMFolderMaildir::canAccess()
 }
 
 //-----------------------------------------------------------------------------
-int KMFolderMaildir::open()
+int KMFolderMaildir::open(const char *)
 {
   int rc = 0;
 
@@ -220,7 +219,7 @@ int KMFolderMaildir::create()
 
 
 //-----------------------------------------------------------------------------
-void KMFolderMaildir::close(bool aForced)
+void KMFolderMaildir::close(const char *, bool aForced)
 {
   if (mOpenCount <= 0) return;
   if (mOpenCount > 0) mOpenCount--;
@@ -406,7 +405,7 @@ if( fileD0.open( IO_WriteOnly ) ) {
   if ( !uidHeader.isEmpty() && stripUid )
     aMsg->removeHeaderField( "X-UID" );
 
-  msgText = aMsg->asString();
+  msgText = aMsg->asString(); // TODO use asDwString instead
   len = msgText.length();
 
   // Re-add the uid so that the take can make use of it, in case the
@@ -435,7 +434,7 @@ if( fileD0.open( IO_WriteOnly ) ) {
   if (!isOpened())
   {
     opened = true;
-    rc = open();
+    rc = open("maildir");
     kdDebug(5006) << "KMFolderMaildir::addMsg-open: " << rc << " of folder: " << label() << endl;
     if (rc) return rc;
   }
@@ -446,7 +445,7 @@ if( fileD0.open( IO_WriteOnly ) ) {
   if (moveInternal(tmp_file, new_loc, filename, aMsg->status()).isNull())
   {
     file.remove();
-    if (opened) close();
+    if (opened) close("maildir");
     return -1;
   }
 
@@ -544,7 +543,7 @@ if( fileD0.open( IO_WriteOnly ) ) {
   emitMsgAddedSignals(idx);
   needsCompact = true;
 
-  if (opened) close();
+  if (opened) close("maildir" );
 /*
 QFile fileD1( "testdat_xx-kmfoldermaildir-1" );
 if( fileD1.open( IO_WriteOnly ) ) {
@@ -594,29 +593,6 @@ DwString KMFolderMaildir::getDwString(int idx)
 }
 
 
-QCString& KMFolderMaildir::getMsgString(int idx, QCString& mDest)
-{
-  KMMsgInfo* mi = (KMMsgInfo*)mMsgList[idx];
-
-  assert(mi!=0);
-
-  QString abs_file(location() + "/cur/");
-  abs_file += mi->fileName();
-
-  if (QFile::exists(abs_file) == false)
-  {
-    kdDebug(5006) << "The " << abs_file << " file doesn't exist!" << endl;
-    return mDest;
-  }
-
-  QFileInfo fi( abs_file );
-  mDest.resize(fi.size()+2);
-  mDest = KPIM::kFileToString(abs_file, false, false);
-  size_t newMsgSize = KMail::Util::crlf2lf( mDest.data(), fi.size() );
-  mDest[newMsgSize] = '\0';
-  return mDest;
-}
-
 void KMFolderMaildir::readFileHeaderIntern(const QString& dir, const QString& file, KMMsgStatus status)
 {
   // we keep our current directory to restore it later
@@ -734,7 +710,7 @@ void KMFolderMaildir::readFileHeaderIntern(const QString& dir, const QString& fi
         int cidx = contentTypeStr.find( "charset=" );
         if ( cidx != -1 ) {
           charset = contentTypeStr.mid( cidx + 8 );
-          if ( charset[0] == '"' ) {
+          if ( !charset.isEmpty() && ( charset[0] == '"' ) ) {
             charset = charset.mid( 1 );
           }
           cidx = 0;
