@@ -33,7 +33,7 @@ class DwMessage;
 class KMMessagePart;
 class KMMsgInfo;
 class KMHeaders;
-class KMForwardCommand;
+class KMForwardDigestCommand;
 
 namespace KMime {
   class CharFreq;
@@ -66,7 +66,7 @@ namespace KMail {
 /** This is a Mime Message. */
 class KMMessage: public KMMsgBase, public KMail::ISubject
 {
-  friend class ::KMForwardCommand;    // needed for MIME Digest forward
+  friend class ::KMForwardDigestCommand; // needed for MIME Digest forward
 
 public:
   // promote some of KMMsgBase's methods to public:
@@ -162,7 +162,8 @@ public:
       is not stored in any folder. Marks this message as replied. */
   KMMessage* createReply( KMail::ReplyStrategy replyStrategy = KMail::ReplySmart,
                           QString selection=QString::null, bool noQuote=FALSE,
-                          bool allowDecryption=TRUE, bool selectionIsBody=FALSE);
+                          bool allowDecryption=TRUE, bool selectionIsBody=FALSE,
+                          const QString &tmpl = QString::null );
 
   /** Create a new message that is a redirect to this message, filling all
     required header fields with the proper values. The returned message
@@ -179,7 +180,7 @@ public:
   /** Create a new message that is a forward of this message, filling all
     required header fields with the proper values. The returned message
     is not stored in any folder. Marks this message as forwarded. */
-  KMMessage* createForward();
+  KMMessage* createForward( const QString &tmpl = QString::null );
 
   /** Create a new message that is a delivery receipt of this message,
       filling required header fileds with the proper values. The
@@ -207,7 +208,7 @@ public:
   /** Parse the string and create this message from it. */
   void fromDwString(const DwString& str, bool setStatus=FALSE);
   void fromString(const QCString& str, bool setStatus=FALSE);
-  void fromByteArray( const QByteArray & ba, bool setStatus=false );
+  void fromByteArray(const QByteArray & ba, bool setStatus=false);
 
   /** Return the entire message contents in the DwString. This function
       is *fast* even for large message since it does *not* involve a
@@ -229,7 +230,7 @@ public:
    * Return the message contents with the headers that should not be
    * sent stripped off.
    */
-  QCString asSendableString() const;
+  QByteArray asSendableString() const;
 
   /**
    * Return the message header with the headers that should not be
@@ -308,25 +309,29 @@ public:
 
   /** Get or set the 'ReplyTo' header field */
   QString replyTo() const;
-  void setReplyTo(const QString& aStr);
+  void setReplyTo( const QString &aStr );
   void setReplyTo(KMMessage*);
 
   /** Get or set the 'Cc' header field */
   QString cc() const;
-  void setCc(const QString& aStr);
+  void setCc( const QString &aStr );
   QString ccStrip() const;
 
   /** Get or set the 'Bcc' header field */
   QString bcc() const;
-  void setBcc(const QString& aStr);
+  void setBcc( const QString &aStr );
 
   /** Get or set the 'Fcc' header field */
   QString fcc() const;
-  void setFcc(const QString& aStr);
+  void setFcc( const QString &aStr );
 
   /** Get or set the 'Drafts' folder */
   QString drafts() const { return mDrafts; }
-  void setDrafts(const QString& aStr);
+  void setDrafts( const QString &aStr );
+
+  /** Get or set the 'Templates' folder */
+  QString templates() const { return mTemplates; }
+  void setTemplates( const QString &aStr );
 
   /** Get or set the 'From' header field */
   QString from() const;
@@ -521,6 +526,8 @@ public:
 
   /** Set the message body. Does not encode the body. */
   void setBody(const QCString& aStr);
+  void setBody(const DwString& aStr);
+  void setBody(const char* aStr); // avoid ambiguous calls
 
   /** Hack to enable structured body parts to be set as flat text... */
   void setMultiPartBody( const QCString & aStr );
@@ -811,7 +818,7 @@ public:
   bool transferInProgress() const;
   /** Set that the message shall not be deleted because it is still required */
   void setTransferInProgress(bool value, bool force = false);
-  
+
   /** Returns an mbox message separator line for this message, i.e. a
       string of the form
       "From local@domain.invalid Sat Jun 12 14:00:00 2004\n".
@@ -842,19 +849,33 @@ public:
   /** Return the textual content of the message as plain text,
       converting HTML to plain text if necessary. */
   QString asPlainText( bool stripSignature, bool allowDecryption ) const;
+
+  /** Get stored cursor position */
+  int getCursorPos() { return mCursorPos; };
+  /** Set cursor position as offset from message start */
+  void setCursorPos(int pos) { mCursorPos = pos; };
+
+  /* This is set in kmreaderwin if a message is being parsed to avoid
+     other parts of kmail (e.g. kmheaders) destroying the message. 
+     Parsing can take longer and can be async (in case of gpg mails) */
+  bool isBeingParsed() const { return mIsParsed; }
+  void setIsBeingParsed( bool t ) { mIsParsed = t; }
+
 private:
 
   /** Initialization shared by the ctors. */
-  void init();
+  void init( DwMessage* aMsg = 0 );
   /** Assign the values of @param other to this message. Used in the copy c'tor. */
   void assign( const KMMessage& other );
 
   QString mDrafts;
+  QString mTemplates;
   mutable DwMessage* mMsg;
   mutable bool       mNeedsAssembly :1;
   bool mDecodeHTML :1;
   bool mReadyToShow :1;
   bool mComplete :1;
+  bool mIsParsed : 1;
   static const KMail::HeaderStrategy * sHeaderStrategy;
   static QString sForwardStr;
   const QTextCodec * mOverrideCodec;
@@ -868,6 +889,7 @@ private:
   KMMsgMDNSentState mMDNSentState;
   KMMessage* mUnencryptedMsg;
   DwBodyPart* mLastUpdated;
+  int mCursorPos;
 };
 
 
