@@ -636,13 +636,13 @@ void KMReaderWin::createActions( KActionCollection * ac ) {
   mSelectEncodingAction->setItems( encodings );
   mSelectEncodingAction->setCurrentItem( 0 );
 
-  mMailToComposeAction = new KAction( i18n("New Message To..."), 0, this,
-                                      SLOT(slotMailtoCompose()), ac,
+  mMailToComposeAction = new KAction( i18n("New Message To..."), "mail_new",
+                                      0, this, SLOT(slotMailtoCompose()), ac,
                                       "mailto_compose" );
-  mMailToReplyAction = new KAction( i18n("Reply To..."), 0, this,
-				    SLOT(slotMailtoReply()), ac,
+  mMailToReplyAction = new KAction( i18n("Reply To..."), "mail_reply",
+                                    0, this, SLOT(slotMailtoReply()), ac,
 				    "mailto_reply" );
-  mMailToForwardAction = new KAction( i18n("Forward To..."),
+  mMailToForwardAction = new KAction( i18n("Forward To..."), "mail_forward",
                                       0, this, SLOT(slotMailtoForward()), ac,
                                       "mailto_forward" );
   mAddAddrBookAction = new KAction( i18n("Add to Address Book"),
@@ -1354,7 +1354,7 @@ void KMReaderWin::displayAboutPage()
 	     "<p style='margin-bottom: 0px'>&nbsp; &nbsp; The KMail Team</p>")
     .arg(KMAIL_VERSION) // KMail version
     .arg("help:/kmail/index.html") // KMail help:// URL
-    .arg("http://kmail.kde.org/") // KMail homepage URL
+    .arg("http://kontact.kde.org/kmail/") // KMail homepage URL
     .arg("1.8").arg("3.4"); // prior KMail and KDE version
 
   QString featureItems;
@@ -1504,7 +1504,14 @@ void KMReaderWin::parseMsg(KMMessage* aMsg)
 
   assert(aMsg!=0);
 
-  delete mRootNode;
+  aMsg->setIsBeingParsed( true );
+
+  if ( mRootNode && !mRootNode->processed() )
+  {
+    kdWarning() << "The root node is not yet processed! Danger!\n";
+    return;
+  } else
+    delete mRootNode;
   mRootNode = partNode::fromMessage( aMsg );
   const QCString mainCntTypeStr = mRootNode->typeString() + '/' + mRootNode->subTypeString();
 
@@ -1614,13 +1621,13 @@ kdDebug(5006) << "KMReaderWin  -  composing unencrypted message" << endl;
       unencryptedMessage->setParent( 0 );
       // because this did not work:
       /*
-      DwMessage dwMsg( DwString( aMsg->asString() ) );
+      DwMessage dwMsg( aMsg->asDwString() );
       dwMsg.Body() = DwBody( DwString( resultString.data() ) );
       dwMsg.Body().Parse();
       KMMessage* unencryptedMessage = new KMMessage( &dwMsg );
       */
-kdDebug(5006) << "KMReaderWin  -  resulting message:" << unencryptedMessage->asString() << endl;
-kdDebug(5006) << "KMReaderWin  -  attach unencrypted message to aMsg" << endl;
+      //kdDebug(5006) << "KMReaderWin  -  resulting message:" << unencryptedMessage->asString() << endl;
+      kdDebug(5006) << "KMReaderWin  -  attach unencrypted message to aMsg" << endl;
       aMsg->setUnencryptedMsg( unencryptedMessage );
       emitReplaceMsgByUnencryptedVersion = true;
     }
@@ -1642,6 +1649,8 @@ kdDebug(5006) << "KMReaderWin  -  attach unencrypted message to aMsg" << endl;
     showHideMimeTree( rootNodeCntType == DwMime::kTypeText &&
 		      rootNodeCntSubtype == DwMime::kSubtypePlain );
   }
+
+  aMsg->setIsBeingParsed( false );
 }
 
 
@@ -1670,21 +1679,10 @@ QString KMReaderWin::writeMessagePartToTempFile( KMMessagePart* aMsgPart,
     fileName = aMsgPart->name();
 
   //--- Sven's save attachments to /tmp start ---
-  KTempFile *tempFile = new KTempFile( QString::null,
-                                       "." + QString::number( aPartNum ) );
-  tempFile->setAutoDelete( true );
-  QString fname = tempFile->name();
-  delete tempFile;
+  QString fname = createTempDir( QString::number( aPartNum ) );
+  if ( fname.isEmpty() )
+    return QString();
 
-  if( ::access( QFile::encodeName( fname ), W_OK ) != 0 )
-    // Not there or not writable
-    if( ::mkdir( QFile::encodeName( fname ), 0 ) != 0
-        || ::chmod( QFile::encodeName( fname ), S_IRWXU ) != 0 )
-      return QString::null; //failed create
-
-  assert( !fname.isNull() );
-
-  mTempDirs.append( fname );
   // strip off a leading path
   int slashPos = fileName.findRev( '/' );
   if( -1 != slashPos )
@@ -1710,6 +1708,24 @@ QString KMReaderWin::writeMessagePartToTempFile( KMMessagePart* aMsgPart,
   return fname;
 }
 
+QString KMReaderWin::createTempDir( const QString &param )
+{
+  KTempFile *tempFile = new KTempFile( QString::null, "." + param );
+  tempFile->setAutoDelete( true );
+  QString fname = tempFile->name();
+  delete tempFile;
+
+  if( ::access( QFile::encodeName( fname ), W_OK ) != 0 )
+    // Not there or not writable
+    if( ::mkdir( QFile::encodeName( fname ), 0 ) != 0
+        || ::chmod( QFile::encodeName( fname ), S_IRWXU ) != 0 )
+      return QString::null; //failed create
+
+  assert( !fname.isNull() );
+
+  mTempDirs.append( fname );
+  return fname;
+}
 
 //-----------------------------------------------------------------------------
 void KMReaderWin::showVCard( KMMessagePart * msgPart ) {
