@@ -12,8 +12,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 
 */
 
@@ -24,6 +24,7 @@
 #include <qstringlist.h>
 #include <qmutex.h>
 
+#include <kdepimmacros.h>
 #include <klibloader.h>
 
 #include <kabc/addressee.h>
@@ -32,6 +33,8 @@
 #include <kio/global.h>
 #include <kio/authinfo.h>
 
+#include <libkcal/event.h>
+
 class KConfig;
 
 #define KMOBILE_MIMETYPE_DEVICE			"kdedevice/mobiledevice"
@@ -59,7 +62,7 @@ class KConfig;
  * @author Helge Deller <deller@kde.org>
  */
 
-class KMobileDevice : public KLibFactory
+class KDE_EXPORT KMobileDevice : public KLibFactory
 {
 Q_OBJECT
     friend class KMobileView;
@@ -167,7 +170,7 @@ public:
     const QString nameForCap(int cap) const;
 
     // returns an error string for the given error code
-    // @see KIO::buildErrorString()
+    // See KIO::buildErrorString()
     QString buildErrorString(KIO::Error err, const QString &errorText) const;
 
 public:
@@ -181,10 +184,9 @@ public:
     /*
      * Calendar support
      */
-    // TODO: TBD
     virtual int numCalendarEntries();
-//    virtual int readCalendarEntry( int index, <type> &entry );
-//    virtual int storeCalendarEntry( int index, <type> &entry );
+    virtual int readCalendarEntry( int index, KCal::Event &entry );
+    virtual int storeCalendarEntry( int index, const KCal::Event &entry );
 
     /*
      * Notes support
@@ -491,7 +493,7 @@ protected:
     QString m_deviceName;	// e.g. "Nokia 6310", "Opie"
     QString m_deviceRevision;	// e.g. "Revision 1.2" or "n/a"
     QString m_connectionName;	// e.g. "IRDA", "USB", "Cable", "gnokii", "gammu", ...
-    int m_caps;			// @see enum Capabilities
+    int m_caps;			// see enum Capabilities
     bool m_connected;
 
 private:
