@@ -418,8 +418,8 @@ void KNConfig::NntpAccountListWidget::slotAddBtnClicked()
   KNNntpAccount *acc = new KNNntpAccount();
 
   if(acc->editProperties(this)) {
-    a_ccManager->newAccount(acc);
-    acc->saveInfo();
+    if (a_ccManager->newAccount(acc))
+      acc->saveInfo();
   }
   else
     delete acc;
@@ -2154,7 +2154,10 @@ KNConfig::XHeaderConfDialog::~XHeaderConfDialog()
 
 QString KNConfig::XHeaderConfDialog::result()
 {
-  return QString("X-%1: %2").arg(n_ame->text()).arg(v_alue->text());
+  QString value = v_alue->text();
+  // just in case someone pastes a newline
+  value.replace( '\n', ' ' );
+  return QString( "X-%1: %2" ).arg( n_ame->text() ).arg( value );
 }
 
 
@@ -2381,27 +2384,27 @@ KNConfig::GroupCleanupWidget::GroupCleanupWidget( Cleanup *data, QWidget *parent
   connect( mExpEnabled, SIGNAL(toggled(bool)), SIGNAL(changed()) );
 
   mExpDays = new KIntSpinBox( 0, 99999, 1, 0, 10, mExpGroup );
-  mExpDays->setSuffix( i18n(" days") );
   QLabel *label = new QLabel( mExpDays, i18n("&Purge groups every:"), mExpGroup );
   grid->addWidget( label, 2, 0 );
   grid->addWidget( mExpDays, 2, 1, Qt::AlignRight );
   connect( mExpDays, SIGNAL(valueChanged(int)), SIGNAL(changed()) );
+  connect( mExpDays, SIGNAL(valueChanged(int)), SLOT(expDaysChanged(int)) );
   connect( mExpEnabled, SIGNAL(toggled(bool)), label, SLOT(setEnabled(bool)) );
   connect( mExpEnabled, SIGNAL(toggled(bool)), mExpDays, SLOT(setEnabled(bool)) );
 
   mExpReadDays = new KIntSpinBox( 0, 99999, 1, 0, 10, mExpGroup );
-  mExpReadDays->setSuffix( i18n(" days") );
   label = new QLabel( mExpReadDays, i18n("&Keep read articles:"), mExpGroup );
   grid->addWidget( label, 3, 0 );
   grid->addWidget( mExpReadDays, 3, 1, Qt::AlignRight );
   connect( mExpReadDays, SIGNAL(valueChanged(int)), SIGNAL(changed()) );
+  connect( mExpReadDays, SIGNAL(valueChanged(int)), SLOT(expReadDaysChanged(int)) );
 
   mExpUnreadDays = new KIntSpinBox( 0, 99999, 1, 0, 10, mExpGroup );
-  mExpUnreadDays->setSuffix( i18n(" days") );
   label = new QLabel( mExpUnreadDays, i18n("Keep u&nread articles:"), mExpGroup );
   grid->addWidget( label, 4, 0 );
   grid->addWidget( mExpUnreadDays, 4, 1, Qt::AlignRight );
   connect( mExpUnreadDays, SIGNAL(valueChanged(int)), SIGNAL(changed()) );
+  connect( mExpUnreadDays, SIGNAL(valueChanged(int)), SLOT(expUnreadDaysChanged(int)) );
 
   mExpUnavailable = new QCheckBox( i18n("&Remove articles that are not available on the server"), mExpGroup );
   grid->addMultiCellWidget( mExpUnavailable, 5, 5, 0, 1 );
@@ -2414,6 +2417,20 @@ KNConfig::GroupCleanupWidget::GroupCleanupWidget( Cleanup *data, QWidget *parent
   grid->setColStretch(1,1);
 }
 
+void KNConfig::GroupCleanupWidget::expDaysChanged(int value)
+{
+  mExpDays->setSuffix( i18n(" day", " days", value) );
+}
+
+void KNConfig::GroupCleanupWidget::expReadDaysChanged(int value)
+{
+  mExpReadDays->setSuffix( i18n(" day", " days", value) );
+}
+
+void KNConfig::GroupCleanupWidget::expUnreadDaysChanged(int value)
+{
+  mExpUnreadDays->setSuffix( i18n(" day", " days", value) );
+}
 
 void KNConfig::GroupCleanupWidget::load()
 {
@@ -2477,11 +2494,11 @@ KNConfig::CleanupWidget::CleanupWidget( QWidget *p, const char *n ) :
   foldersL->addMultiCellWidget(f_olderCB,1,1,0,1);
 
   f_olderDays=new KIntSpinBox(0, 99999, 1, 0, 10, foldersB);
-  f_olderDays->setSuffix(i18n(" days"));
   f_olderDaysL=new QLabel(f_olderDays,i18n("P&urge folders every:"), foldersB);
   foldersL->addWidget(f_olderDaysL,2,0);
   foldersL->addWidget(f_olderDays,2,1,Qt::AlignRight);
   connect(f_olderDays, SIGNAL(valueChanged(int)), SLOT(changed()));
+  connect(f_olderDays, SIGNAL(valueChanged(int)), SLOT(slotFolderDaysChanged(int)));
 
   foldersL->setColStretch(1,1);
 
@@ -2523,6 +2540,10 @@ void KNConfig::CleanupWidget::slotFolderCBtoggled(bool b)
   emit changed(true);
 }
 
+void KNConfig::CleanupWidget::slotFolderDaysChanged(int value)
+{
+  f_olderDays->setSuffix(i18n(" day", " days", value));
+}
 
 //END: Cleanup configuration widgets -----------------------------------------
 
