@@ -1,8 +1,7 @@
 /*
-    knmainwidget.h
-
     KNode, the KDE newsreader
     Copyright (c) 2003 Zack Rusin <zack@kde.org>
+    Copyright (c) 2004-2005 Volker Krause <volker.krause@rwth-aachen.de>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -10,7 +9,7 @@
     (at your option) any later version.
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software Foundation,
-    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, US
+    Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, US
 */
 #ifndef KNMAINWIDGET_H
 #define KNMAINWIDGET_H
@@ -22,6 +21,7 @@
 #include "resource.h"
 
 #include <qglobal.h>
+#include <kdepimmacros.h>
 
 class QListViewItem;
 
@@ -30,12 +30,13 @@ class KAccel;
 class KAction;
 class KToggleAction;
 class KSelectAction;
-class KStatusBarLabel;
+class KRSqueezedTextLabel;
 class KLineEdit;
 class KXMLGUIClient;
 
-class KNListView;
+class KNHeaderView;
 class KNCollectionView;
+class KNCollectionViewItem;
 class KNProgress;
 class KNConfigManager;
 class KNAccountManager;
@@ -51,13 +52,15 @@ class KNNetAccess;
 namespace Kpgp {
     class Module;
 }
-class KNArticleWidget;
+namespace KNode {
+  class ArticleWidget;
+}
 class KNArticle;
 class KNLocalArticle;
 class KNRemoteArticle;
 class KActionCollection;
 
-class KNMainWidget : public KDockArea, virtual public KNodeIface
+class KDE_EXPORT KNMainWidget : public KDockArea, virtual public KNodeIface
 {
   Q_OBJECT
 public:
@@ -89,10 +92,10 @@ public:
 
   /** access to GUI-elements */
   KNCollectionView* collectionView()const  { return c_olView; }
-  KNListView*       headerView()const      { return h_drView; }
-  KNArticleWidget*  articleView()const     { return a_rtView; }
-  KStatusBarLabel*  statusBarLabelGroup() const { return s_tatusGroup; }
-  KStatusBarLabel*  statusBarLabelFilter() const { return s_tatusFilter; }
+  KNHeaderView*       headerView()const      { return h_drView; }
+  KNode::ArticleWidget* articleViewer() const     { return mArticleViewer; }
+  KRSqueezedTextLabel*  statusBarLabelGroup() const { return s_tatusGroup; }
+  KRSqueezedTextLabel*  statusBarLabelFilter() const { return s_tatusFilter; }
   public: //The dcop interface
   // Implementation of KNodeIface
   /* Navigation */
@@ -179,18 +182,18 @@ protected:
   bool eventFilter(QObject *, QEvent *);
 
   // convenience methods...
-  void getSelectedArticles(QPtrList<KNArticle> &l);
-  void getSelectedArticles(QPtrList<KNRemoteArticle> &l);
-  void getSelectedThreads(QPtrList<KNRemoteArticle> &l);
-  void getSelectedArticles(QPtrList<KNLocalArticle> &l);
+  void getSelectedArticles( QValueList<KNArticle*> &l );
+  void getSelectedArticles( QValueList<KNRemoteArticle*> &l );
+  void getSelectedThreads( QValueList<KNRemoteArticle*> &l );
+  void getSelectedArticles( QValueList<KNLocalArticle*> &l );
   void closeCurrentThread();
 
   //GUI
   KAccel          *a_ccel;
   KNProgress      *p_rogBar;
-  KNArticleWidget *a_rtView;
+  KNode::ArticleWidget *mArticleViewer;
   KNCollectionView *c_olView;
-  KNListView      *h_drView;
+  KNHeaderView      *h_drView;
   KDockWidget     *c_olDock, *h_drDock, *a_rtDock;
   bool b_lockui;
   KToolBar        *q_uicksearch;
@@ -215,14 +218,13 @@ protected slots:
   void slotArticleSelectionChanged();
   void slotCollectionSelected(QListViewItem*);
   void slotCollectionRenamed(QListViewItem*);
-  void slotCollectionViewDrop(QDropEvent* e, QListViewItem* after);
-  void slotArticleRMB(QListViewItem *i, const QPoint &p, int);
-  void slotCollectionRMB(QListViewItem *i, const QPoint &p, int);
-  void slotArticleMMB(QListViewItem *item);
+  void slotCollectionViewDrop(QDropEvent* e, KNCollectionViewItem* after);
+  void slotArticleRMB(KListView*, QListViewItem *i, const QPoint &p);
+  void slotCollectionRMB(KListView*, QListViewItem *i, const QPoint &p);
+  /** Open selected article in own composer/reader window */
+  void slotOpenArticle(QListViewItem *item);
   void slotHdrViewSortingChanged(int i);
 
-  void slotReparented();
-
   //network slots
   void slotNetworkActive(bool b);
 
@@ -284,7 +286,6 @@ protected:
   KAction               *a_ctArtSortHeadersKeyb,
     *a_ctArtFilterKeyb,
     *a_ctArtSearch,
-    *a_ctArtFind,
     *a_ctArtRefreshList,
     *a_ctArtCollapseAll,
     *a_ctArtExpandAll,
@@ -325,14 +326,10 @@ protected:
   KAction *a_ctSwitchToGroupView,
     *a_ctSwitchToHeaderView,
     *a_ctSwitchToArticleViewer;
-    
+
 protected slots:
-  void slotNavNextArt();
-  void slotNavPrevArt();
   void slotNavNextUnreadArt();
   void slotNavNextUnreadThread();
-  void slotNavNextGroup();
-  void slotNavPrevGroup();
   void slotNavReadThrough();
 
   void slotAccProperties();
@@ -367,7 +364,6 @@ protected slots:
   void slotArtSortHeaders(int i);
   void slotArtSortHeadersKeyb();
   void slotArtSearch();
-  void slotArtFind();
   void slotArtRefreshList();
   void slotArtCollapseAll();
   void slotArtExpandAll();
@@ -408,8 +404,8 @@ protected slots:
   //--------------------------- </Actions> -----------------------------
 
 private:
-  KStatusBarLabel *s_tatusGroup; // widget used in the statusBar() for the group status
-  KStatusBarLabel *s_tatusFilter;
+  KRSqueezedTextLabel *s_tatusGroup; // widget used in the statusBar() for the group status
+  KRSqueezedTextLabel *s_tatusFilter;
   KXMLGUIClient *m_GUIClient;
 };
 
