@@ -1,7 +1,7 @@
 /*******************************************************************
  KNotes -- Notes for the KDE project
 
- Copyright (c) 1997-2005, The KNotes Developers
+ Copyright (c) 1997-2006, The KNotes Developers
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
@@ -40,7 +40,6 @@ class KActionMenu;
 class KGlobalAccel;
 class KXMLGUIFactory;
 class KXMLGUIBuilder;
-class KExtendedSocket;
 class KNotesAlarm;
 class KNotesResourceManager;
 
@@ -48,6 +47,10 @@ namespace KCal {
     class Journal;
 }
 
+namespace KNetwork {
+    class KServerSocket;
+}
+
 
 class KNotesApp : public QLabel, public KSessionManaged, virtual public KXMLGUIClient,
     virtual public KNotesAppIface
@@ -69,8 +72,20 @@ public:
     void setName( const QString& id, const QString& newName );
     void setText( const QString& id, const QString& newText );
 
+    QString fgColor( const QString& id ) const;
+    QString bgColor( const QString& id ) const;
+
+    void setColor( const QString& id, const QString& fgColor,
+                                      const QString& bgColor );
+
     QMap<QString,QString> notes() const;
 
+    int width( const QString& noteId ) const;
+    int height( const QString& noteId ) const;
+
+    void move( const QString& noteId, int x, int y ) const;
+    void resize( const QString& noteId, int width, int height ) const;
+
     void sync( const QString& app );
     bool isNew( const QString& app, const QString& id ) const;
     bool isModified( const QString& app, const QString& id ) const;
@@ -127,7 +142,7 @@ private:
     KNotesResourceManager *m_manager;
 
     KNotesAlarm     *m_alarm;
-    KExtendedSocket *m_listener;
+    KNetwork::KServerSocket   *m_listener;
 
     QDict<KNote>    m_noteList;
     KNoteActionList m_noteActions;
