@@ -34,6 +34,7 @@ class DCOPClient;
 class DCOPObject;
 class KAboutData;
 class KAction;
+class KConfig;
 class QWidget;
 namespace KParts { class ReadOnlyPart; }
 
@@ -41,7 +42,7 @@ namespace KParts { class ReadOnlyPart; }
   Increase this version number whenever you make a change
   in the API.
  */
-#define KONTACT_PLUGIN_VERSION 4
+#define KONTACT_PLUGIN_VERSION 5
 
 namespace Kontact
 {
@@ -231,6 +232,16 @@ class KDE_EXPORT Plugin : public QObject, virtual public KXMLGUIClient
     */
     virtual void processDropEvent( QDropEvent * ) {}
 
+    /**
+     * Session management: read properties
+     */
+    virtual void readProperties( KConfig * ) {}
+
+    /**
+     * Session management: save properties
+     */
+    virtual void saveProperties( KConfig * ) {}
+
     Core *core() const;
 
   public slots:
