@@ -17,17 +17,18 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #include <qcombobox.h>
 #include <qhbox.h>
 #include <qimage.h>
 #include <qobjectlist.h>
-#include <qsplitter.h>
-#include <qwhatsthis.h>
-#include <qtimer.h>
 #include <qprogressbar.h>
+#include <qpushbutton.h>
+#include <qsplitter.h>
+#include <qtimer.h>
+#include <qwhatsthis.h>
 
 #include <dcopclient.h>
 #include <kapplication.h>
@@ -63,6 +64,7 @@
 #include <krun.h>
 #include <kaboutdata.h>
 #include <kmenubar.h>
+#include <kstdaccel.h>
 
 #include "aboutdialog.h"
 #include "iconsidepane.h"
@@ -75,10 +77,27 @@
 
 using namespace Kontact;
 
+class SettingsDialogWrapper : public KSettings::Dialog
+{
+  public:
+    SettingsDialogWrapper( ContentInListView content, QWidget * parent = 0 )
+      : KSettings::Dialog( content, parent, 0 )
+    {
+    }
+
+
+    void fixButtonLabel( QWidget *widget )
+    {
+      QObject *object = widget->child( "KJanusWidget::buttonBelowList" );
+      QPushButton *button = static_cast<QPushButton*>( object );
+      if ( button )
+        button->setText( i18n( "Select Components..." ) );
+    }
+};
+
 MainWindow::MainWindow()
   : Kontact::Core(), mTopWidget( 0 ), mSplitter( 0 ),
-    mCurrentPlugin( 0 ), mAboutDialog( 0 ), mReallyClose( false ),
-    mStartupCompleted( false )
+    mCurrentPlugin( 0 ), mAboutDialog( 0 ), mReallyClose( false )
 {
   // Set this to be the group leader for all subdialogs - this means
   // modal subdialogs will only affect this dialog, not the other windows
@@ -130,10 +149,9 @@ void MainWindow::initObject()
 
   loadPlugins();
 
-  if ( mSidePane )
-  {
-      mSidePane->updatePlugins();
-      plugActionList( "navigator_actionlist", mSidePane->actions() );
+  if ( mSidePane ) {
+    mSidePane->updatePlugins();
+    plugActionList( "navigator_actionlist", mSidePane->actions() );
   }
 
   KSettings::Dispatcher::self()->registerInstance( instance(), this,
@@ -154,9 +172,6 @@ void MainWindow::initObject()
   // launch commandline specified module if any
   activatePluginModule();
 
-  // keep this above the lastVersionSeen check!
-  mStartupCompleted = true;
-
   if ( Prefs::lastVersionSeen() == kapp->aboutData()->version() ) {
     selectPlugin( mCurrentPlugin );
   }
@@ -214,7 +229,7 @@ void MainWindow::initWidgets()
   QValueList<int> sizes;
   sizes << 0;
   mSplitter->setSizes(sizes);
- 
+
   mSidePane->setActionCollection( actionCollection() );
 
   connect( mSidePane, SIGNAL( pluginSelected( Kontact::Plugin * ) ),
@@ -301,13 +316,9 @@ void MainWindow::setupActions()
 {
   KStdAction::quit( this, SLOT( slotQuit() ), actionCollection() );
   mNewActions = new KToolBarPopupAction( KGuiItem( i18n( "New" ), "" ),
-                                         KShortcut(), this, SLOT( slotNewClicked() ),
+                                         KStdAccel::shortcut(KStdAccel::New), this, SLOT( slotNewClicked() ),
                                          actionCollection(), "action_new" );
 
-  new KAction( i18n( "Select Components..." ), "configure", 0, this,
-               SLOT( slotSelectComponents() ),
-               actionCollection(), "settings_select_components" );
-
   new KAction( i18n( "Configure Kontact..." ), "configure", 0, this, SLOT( slotPreferences() ),
                actionCollection(), "settings_configure_kontact" );
 
@@ -437,6 +448,16 @@ bool MainWindow::removePlugin( const KPluginInfo *info )
       delete plugin; // removes the part automatically
       mPlugins.remove( it );
 
+      if ( mCurrentPlugin == 0 ) {
+        PluginList::Iterator it;
+        for ( it = mPlugins.begin(); it != mPlugins.end(); ++it ) {
+          if ( (*it)->showInSideBar() ) {
+            selectPlugin( *it );
+            return true;
+          }
+        }
+      }
+
       return true;
     }
 
@@ -545,10 +566,8 @@ void MainWindow::selectPlugin( Kontact::Plugin *plugin )
   Q_ASSERT( view );
 
   if ( view ) {
-    if ( mStartupCompleted ) {
-      mPartsStack->raiseWidget( view );
-      view->show();
-    }
+    mPartsStack->raiseWidget( view );
+    view->show();
 
     if ( mFocusWidgets.contains( plugin->identifier() ) ) {
       focusWidget = mFocusWidgets[ plugin->identifier() ];
@@ -559,7 +578,7 @@ void MainWindow::selectPlugin( Kontact::Plugin *plugin )
 
     mCurrentPlugin = plugin;
     KAction *action = plugin->newActions()->first();
-    
+
     createGUI( plugin->part() );
 
     setCaption( i18n( "Plugin dependent window title" ,"%1 - Kontact" ).arg( plugin->title() ) );
@@ -665,28 +684,31 @@ void MainWindow::slotQuit()
 
 void MainWindow::slotPreferences()
 {
-  static KSettings::Dialog *dlg = 0;
+  static SettingsDialogWrapper *dlg = 0;
   if ( !dlg ) {
-    dlg = new KSettings::Dialog( KSettings::Dialog::Configurable, this );
-    dlg->addPluginInfos( mPluginInfos );
+    dlg = new SettingsDialogWrapper( KSettings::Dialog::Configurable, this );
+
+    // do not show settings of components running standalone
+    QValueList<KPluginInfo*> filteredPlugins = mPluginInfos;
+    PluginList::ConstIterator it;
+    for ( it = mPlugins.begin(); it != mPlugins.end(); ++it )
+      if ( (*it)->isRunningStandalone() ) {
+        QValueList<KPluginInfo*>::ConstIterator infoIt;
+        for ( infoIt = filteredPlugins.begin(); infoIt != filteredPlugins.end(); ++infoIt ) {
+          if ( (*infoIt)->pluginName() == (*it)->identifier() ) {
+            filteredPlugins.remove( *infoIt );
+            break;
+          }
+        }
+      }
+
+    dlg->addPluginInfos( filteredPlugins );
     connect( dlg, SIGNAL( pluginSelectionChanged() ),
              SLOT( pluginsChanged() ) );
   }
 
   dlg->show();
-}
-
-void MainWindow::slotSelectComponents()
-{
-  static KSettings::ComponentsDialog *dlg = 0;
-  if ( !dlg ) {
-    dlg = new KSettings::ComponentsDialog( this );
-    dlg->setPluginInfos( mPluginInfos );
-    connect( dlg, SIGNAL( okClicked() ), SLOT( pluginsChanged() ) );
-    connect( dlg, SIGNAL( applyClicked() ), SLOT( pluginsChanged() ) );
-  }
-
-  dlg->show();
+  dlg->fixButtonLabel( this );
 }
 
 int MainWindow::startServiceFor( const QString& serviceType,
@@ -783,6 +805,46 @@ void MainWindow::slotOpenUrl( const KURL &url )
     new KRun( url, this );
 }
 
+void MainWindow::readProperties( KConfig *config )
+{
+  Core::readProperties( config );
+
+  QStringList activePlugins = config->readListEntry( "ActivePlugins" );
+  QValueList<Plugin*>::ConstIterator it = mPlugins.begin();
+  QValueList<Plugin*>::ConstIterator end = mPlugins.end();
+  for ( ; it != end; ++it ) {
+    Plugin *plugin = *it;
+    if ( !plugin->isRunningStandalone() ) {
+      QStringList::ConstIterator activePlugin = activePlugins.find( plugin->identifier() );
+      if ( activePlugin != activePlugins.end() ) {
+        plugin->readProperties( config );
+      }
+    }
+  }
+}
+
+void MainWindow::saveProperties( KConfig *config )
+{
+  Core::saveProperties( config );
+
+  QStringList activePlugins;
+
+  KPluginInfo::List::Iterator it = mPluginInfos.begin();
+  KPluginInfo::List::Iterator end = mPluginInfos.end();
+  for ( ; it != end; ++it ) {
+    KPluginInfo *info = *it;
+    if ( info->isPluginEnabled() ) {
+      Plugin *plugin = pluginFromInfo( info );
+      if ( plugin ) {
+        activePlugins.append( plugin->identifier() );
+        plugin->saveProperties( config );
+      }
+    }
+  }
+
+  config->writeEntry( "ActivePlugins", activePlugins );
+}
+
 bool MainWindow::queryClose()
 {
   if ( kapp->sessionSaving() || mReallyClose )
