@@ -18,7 +18,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -139,11 +139,23 @@ class KDE_EXPORT ActionManager : public QObject, public KCalendarIface
     void closeURL();
     /** Get current URL as QString */
     QString getCurrentURLasString() const;
-    /** Delete event with the given unique id from current calendar. */
-    virtual bool deleteIncidence( const QString& uid );
+    /**
+      Delete the incidence with the given unique id from current calendar.
+      @param uid UID of the incidence to delete.
+      @param force If true, all recurrences and sub-todos (if applicable) will be
+                         deleted without prompting for confirmation.
+    */
+    virtual bool deleteIncidence( const QString& uid, bool force = false );
 
     bool editIncidence( const QString& uid );
 
+    /**
+      Add an incidence to the active calendar.
+      @param ical A calendar in iCalendar format containing the incidence.
+    */
+
+    bool addIncidence( const QString& ical );
+
     //// Implementation of the DCOP interface
     virtual ResourceRequestReply resourceRequest( const QValueList<QPair<QDateTime, QDateTime> >& busy,
                                                   const QCString& resource,
@@ -252,7 +264,7 @@ class KDE_EXPORT ActionManager : public QObject, public KCalendarIface
     void file_open( const KURL &url );
 
     /** import a calendar from another program like ical. */
-    void file_import();
+    void file_icalimport();
 
     /** open a calendar and add the contents to the current calendar. */
     void file_merge();
@@ -353,10 +365,12 @@ class KDE_EXPORT ActionManager : public QObject, public KCalendarIface
     KToggleAction *mTodoViewShowAction;
     KToggleAction *mResourceViewShowAction;
     KToggleAction *mEventViewerShowAction;
+//     KToggleAction *mToggleAlarmAction;
 
     KAction *mShowIncidenceAction;
     KAction *mEditIncidenceAction;
     KAction *mDeleteIncidenceAction;
+//     KAction *mAssignResourceAction;
 
     KAction *mCutAction;
     KAction *mCopyAction;
