@@ -2,6 +2,7 @@
     This file is part of KOrganizer.
 
     Copyright (c) 2001 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,7 +16,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -24,23 +25,24 @@
 
 #include "koeditordetails.h"
 
-#include <qtooltip.h>
-#include <qfiledialog.h>
-#include <qlayout.h>
-#include <qvbox.h>
 #include <qbuttongroup.h>
-#include <qvgroupbox.h>
-#include <qwidgetstack.h>
+#include <qcheckbox.h>
+#include <qcombobox.h>
 #include <qdatetime.h>
 #include <qdragobject.h>
-#include <qcombobox.h>
-#include <qlineedit.h>
-#include <qlabel.h>
-#include <qcheckbox.h>
-#include <qpushbutton.h>
+#include <qfiledialog.h>
 #include <qgroupbox.h>
+#include <qlabel.h>
+#include <qlayout.h>
+#include <qlineedit.h>
+#include <qpushbutton.h>
 #include <qradiobutton.h>
 #include <qregexp.h>
+#include <qtooltip.h>
+#include <qvbox.h>
+#include <qvgroupbox.h>
+#include <qwhatsthis.h>
+#include <qwidgetstack.h>
 
 #include <kdebug.h>
 #include <klocale.h>
@@ -55,7 +57,7 @@
 #include <kabc/stdaddressbook.h>
 #endif
 #include <libkdepim/kvcarddrag.h>
-#include <libkdepim/email.h>
+#include <libemailfunctions/email.h>
 
 #include <libkcal/incidence.h>
 
@@ -67,13 +69,13 @@
 #include "kocore.h"
 
 template <>
-CustomListViewItem<class Attendee *>::~CustomListViewItem()
+CustomListViewItem<KCal::Attendee *>::~CustomListViewItem()
 {
   delete mData;
 }
 
 template <>
-void CustomListViewItem<class Attendee *>::updateItem()
+void CustomListViewItem<KCal::Attendee *>::updateItem()
 {
   setText(0,mData->name());
   setText(1,mData->email());
@@ -188,13 +190,35 @@ KOEditorDetails::KOEditorDetails( int spacing, QWidget *parent,
   // readEvent will delete it and set another label text instead, if the user
   // isn't the organizer.
   // Note that the i18n text below is duplicated in readEvent
+  QString whatsThis = i18n("Sets the identity corresponding to "
+		  	   "the organizer of this to-do or event. "
+			   "Identities can be set in the 'Personal' "
+			   "section of the KOrganizer configuration, or in the "
+			   "'Security & Privacy'->'Password & User Account' "
+			   "section of the KDE Control Center. In addition, "
+			   "identities are gathered from your KMail settings "
+			   "and from your address book. If you choose "
+			   "to set it globally for KDE in the Control Center, "
+			   "be sure to check 'Use email settings from "
+			   "Control Center' in the 'Personal' section of the "
+			   "KOrganizer configuration.");
   mOrganizerLabel = new QLabel( i18n( "Identity as organizer:" ),
                                 mOrganizerHBox );
   mOrganizerCombo = new QComboBox( mOrganizerHBox );
+  QWhatsThis::add( mOrganizerLabel, whatsThis );
+  QWhatsThis::add( mOrganizerCombo, whatsThis );
   fillOrganizerCombo();
   mOrganizerHBox->setStretchFactor( mOrganizerCombo, 100 );
 
   mListView = new KOAttendeeListView( this, "mListView" );
+  QWhatsThis::add( mListView,
+		   i18n("Displays information about current attendees. "
+		   	"To edit an attendee, select it in this list "
+			"and modify the values in the area below. "
+		   	"Clicking on a column title will sort the list "
+			"according to that column. The RSVP column "
+			"indicates whether or not a response is requested "
+			"from the attendee.") );
   mListView->addColumn( i18n("Name"), 200 );
   mListView->addColumn( i18n("Email"), 200 );
   mListView->addColumn( i18n("Role"), 60 );
@@ -212,38 +236,52 @@ KOEditorDetails::KOEditorDetails( int spacing, QWidget *parent,
            SLOT( insertAttendee( Attendee * ) ) );
 #endif
 
+  whatsThis = i18n("Edits the name of the attendee selected in the list "
+  		   "above, or adds a new attendee if there are no attendees"
+		   "in the list.");
   QLabel *attendeeLabel = new QLabel( this );
+  QWhatsThis::add( attendeeLabel, whatsThis );
   attendeeLabel->setText( i18n("Na&me:") );
 
   mNameEdit = new KPIM::AddresseeLineEdit( this );
+  QWhatsThis::add( mNameEdit, whatsThis );
   mNameEdit->setClickMessage( i18n("Click to add a new attendee") );
   attendeeLabel->setBuddy( mNameEdit );
   mNameEdit->installEventFilter( this );
   connect( mNameEdit, SIGNAL( textChanged( const QString & ) ),
            SLOT( updateAttendeeItem() ) );
 
-  mUidEdit = new QLineEdit( 0 );
-  mUidEdit->setText( "" );
-
+  whatsThis = i18n("Edits the role of the attendee selected "
+  		   "in the list above.");
   QLabel *attendeeRoleLabel = new QLabel( this );
+  QWhatsThis::add( attendeeRoleLabel, whatsThis );
   attendeeRoleLabel->setText( i18n("Ro&le:") );
 
   mRoleCombo = new QComboBox( false, this );
+  QWhatsThis::add( mRoleCombo, whatsThis );
   mRoleCombo->insertStringList( Attendee::roleList() );
   attendeeRoleLabel->setBuddy( mRoleCombo );
   connect( mRoleCombo, SIGNAL( activated( int ) ),
            SLOT( updateAttendeeItem() ) );
 
+  whatsThis = i18n("Edits the current attendance status of the attendee "
+  		   "selected in the list above.");
   QLabel *statusLabel = new QLabel( this );
+  QWhatsThis::add( statusLabel, whatsThis );
   statusLabel->setText( i18n("Stat&us:") );
 
   mStatusCombo = new QComboBox( false, this );
+  QWhatsThis::add( mStatusCombo, whatsThis );
   mStatusCombo->insertStringList( Attendee::statusList() );
   statusLabel->setBuddy( mStatusCombo );
   connect( mStatusCombo, SIGNAL( activated( int ) ),
            SLOT( updateAttendeeItem() ) );
 
   mRsvpButton = new QCheckBox( this );
+  QWhatsThis::add( mRsvpButton,
+		   i18n("Edits whether to send an email to the attendee "
+			"selected in the list above to request "
+			"a response concerning attendance.") );
   mRsvpButton->setText( i18n("Re&quest response") );
   connect( mRsvpButton, SIGNAL( clicked() ), SLOT( updateAttendeeItem() ) );
 
@@ -251,15 +289,29 @@ KOEditorDetails::KOEditorDetails( int spacing, QWidget *parent,
   QVBoxLayout *buttonLayout = new QVBoxLayout( buttonBox );
 
   QPushButton *newButton = new QPushButton( i18n("&New"), buttonBox );
+  QWhatsThis::add( newButton,
+		   i18n("Adds a new attendee to the list. Once the "
+		   	"attendee is added, you will be able to "
+			"edit the attendee's name, role, attendance "
+			"status, and whether or not the attendee is required "
+			"to respond to the invitation. To select an attendee "
+			"from your addressbook, click the 'Select Addressee' "
+			"button instead.") );
   buttonLayout->addWidget( newButton );
   connect( newButton, SIGNAL( clicked() ), SLOT( addNewAttendee() ) );
 
   mRemoveButton = new QPushButton( i18n("&Remove"), buttonBox );
+  QWhatsThis::add( mRemoveButton,
+		   i18n("Removes the attendee selected in "
+		   	"the list above.") );
   buttonLayout->addWidget( mRemoveButton );
   connect( mRemoveButton, SIGNAL( clicked() ), SLOT( removeAttendee() ) );
 
   mAddressBookButton = new QPushButton( i18n("Select Addressee..."),
                                         buttonBox );
+  QWhatsThis::add( mAddressBookButton,
+		   i18n("Opens your address book, allowing you to select "
+			"new attendees from it.") );
   buttonLayout->addWidget( mAddressBookButton );
   connect( mAddressBookButton, SIGNAL( clicked() ), SLOT( openAddressBook() ) );
 
@@ -292,6 +344,11 @@ KOEditorDetails::~KOEditorDetails()
 {
 }
 
+bool KOEditorDetails::hasAttendees()
+{
+  return mListView->childCount() > 0;
+}
+
 bool KOEditorDetails::eventFilter( QObject *watched, QEvent *ev)
 {
   if ( watched && watched == mNameEdit && ev->type() == QEvent::FocusIn &&
@@ -332,12 +389,12 @@ void KOEditorDetails::openAddressBook()
           itr != aList.end(); ++itr ) {
       KABC::Addressee a = (*itr);
       bool myself = KOPrefs::instance()->thatIsMe( a.preferredEmail() );
-      bool sameAsOrganizer = mOrganizerCombo &&
+      bool sameAsOrganizer = mOrganizerCombo && 
         KPIM::compareEmail( a.preferredEmail(), mOrganizerCombo->currentText(), false );
       KCal::Attendee::PartStat partStat;
-      if ( myself && sameAsOrganizer )
+      if ( myself && sameAsOrganizer ) 
         partStat = KCal::Attendee::Accepted;
-      else
+      else 
         partStat = KCal::Attendee::NeedsAction;
       insertAttendee( new Attendee( a.realName(), a.preferredEmail(),
                                     !myself, partStat,
@@ -368,7 +425,7 @@ void KOEditorDetails::openAddressBook()
 void KOEditorDetails::addNewAttendee()
 {
   Attendee *a = new Attendee( i18n("Firstname Lastname"),
-                              i18n("name@domain.com"), true );
+                              i18n("name") + "@example.net", true );
   insertAttendee( a, false );
   // We don't want the hint again
   mNameEdit->setClickMessage( "" );
@@ -422,17 +479,19 @@ void KOEditorDetails::readEvent( Incidence *event )
     }
     mOrganizerLabel->setText( i18n( "Identity as organizer:" ) );
 
-    // This might not be enough, if the combo as a full name too, hence the loop below
-    // mOrganizerCombo->setCurrentText( event->organizer().fullName() );
+    int found = -1;
+    QString fullOrganizer = event->organizer().fullName();
     for ( int i = 0 ; i < mOrganizerCombo->count(); ++i ) {
-      QString itemTxt = KPIM::getEmailAddr( mOrganizerCombo->text( i ) );
-      if ( KPIM::compareEmail( event->organizer().email(), itemTxt, false ) ) {
-        // Make sure we match the organizer setting completely
-        mOrganizerCombo->changeItem( event->organizer().fullName(), i );
+      if ( mOrganizerCombo->text( i ) == fullOrganizer ) {
+        found = i;
         mOrganizerCombo->setCurrentItem( i );
         break;
       }
     }
+    if ( found < 0 ) {
+      mOrganizerCombo->insertItem( fullOrganizer, 0 );
+      mOrganizerCombo->setCurrentItem( 0 );
+    }
   } else { // someone else is the organizer
     if ( mOrganizerCombo ) {
       delete mOrganizerCombo;
@@ -456,7 +515,6 @@ void KOEditorDetails::writeEvent(Incidence *event)
     event->addAttendee(new Attendee(*(a->data())));
   }
   if ( mOrganizerCombo ) {
-    // TODO: Don't take a string and split it up... Is there a better way?
     event->setOrganizer( mOrganizerCombo->currentText() );
   }
 }
@@ -478,7 +536,6 @@ bool KOEditorDetails::validateInput()
 
 void KOEditorDetails::updateAttendeeInput()
 {
-
   setEnableAttendeeInput(!mNameEdit->text().isEmpty());
   QListViewItem *item = mListView->selectedItem();
   AttendeeListItem *aItem = static_cast<AttendeeListItem *>( item );
@@ -492,7 +549,7 @@ void KOEditorDetails::updateAttendeeInput()
 void KOEditorDetails::clearAttendeeInput()
 {
   mNameEdit->setText("");
-  mUidEdit->setText("");
+  mUid = QString::null;
   mRoleCombo->setCurrentItem(0);
   mStatusCombo->setCurrentItem(0);
   mRsvpButton->setChecked(true);
@@ -505,15 +562,11 @@ void KOEditorDetails::fillAttendeeInput( AttendeeListItem *aItem )
   mDisableItemUpdate = true;
   QString name = a->name();
   if (!a->email().isEmpty()) {
-    // Taken from KABC::Addressee::fullEmail
-    QRegExp needQuotes( "[^ 0-9A-Za-z\\x0080-\\xFFFF]" );
-    if ( name.find( needQuotes ) != -1 )
-      name = "\"" + name + "\" <" + a->email() + ">";
-    else
-      name += " <" + a->email() + ">";
+    name = KPIM::quoteNameIfNecessary( name );
+    name += " <" + a->email() + ">";
   }
   mNameEdit->setText(name);
-  mUidEdit->setText(a->uid());
+  mUid = a->uid();
   mRoleCombo->setCurrentItem(a->role());
   mStatusCombo->setCurrentItem(a->status());
   mRsvpButton->setChecked(a->RSVP());
@@ -542,6 +595,7 @@ void KOEditorDetails::updateAttendeeItem()
   if ( !aItem ) return;
 
   Attendee *a = aItem->data();
+
   QString name;
   QString email;
   KPIM::getNameAndMail(mNameEdit->text(), name, email);
@@ -551,7 +605,7 @@ void KOEditorDetails::updateAttendeeItem()
   if ( iAmTheOrganizer ) {
     bool myself =
       KPIM::compareEmail( email, mOrganizerCombo->currentText(), false );
-    bool wasMyself =
+    bool wasMyself = 
       KPIM::compareEmail( a->email(), mOrganizerCombo->currentText(), false );
     if ( myself ) {
       mStatusCombo->setCurrentItem( KCal::Attendee::Accepted );
@@ -565,7 +619,7 @@ void KOEditorDetails::updateAttendeeItem()
     }
   }
   a->setName( name );
-  a->setUid( mUidEdit->text() );
+  a->setUid( mUid );
   a->setEmail( email );
   a->setRole( Attendee::Role( mRoleCombo->currentItem() ) );
   a->setStatus( Attendee::PartStat( mStatusCombo->currentItem() ) );
