@@ -1,6 +1,7 @@
 /*
     This file is part of KOrganizer.
     Copyright (c) 2000,2001 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -40,12 +41,12 @@ class Attendee;
 class Incidence;
 }
 using namespace KCal;
-
+    
 namespace KPIM {
 class AddresseeLineEdit;
 }
 
-typedef CustomListViewItem<Attendee *> AttendeeListItem;
+typedef CustomListViewItem<KCal::Attendee *> AttendeeListItem;
 
 
 /** KOAttendeeListView is a child class of KListView  which supports
@@ -93,6 +94,9 @@ class KOEditorDetails : public QWidget
     /** Set the gantt view */
     void setFreeBusyWidget( KOEditorFreeBusy * );
 
+    /** Returns whether at least one attendee was added */
+    bool hasAttendees();
+
   public slots:
     void insertAttendee(Attendee *);
 
@@ -116,7 +120,7 @@ class KOEditorDetails : public QWidget
     bool mDisableItemUpdate;
 
     KPIM::AddresseeLineEdit *mNameEdit;
-    QLineEdit *mUidEdit;
+    QString mUid;
     KListView *mListView;
     QComboBox* mRoleCombo;
     QCheckBox* mRsvpButton;
