@@ -291,11 +291,11 @@ void ResourceView::updateView()
   for( it = manager->begin(); it != manager->end(); ++it ) {
     addResourceItem( *it );
   }
-  manager->writeConfig();
 }
 
 void ResourceView::emitResourcesChanged()
 {
+  mCalendar->resourceManager()->writeConfig();
   emit resourcesChanged();
 }
 
@@ -324,10 +324,10 @@ void ResourceView::addResource()
 
   resource->setResourceName( i18n("%1 resource").arg( type ) );
 
-  KRES::ConfigDialog dlg( this, QString("calendar"), resource,
+  KRES::ConfigDialog *dlg = new KRES::ConfigDialog( this, QString("calendar"), resource,
                           "KRES::ConfigDialog" );
 
-  if ( dlg.exec() ) {
+  if ( dlg && dlg->exec() ) {
     resource->setTimeZoneId( KOPrefs::instance()->mTimeZoneId );
     manager->add( resource );
     // we have to call resourceAdded manually, because for in-process changes
@@ -338,7 +338,8 @@ void ResourceView::addResource()
     delete resource;
     resource = 0;
   }
-  emit resourcesChanged();
+  if ( dlg ) delete dlg;
+  emitResourcesChanged();
 }
 
 void ResourceView::addResourceItem( ResourceCalendar *resource )
@@ -384,7 +385,7 @@ void ResourceView::slotSubresourceAdded( ResourceCalendar *calendar,
 
   ResourceItem *item = static_cast<ResourceItem *>( i );
   ( void )new ResourceItem( calendar, resource, label, this, item );
-  emit resourcesChanged();
+  emitResourcesChanged();
 }
 
 // Remove an entry
@@ -451,7 +452,7 @@ void ResourceView::removeResource()
 void ResourceView::editResource()
 {
   ResourceItem *item = currentItem();
-
+  if (!item) return;
   ResourceCalendar *resource = item->resource();
 
   KRES::ConfigDialog dlg( this, QString("calendar"), resource,
@@ -462,7 +463,7 @@ void ResourceView::editResource()
 
     mCalendar->resourceManager()->change( resource );
   }
-  emit resourcesChanged();
+  emitResourcesChanged();
 }
 
 void ResourceView::currentChanged( QListViewItem *item )
@@ -475,7 +476,6 @@ void ResourceView::currentChanged( QListViewItem *item )
      mDeleteButton->setEnabled( true );
      mEditButton->setEnabled( true );
    }
-   mCalendar->resourceManager()->writeConfig();
 }
 
 ResourceItem *ResourceView::findItem( ResourceCalendar *r )
@@ -565,7 +565,7 @@ void ResourceView::assignColor()
     KOPrefs::instance()->setResourceColor( identifier, myColor );
     item->setResourceColor( myColor );
     item->update();
-    emit resourcesChanged();
+    emitResourcesChanged();
   }
 }
 
@@ -582,7 +582,7 @@ void ResourceView::disableColor()
   KOPrefs::instance()->setResourceColor( identifier, colorInvalid );
   item->setResourceColor( colorInvalid );
   item->update();
-  emit resourcesChanged();
+  emitResourcesChanged();
 }
 void ResourceView::showInfo()
 {
